// file: app_event_source.js
import {config} from "./config";
import {parseBoard, parseHiddenByOpEvent, parsePost} from "./parsers";
import * as repo from "./repository";
import {addIsEventSourceListener, isCurrentTabEventSource} from "./services/alive_tabs_service";

export class AppEventSource {
    constructor(url, configuration) {
        this._url = url;
        this._configuration = configuration;

        this._sseListeners = [];
        this._onErrorEventListeners = [];

        this._sseChannel = new BroadcastChannel(config.sse.broadcastChannel);

        this._isEventSourceTab = isCurrentTabEventSource();

        if (this._isEventSourceTab) {
            console.log("Current tab must connect!"); // todo: delete
            this._connect();
        } else {
            console.log("Do not connect. Listen to other tab!"); // todo: delete
        }

        addIsEventSourceListener(isEventSourceTab => {
            this._isEventSourceTab = isEventSourceTab;

            if (this._isEventSourceTab) {
                console.log("Current tab must connect!"); // todo: delete
                this._connect();
            } else {
                console.log("Disconnect. listen to other tab"); // todo: delete
                this._closeEventSource();
            }
        });

        this.addConnectedEventListener((e, data) => this._onConnected(e, data));

        this.addConnectedEventListener(e => {
            if (!this._isEventSourceTab) return;
            this._sseChannel.postMessage({type: e.type, data: e.data});
        });

        this.addPostsListener(e => {
            if (!this._isEventSourceTab) return;
            this._sseChannel.postMessage({type: e.type, data: e.data});
        });

        this.addBoardDescriptionListener(e => {
            if (!this._isEventSourceTab) return;
            this._sseChannel.postMessage({type: e.type, data: e.data});
        });

        this.addHiddenByOpListener(e => {
            if (!this._isEventSourceTab) return;
            this._sseChannel.postMessage({type: e.type, data: e.data});
        });

        this.addErrorEventListener(e => {
            if (!this._isEventSourceTab) return;
            this._sseChannel.postMessage({type: e.type, readyState: this._eventSource.readyState});
        });

        // Listen to other tab.
        this._sseChannel.addEventListener('message', e => this._sseChannelListener(e));

        addEventListener('beforeunload', e => this._close());
    }

    addErrorEventListener(listener) {
        this._onErrorEventListeners.push(listener);
    }

    addPostsListener(listener) {
        this._addJsonEventListener(config.sse.type.post, (e, data) => listener(e, parsePost(data)));
    }

    addBoardDescriptionListener(listener) {
        this._addJsonEventListener(
            config.sse.type.boardDescription,
            (e, data) => listener(e, parseBoard(data)),
        );
    }

    addHiddenByOpListener(listener) {
        this._addJsonEventListener(
            config.sse.type.hiddenByOp,
            (e, data) => listener(e, parseHiddenByOpEvent(data))
        );
    }

    addConnectedEventListener(listener) {
        if (!this._isEventSourceTab) listener({type: config.sse.type.connected}, {});

        this._addJsonEventListener(config.sse.type.connected, listener);
    }

    _close() {
        this._sseChannel.close();
        this._closeEventSource();
    }

    _closeEventSource() {
        if (this._eventSource) this._eventSource.close();
    }

    _connect() {
        this._eventSource = new EventSource(this._url, this._configuration);
        this._eventSource.onerror = e => this._onError(e);

        this._sseListeners.forEach(
            sseListener => this._restoreEventListener(sseListener.type, sseListener.listener)
        );
    }

    _onConnected(e, data) {
        if (!this._isEventSourceTab) return;

        this._subscriptionId = data.subscriptionId;
        this._keepSubscriptionAlive();
        console.log(config.sse.type.connected, this._subscriptionId);
    }

    _keepSubscriptionAlive() {
        if (!this._isEventSourceTab) return;

        clearTimeout(this._keepAliveTimeoutId);

        if (this._eventSource.readyState !== EventSource.OPEN) return;

        console.log('keep alive', this._subscriptionId);

        this._keepAliveTimeoutId = setTimeout(() => {
            repo.continueSubscription(this._subscriptionId).then(
                () => this._keepSubscriptionAlive()
            );
        }, config.sse.continueSubscriptionTimeout);
    }

    _addJsonEventListener(type, listener) {
        this._addEventListener(type, e => listener(e, JSON.parse(e.data)));
    }

    _addEventListener(type, listener) {
        this._sseListeners.push(new SseListener(type, listener));
        if (this._eventSource) this._eventSource.addEventListener(type, listener);
    }

    _restoreEventListener(type, listener) {
        this._eventSource.addEventListener(type, listener);
    }

    _onError(e) {
        console.log('Event Source failed, readyState = ', this._eventSource.readyState);

        this._onErrorEventListeners.forEach(
            listener => listener(e, this._eventSource.readyState)
        );

        if (this._eventSource.readyState !== EventSource.CLOSED) return;

        // firefox won't try to reconnect after CLOSED state, so we have to do it ourselves
        clearTimeout(this._reconnectTimeoutId);
        this._reconnectTimeoutId = setTimeout(() => this._connect(), config.sse.reconnectTimeout);
    }

    _sseChannelListener(e) {
        if (this._isEventSourceTab) return;

        const sse = e.data;
        this._sseListeners.filter(sseListener => sseListener.type === sse.type)
            .forEach(sseListener => sseListener.listener(sse, sse.data));

        if (sse.type === 'error') {
            this._onErrorEventListeners.forEach(listener => {
                listener(sse, sse.readyState);
            });
        }
    }
}

class SseListener {
    constructor(type, listener) {
        this.type = type;
        this.listener = listener;
    }
}