// file: repository.js
import * as httpService from "./services/http_service";
import * as pcService from "./services/post_cache_service";
import {AppEventSource} from "./app_event_source";
import {getOrCreateClientId} from "./services/client_service";
import {config} from "./config";
import {
    addOrRemoveFavoriteBoard, addOrRemoveFavoriteThread,
    getFavoriteBoards, getFavoriteThreads,
} from "./services/favorites_service";
import * as trackNewPostsService from "./services/track_new_posts_service";
import {getSubscribeApiUrl} from "./services/url_service";
import {SuccessResponse} from "./models";

export function getRecentPosts(threadId) {
    return httpService.getRecentPosts(threadId).then(r => r.data).then(pcService.saveForRecentPosts);
}

export function getPostsBefore(postId, threadId) {
    return httpService.getPostsBefore(postId, threadId).then(r => r.data).then(pcService.saveMany);
}

export function getPostsAfter(postId, threadId) {
    return httpService.getPostsAfter(postId, threadId).then(r => r.data).then(pcService.saveMany);
}

export function getLogsPage(pageNumber) {
    return httpService.getLogsPage(pageNumber).then(r => r.data).then(pcService.saveMany);
}

export function getPost(postId, includeIndex = false) {
    const cachedPost = pcService.get(postId);
    if (cachedPost && !includeIndex) return Promise.resolve(cachedPost);

    return httpService.getPost(postId, includeIndex).then(r => r.data).then(pcService.save);
}

export function getBoards(isFavorite, skip = 0) {
    if (!config.isClientWebUI) {
        const favoriteBoards = getFavoriteBoards();

        if (isFavorite) {
            // return favorite boards from local storage
            return Promise.resolve(favoriteBoards.skip(skip).take(config.pageSize.general));
        }

        const favoriteBoardsNames = favoriteBoards.map(b => b.name);

        return httpService.getBoards(null, skip).then(r => {
            const boards = r.data;
            // todo: here's a potential bug if a user added all boards as favorites for this page
            return boards.filter(b => !favoriteBoardsNames.includes(b.name));
        });
    }

    return httpService.getBoards(isFavorite, skip).then(r => r.data);
}

export function getThreads(isFavorite, boardName = null, skip = 0) {
    if (!config.isClientWebUI) {
        const favoriteThreads = getFavoriteThreads()
            .filter(t => !boardName || t.board.name === boardName);

        if (isFavorite) {
            // return favorite threads from local storage
            return Promise.resolve(favoriteThreads.skip(skip).take(config.pageSize.general));
        }

        const favoriteThreadsIds = favoriteThreads.map(t => t.threadId);

        return httpService.getThreads(null, boardName, skip).then(r => {
            const threads = r.data;
            // todo: here's a potential bug if a user added all threads as favorites for this page
            return threads.filter(t => !favoriteThreadsIds.includes(t.threadId));
        });
    }

    return httpService.getThreads(isFavorite, boardName, skip).then(r => r.data);
}

export function getThreadPage(threadId, pageNumber = null) {
    // null means the last page

    return httpService.getThreadPage(threadId, pageNumber).then(r => {
        r.data.op.thread = r.data.thread;

        r.data.posts = r.data.posts.map(p => {
            p.thread = r.data.thread;
            return p;
        });
        pcService.saveForThreadSlice(r.data);
        return r;
    });
}

export function getWarpPage(boardName = null, pageNumber = 0) {
    return httpService.getWarpPage(boardName, pageNumber).then(r => {
        r.data.shellThreads = r.data.shellThreads.map(st => {
            st.thread.board = st.thread.board || r.data.board;
            st.op.thread = st.thread;

            st.posts = st.posts.map(p => {
                p.thread = st.thread;
                return p;
            });
            return st;
        });
        pcService.saveForWarpSlice(r.data);
        return r;
    });
}

export function getServerInfo() {
    return httpService.getServerInfo().then(r => r.data);
}

export function getSearch(searchArgs) {
    const defaultValue = new SuccessResponse([], 0, 0, 0);

    if (!searchArgs) {
        return Promise.resolve(defaultValue);
    }

    return httpService.getSearch(searchArgs).then(r => config.supportSearching ? r : defaultValue);
}

export function createUser(newUser) {
    return httpService.createUser(newUser).then(r => r.data);
}

export function updateBoardDescription(board) {
    return httpService.updateBoardDescription(board).then(r => r.data);
}

export function updateHiddenByOp(updatedHiddenByOp) {
    return getOrCreateClientId().then(clientId => {
        updatedHiddenByOp.clientId = clientId;

        return httpService.updateHiddenByOp(updatedHiddenByOp).then(r => r.data);
    });
}

export function publish(newPost) {
    return getOrCreateClientId().then(clientId => {
        newPost.clientId = clientId;

        return httpService.publish(newPost).then(r => r.data);
    });
}

export function updateBoardIsFavorite(board) {
    if (!config.isClientWebUI) return Promise.resolve(addOrRemoveFavoriteBoard(board));

    return httpService.updateBoardIsFavorite(board.name, board.isFavorite);
}

export function updateThreadIsFavorite(thread) {
    if (!config.isClientWebUI) return Promise.resolve(addOrRemoveFavoriteThread(thread));

    return httpService.updateThreadIsFavorite(thread.threadId, thread.isFavorite);
}

export function updateLastSeenAt(threadId, lastSeenAt) {
    if (!config.isClientWebUI) return trackNewPostsService.updateLastSeenAt(threadId, lastSeenAt);

    return httpService.updateLastSeenAt(threadId, lastSeenAt);
}

export function deleteAllLastSeenAt() {
    if (!config.isClientWebUI) return trackNewPostsService.deleteAllLastSeenAt();

    return httpService.resetAllLastSeenAt();
}

export function resetAllLastSeenAt() {
    if (!config.isClientWebUI) return trackNewPostsService.deleteAllLastSeenAt();

    return httpService.deleteAllLastSeenAt();
}

export function authorize(auth) {
    return httpService.authorize(auth).then(r => r.data)
}


let eventSource;

export function subscribe() {
    eventSource = new AppEventSource(getSubscribeApiUrl());
}

export function addErrorEventListener(listener) {
    return eventSource.addErrorEventListener(listener);
}

export function addPostsListener(listener) {
    return eventSource.addPostsListener((e, post) => listener(e, pcService.save(post)));
}

export function addBoardDescriptionListener(listener) {
    return eventSource.addBoardDescriptionListener(listener);
}

export function addHiddenByOpListener(listener) {
    return eventSource.addHiddenByOpListener(listener);
}

export function addConnectedEventListener(listener) {
    return eventSource.addConnectedEventListener(listener);
}

export function continueSubscription(subscriptionId) {
    return httpService.continueSubscription(subscriptionId);
}

export function getClientId() {
    return httpService.getClientId().then(r => r.data);
}

export function updateClientId() {
    return httpService.updateClientId().then(r => r.data);
}
