// file: http_service.js
import {
    parseBoard, parseRecentPosts, parseThread,
    parseErrorResponse,
    parsePost, parseSuccessResponse, parseThreadSlice, parseWarpSlice, parseServerInfo, parseAuthInfo,
} from "../parsers";
import {
    getDeleteAllLastSeenAtApiUrl, getResetAllLastSeenAtApiUrl,
    getSearchApiUrl, getUpdateBoardIsFavoriteApiUrl, getUpdateLastSeenAtApiUrl,
    getUpdateThreadIsFavorite,
    getBoardsApiUrl, getThreadsApiUrl, getClientIdApiUrl,
    getContinueSubscriptionApiUrl, getLogsPageApiUrl, getPostApiUrl, getPostsAfterApiUrl,
    getPostsBeforeApiUrl, getPublishApiUrl, getRecentPostsApiUrl,
    getThreadPageApiUrl, getUpdateBoardDescriptionApiUrl, getWarpPageApiUrl, getUpdateHiddenByOpApiUrl,
    getServerInfoApiUrl, getAuthorizeApiUrl, getCreateUserApiUrl,
} from "./url_service";
import {RecentPosts, RequestBody} from "../models";
import {config} from "../config";
import {getAccessToken} from "./access_token_service";

export function getRecentPosts(threadId) {
    return getJson(getRecentPostsApiUrl(threadId)).thenA(obj => {
        return config.isClientWebUI
            ? parseRecentPosts(obj)
            : new RecentPosts(null, obj.map(parsePost).reverse());
    });
}

export function getPostsBefore(postId, threadId) {
    return getJson(getPostsBeforeApiUrl(postId, threadId)).thenA(ps => ps.map(parsePost).reverse());
}

export function getPostsAfter(postId, threadId) {
    return getJson(getPostsAfterApiUrl(postId, threadId)).thenA(ps => ps.map(parsePost));
}

export function getBoards(isFavorite, skip) {
    return getJson(getBoardsApiUrl(isFavorite, skip)).thenA(bs => bs.map(parseBoard));
}

export function getThreads(isFavorite, boardName, skip) {
    return getJson(getThreadsApiUrl(isFavorite, boardName, skip)).thenA(
        ts => ts.map(parseThread)
    );
}

export function getLogsPage(pageNumber) {
    return getJson(getLogsPageApiUrl(pageNumber)).thenA(ps => ps.map(parsePost));
}

export function getPost(postId, includeIndex) {
    return getJson(getPostApiUrl(postId, includeIndex)).thenA(parsePost);
}

export function getThreadPage(threadId, pageNumber) {
    // null means the last page

    return getJson(getThreadPageApiUrl(threadId, pageNumber)).thenA(parseThreadSlice);
}

export function getWarpPage(boardName, pageNumber) {
    return getJson(getWarpPageApiUrl(boardName, pageNumber)).thenA(parseWarpSlice);
}

export function getServerInfo() {
    return getJson(getServerInfoApiUrl()).thenA(parseServerInfo);
}

export function getSearch(searchArgs) {
    return getJson(getSearchApiUrl(searchArgs)).thenA(ps => ps.map(parsePost));
}

export function updateBoardDescription(board) {
    return patchJson(getUpdateBoardDescriptionApiUrl(), new RequestBody(board)).thenA(message => message);
}

export function updateHiddenByOp(updatedHiddenByOp) {
    return patchJson(getUpdateHiddenByOpApiUrl(), new RequestBody(updatedHiddenByOp)).thenA(message => message);
}

export function publish(newPost) {
    return postJson(getPublishApiUrl(), new RequestBody(newPost)).thenA(p => parsePost(p));
}

export function authorize(auth) {
    return postJson(getAuthorizeApiUrl(), new RequestBody(auth)).thenA(parseAuthInfo);
}

export function createUser(newUser) {
    return postJson(getCreateUserApiUrl(), new RequestBody(newUser)).thenA(message => message);
}

export function continueSubscription(subscriptionId) {
    return postJson(getContinueSubscriptionApiUrl(subscriptionId)).thenA(message => message);
}

export function updateBoardIsFavorite(boardName, isFavorite) {
    return (isFavorite ? postJson : deleteJson)(getUpdateBoardIsFavoriteApiUrl(boardName));
}

export function updateThreadIsFavorite(threadId, isFavorite) {
    return (isFavorite ? postJson : deleteJson)(getUpdateThreadIsFavorite(threadId));
}

export function updateLastSeenAt(threadId, lastSeenAt) {
    return patchJson(getUpdateLastSeenAtApiUrl(threadId), new RequestBody({lastSeenAt}));
}

export function resetAllLastSeenAt() {
    return deleteJson(getResetAllLastSeenAtApiUrl());
}

export function deleteAllLastSeenAt() {
    return patchJson(getDeleteAllLastSeenAtApiUrl());
}

export function getClientId() {
    return getJson(getClientIdApiUrl());
}

export function updateClientId() {
    return patchJson(getClientIdApiUrl());
}


function getJson(url) {
    return handleFetchPromise(fetch(url));
}

function postJson(url, data) {
    return httpRequestJson(url, 'POST', data);
}

function patchJson(url, data) {
    return httpRequestJson(url, 'PATCH', data);
}

function deleteJson(url, data) {
    return httpRequestJson(url, 'DELETE', data);
}

function httpRequestJson(url, method, data) {
    let headers = {
        'Accept': 'application/json',
        'Content-Type': 'application/json',
    };
    if (['POST', 'PATCH', 'DELETE'].includes(method)) {
        const accessToken = getAccessToken();
        if (accessToken) headers['Authorization'] = accessToken;
    }
    return handleFetchPromise(fetch(url, {
        method: method,
        headers,
        body: JSON.stringify(data)
    }));
}

function handleFetchPromise(fetchPromise) {
    return fetchPromise.then(response =>
        Promise.all([Promise.resolve(response), response.json()])
    ).then(([response, json]) => {
        if (response.ok) return json;

        throw parseErrorResponse(json);
    }).then(
        json => parseSuccessResponse(json)
    );
}
