// file: location_service.js
import {
    getHomeUrl,
    getSearchBaseUrl, getSearchUrl, getThreadUrl, getWarpUrl, parseSearchArgs
} from "./url_service";
import * as customEvents from "../custom_events";
import {config} from "../config";
import {parsePostReference, parseUuid} from "./text_parsing_service";

export function getHref() {
    return location.href;
}

function getPathName() {
    return location.pathname.substring(config.basePath.length);
}

function getHrefWithoutHost() {
    return getPathName() + location.search + location.hash;
}

function getPathParts() {
    return getPathName().split('/');
}

export function showAllThreads() {
    return ("allt" in getQueryStrings()) || !getSelectedBoardName();
}

export function isSearchPage() {
    return location.pathname.startsWith(getSearchBaseUrl());
}

export function getSearchArgs() {
    return parseSearchArgs(getQueryStrings());
}

export function getSelectedBoardName() {
    const name = getPathParts()[1].toLowerCase();
    return /^[a-z]{1,3}$/.test(name) ? name : null;
}

export function getSelectedThreadId() {
    if (!getSelectedBoardName()) return null;

    return getThreadId();
}

export function getThreadId() {
    const threadId = parseUuid(getPathParts()[2]);
    return threadId ? threadId.match : null;
}

export function getPage() {
    const pageNumber = parseInt(getQueryStrings().p);
    return isNaN(pageNumber) ? null : pageNumber;
}

export function getPostId() {
    const match = /(@.+)/.exec(getPathName());
    if (!match) return null;
    return parsePostReference(match[1]).match;
}

export function getPostIdAnchor() {
    const postId = parseUuid(location.hash.slice(2));
    return postId ? postId.match : null;
}

export function activateTarget() {
    if (location.hash) {
        // noinspection SillyAssignmentJS
        location.hash = location.hash;
    }
}

export function setPostIdHash(hash) {
    pushEmptyState(location.pathname + location.search + hash);

    customEvents.setPostIdHash(hash);
    activateTarget();
}

export function setHome() {
    pushEmptyState(getHomeUrl());
    customEvents.selectGlobalPage(0);
}

export function setBoard(boardName) {
    setWarp(boardName, 0);
}

export function setWarp(boardName, pageNumber) {
    pushEmptyState(getWarpUrl(boardName, pageNumber));

    customEvents.selectBoard(boardName, pageNumber);
}

export function setThread(boardName, threadId) {
    const queryString = showAllThreads() ? `?allt` : ``;
    const url = getThreadUrl(boardName, threadId) + queryString;

    pushEmptyState(url);

    customEvents.selectThread(boardName, threadId, null);
}

export function setThreadPage(boardName, threadId, pageNumber, postId = null) {
    const url = (
        getThreadUrl(boardName, threadId, pageNumber) +
        (showAllThreads() ? `${pageNumber !== null ? '&' : '?'}allt` : ``) +
        (postId === null ? `` : `#p${postId}`)
    );

    pushEmptyState(url);

    customEvents.selectThread(boardName, threadId, pageNumber, postId);
}

export function setSearch(searchArgs) {
    pushEmptyState(getSearchUrl(searchArgs));
    customEvents.selectSearch(searchArgs);
}

export function replaceByThreadPage(boardName, threadId, pageNumber, postId) {
    const url = getThreadUrl(boardName, threadId, pageNumber, postId);
    replaceEmptyState(url);
}

function pushEmptyState(url) {
    // do not push the same state twice
    if (url === getHrefWithoutHost()) return;

    history.pushState({}, ``, url);
}

function replaceEmptyState(url) {
    history.replaceState({}, ``, url);
}

function getQueryStrings() {
    let result = {};
    location.search.substring(1).split('&').forEach(x => {
        const idx = x.indexOf('=');
        if (idx > 0) {
            result[decodeURIComponent(x.substring(0, idx))]
                = decodeURIComponent(x.substring(idx + 1));
        } else {
            result[decodeURIComponent(x)] = null;
        }
    });
    return result;
}
