// file: text_parsing_service.js
import {toHyphenatedUuid} from "./uuid_service";

export function parsePostReference(text) {
    if (!text.startsWith('@')) return null;

    let uuid = parseUuid(text.slice(1));
    if (!uuid) return null;
    uuid.match = toHyphenatedUuid(uuid.match);
    return uuid;
}

export function parseUuid(text) {
    const simpleUuid = parseSimpleUuid(text);
    if (simpleUuid) return simpleUuid;

    const hyphenatedUuid = parseHyphenatedUuid(text);
    if (hyphenatedUuid) return hyphenatedUuid;

    return null;
}

export function parseSimpleUuid(text) {
    const simpleUuidRe = /^[\da-f]{8}[\da-f]{4}4[\da-f]{3}[89ab][\da-f]{3}[\da-f]{12}/i;
    return parseByRe(simpleUuidRe, text);
}

export function parseHyphenatedUuid(text) {
    const hyphenatedUuidRe = /^[\da-f]{8}-[\da-f]{4}-4[\da-f]{3}-[89ab][\da-f]{3}-[\da-f]{12}/i;
    return parseByRe(hyphenatedUuidRe, text);
}

const shortcutUuidRe = /^[\da-f]{5}$/;

export function parseByRe(re, text) {
    const match = re.exec(text);
    if (!match) return null;
    return { match: match[0], rest: text.slice(match[0].length)}
}


export function parserNewline(text) {
    if (text[0] === '\n') {
        return { rest: text.slice(1) };
    }
    return null;
}

export function parseQuote(text, markupFreePrefix, element, level) {
    if (level !== 0 || text[0] !== '>') return null;
    // quit if the prefix contains non-whitespace chars
    if (/\S/.test(markupFreePrefix)) return null;

    const prevE = element.lastChild;
    const afterPostRef = prevE && prevE.className === 'post_ref';
    if (prevE && prevE.tagName !== 'BR' && !afterPostRef) {
        return;
    }
    const idx = text.indexOf('\n', 1);
    return {
        match: idx < 0 ? text : text.slice(0, idx),
        rest: idx < 0 ? `` : text.slice(idx),
        afterPostRef: afterPostRef
    };
}

export function parseNamedLink(text) {
    const match = /^\[([^\n\r\]]+)\]\(([^\)]+)\)/.exec(text);
    if (!match) return null;
    const name = match[1];
    // const urlMatch = urlRe.exec(match[2]);
    // if (!urlMatch) return null;
    // const url = urlMatch[0];
    const url = match[2];
    const lengthToSkip = match[0].length;
    return { match: url, name: name, rest: text.slice(lengthToSkip) };
}

export function parseLink(text) {
    const match = urlRe.exec(text);
    if (!match) return null;
    const url = match[0];
    return { match: url, rest: text.slice(url.length) };
}

export function parseBold(text) { return parseSimpleMarkup("**", text); }
export function parseItalic(text) { return parseSimpleMarkup("//", text); }
export function parseMono(text) { return parseSimpleMarkup("`", text); }
export function parseStrike(text) { return parseSimpleMarkup("~~", text); }
export function parseTeX(text) { return parseSimpleMarkup("$", text); }
export function parseSpoiler(text) { return parseSimpleMarkup("%%", text); }
export function parseCode(text) {
    const code = parseSimpleMarkup("```", text);
    if (!code) return code;
    const langMatch = /^[^\s]*/.exec(code.match);
    if (!langMatch) return code;
    code.lang = langMatch[0];
    code.match = code.match.slice(code.lang.length).replace(/^\s/, "");
    return code;
}

export function parseSimpleMarkup(markup, text) {
    if (!text.startsWith(markup)) return null;
    const idx = text.indexOf(markup, markup.length);
    if (idx < 0) return null;
    const match = text.substring(markup.length, idx);
    return { match: match, rest: text.slice(idx + markup.length) };
}

const urlRe = /^(?:(?:(?:https?|ftp):\/\/)(?:\S+(?::\S*)?@)?(?:(?!10(?:\.\d{1,3}){3})(?!127(?:\.\d{1,3}){3})(?!169\.254(?:\.\d{1,3}){2})(?!192\.168(?:\.\d{1,3}){2})(?!172\.(?:1[6-9]|2\d|3[0-1])(?:\.\d{1,3}){2})(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))|(?:(?:[a-z\u00a1-\uffff0-9]+-?)*[a-z\u00a1-\uffff0-9]+)(?:\.(?:[a-z\u00a1-\uffff0-9]+-?)*[a-z\u00a1-\uffff0-9]+)*(?:\.(?:[a-z\u00a1-\uffff0-9]{2,})))(?::\d{2,5})?(?:\/[^\s]*)?|magnet:\?(?:[^\s\]\)]+))/;

export function getPostReferences(text) {
    const ids = [];

    for (let i = 0; i < text.length; i++) {
        const textToParse = text.slice(i);

        const postReference = parsePostReference(textToParse);
        if (postReference) {
            ids.push(postReference.match);
        }
    }
    return ids;
}
