// file: url_service.js
import {SearchArgs} from "../models";
import {config} from "../config";

const api = config.isClientWebUI ? `c_api` : `s_api`;
const prefix = config.basePath;

export function getRecentPostsApiUrl(threadId) {
    return `/${api}/posts?${toQuery({
        includeThread: true,
        includeBoard: true,
        sortByDate: "desc",
        thread: threadId,
    })}`;
}

export function getPostsBeforeApiUrl(postId, threadId) {
    return `/${api}/posts?${toQuery({
        includeThread: true,
        includeBoard: true,
        sortByDate: "desc",
        thread: threadId,
        beforePost: postId,
    })}`;
}

export function getPostsAfterApiUrl(postId, threadId) {
    return `/${api}/posts?${toQuery({
        includeThread: true,
        includeBoard: true,
        sortByDate: "asc",
        thread: threadId,
        afterPost: postId,
    })}`;
}

export function getLogsPageApiUrl(pageNumber) {
    return `/${api}/posts?${toQuery({
        includeThread: true,
        includeBoard: true,
        includeTotal: true,
        sortByDate: "desc",
        skip: pageNumber * config.pageSize.general,
    })}`
}

export function getPostApiUrl(postId, includeIndex) {
    return `/${api}/posts/${postId}?${toQuery({
        includeThread: true,
        includeBoard: true,
        includeIndex,
    })}`;
}

export function getBoardsApiUrl(isFavorite, skip) {
    return `/${api}/boards?${toQuery({
        skip,
        isFavorite,
    })}`;
}

export function getThreadsApiUrl(isFavorite, boardName, skip) {
    return `/${api}/threads?${toQuery({
        board: boardName,
        includeBoard: true,
        skip,
        isFavorite,
    })}`;
}

export function getBoardUrl(boardName) {
    return `${prefix}/${boardName}/`;
}

export function getThreadUrl(boardName, threadId, pageNumber = null, postId = null) {
    const path = `${prefix}/${boardName}/${threadId}/`;

    const pathAndQuery = pageNumber === null ? path : `${path}?${toQuery({p: pageNumber})}`;

    return postId === null ? pathAndQuery : `${pathAndQuery}#p${postId}`;
}

export function getWarpUrl(boardName, pageNumber) {
    return prefix + (
        (boardName ? `/${boardName}/` : `/`) +
        (pageNumber ? `?${toQuery({p: pageNumber})}` : ``)
    );
}

export function getHomeUrl() {
    return `${prefix}/`;
}

export function getPostLinkUrl(postId) {
    return `${prefix}/@${postId}`;
}

export function getThreadPageApiUrl(threadId, pageNumber) {
    // null means the last page
    // + 1 because we want to skip OP

    return `/${api}/threadSlices/${threadId}?${toQuery({
        skip: (pageNumber === null ? 0 : pageNumber * config.pageSize.general) + 1,
        lastSlice: pageNumber === null,
        includeBoard: true,
    })}`;
}

export function getWarpPageApiUrl(boardName, pageNumber) {
    return (`/${api}/warpSlices?${toQuery({
        board: boardName,
        skip: pageNumber * config.pageSize.warp,
        includeBoard: !boardName,
    })}`);
}

export function getPublishApiUrl() {
    return `/${api}/posts`;
}

export function getUpdateHiddenByOpApiUrl() {
    return `/${api}/posts`;
}

export function getUpdateBoardDescriptionApiUrl() {
    return `/${api}/boards`;
}

export function getSubscribeApiUrl() {
    return `/${api}/subscribe`;
}

export function getContinueSubscriptionApiUrl(subscriptionId) {
    return `/${api}/continueSubscription/${subscriptionId}`;
}

export function getClientIdApiUrl() {
    return `/${api}/clientId`;
}

export function getAuthorizeApiUrl() {
    return `/${api}/authorize`;
}

export function getCreateUserApiUrl() {
    return `/${api}/users`;
}

export function getUpdateBoardIsFavoriteApiUrl(boardName) {
    return `/${api}/boards/${boardName}/favorites`;
}

export function getUpdateThreadIsFavorite(threadId) {
    return `/${api}/threads/${threadId}/favorites`;
}

export function getUpdateLastSeenAtApiUrl(threadId) {
    return `/${api}/threads/${threadId}/lastSeenAt`;
}

export function getResetAllLastSeenAtApiUrl() {
    return `/${api}/threads/lastSeenAt`;
}

export function getDeleteAllLastSeenAtApiUrl() {
    return `/${api}/threads/lastSeenAt/reset`;
}

export function getSearchBaseUrl() {
    return `${prefix}/search`;
}

export function getSearchUrl(searchArgs) {
    if (!searchArgs) return getSearchBaseUrl();

    return `${getSearchBaseUrl()}?${toQuery({
        q: searchArgs.query,
        range: searchArgs.range,
        order_desc: searchArgs.orderDesc,
        boards: searchArgs.boards,
        p: searchArgs.pageNumber,
    })}`;
}

export function getServerInfoApiUrl() {
    return `/${api}/serverInfo`;
}

export function getSearchApiUrl(searchArgs) {
    return `/${api}${getSearchUrl(searchArgs)}`;
}

export function parseSearchArgs(queryStrings) {
    if (!queryStrings.q) return null;

    return new SearchArgs(
        queryStrings.q,
        queryStrings.range,
        queryStrings.order_desc ? JSON.parse(queryStrings.order_desc) : null,
        queryStrings.boards,
        queryStrings.p ? parseInt(queryStrings.p) : null
    );
}

export function toQuery(obj) {
    return Object.keys(obj)
        .filter(k => obj[k] !== null && obj[k] !== undefined)
        .map(k => `${encodeURIComponent(k)}=${encodeURIComponent(obj[k])}`).join('&');
}

