// file: utils.js
window.$ = (selector) => document.querySelector(selector);
window.$a = (selector) => document.querySelectorAll(selector);
Element.prototype.$ = function (selector) {
    return this.querySelector(selector);
};
Element.prototype.$a = function (selector) {
    return this.querySelectorAll(selector);
};
NodeList.prototype.each = function (fn) {
    return Array.prototype.forEach.call(this, fn);
};
NodeList.prototype.indexOf = function (el) {
    return Array.prototype.indexOf.call(this, el);
};
HTMLCollection.prototype.forEach = function (fn) {
    return Array.prototype.forEach.call(this, fn);
};
HTMLCollection.prototype.map = function (fn) {
    return Array.prototype.map.call(this, fn);
};
HTMLCollection.prototype.filter = function (fn) {
    return Array.prototype.filter.call(this, fn);
};

// adds the className if el doesn't have it, and removes it if el does have it.
Element.prototype.toggleClass = function (className) {
    if (this.classList.contains(className)) {
        this.classList.remove(className);
    } else {
        this.classList.add(className);
    }
};

// adds the className if the condition is true and removes it otherwise
Element.prototype.addOrRemoveClass = function (condition, className) {
    if (condition) {
        this.classList.add(className);
    } else {
        this.classList.remove(className);
    }
};

Element.prototype.appendText = function (x) {
    const text = x.toString();
    if (text) this.appendChild(document.createTextNode(text));
    return this;
};
Element.prototype.appendAny = function (x) {
    if (x instanceof Element) {
        this.appendChild(x);
    } else if (typeof x === 'string') {
        this.appendText(x);
    } else if (Array.isArray(x)) {
        x.forEach(item => this.appendAny(item));
    } else {
        console.error(this, x);
        throw Error("appendAny: invalid argument");
    }
    return this;
};
Element.prototype.appendBefore = function (x) {
    x.parentNode.insertBefore(this, x);
};
Element.prototype.appendAfter = function (x) {
    x.parentNode.insertBefore(this, x.nextSibling);
};

Element.prototype.prependChild = function (el) {
    if (!this.firstChild) {
        this.appendChild(el);
    } else {
        this.insertBefore(el, this.firstChild);
    }
};
Element.prototype.prependText = function (text) {
    this.prependChild(document.createTextNode(text));
};

Element.prototype.moveChildNodesTo = function (el) {
    while (this.childNodes.length > 0) {
        el.appendChild(this.childNodes[0]);
    }
};

Element.prototype.show = function () {
    this.classList.remove('hidden');
};
Element.prototype.hide = function () {
    this.classList.add('hidden');
};
Element.prototype.toggleHide = function () {
    this.toggleClass('hidden');
};
Element.prototype.empty = function () {
    while (this.lastChild) this.removeChild(this.lastChild);
};
Element.prototype.showOrHide = function (condition) {
    if (condition) this.show(); else this.hide();
};
Element.prototype.isHidden = function () {
    return this.classList.contains('hidden');
};
Node.prototype.detach = function () {
    if (this.parentNode) this.parentNode.removeChild(this);
};

Element.prototype.scrollToBottom = function () {
    this.scrollTop = this.scrollHeight;
};
Element.prototype.scrollToTop = function () {
    return this.scrollTop = 0;
};
Element.prototype.getMaxScrollTop = function () {
    return (this.scrollHeight - this.clientHeight);
};

Element.prototype.replaceChildrenWith = function (x) {
    this.empty();
    this.appendAny(x);
};

export function parseCssDimension(str) {
    return str === "" ? 0 : parseFloat(str.slice(0, -2));
}

export function getComputedWidth(el) {
    return parseCssDimension(getComputedStyle(el).width);
}

export function getComputedHeight(el) {
    return parseCssDimension(getComputedStyle(el).height);
}

// export function getComputedVisibleWidth(el) { return (el.isHidden() ? 0 : getComputedWidth(el)); }

Number.prototype.getNthByte = function (n) {
    const shift = (n - 1) * 8;
    return (this >> shift) % 0x100;
};

Number.prototype.toRGB = function () {
    return `rgb(${this.getNthByte(3)}, ${this.getNthByte(2)}, ${this.getNthByte(1)})`;
};

Number.prototype.toHSL = function () {
    let h = (this >> 16) % 360;
    let s = this.getNthByte(2) % 101;
    let l = this.getNthByte(1) % 101;
    return `hsl(${h}, ${s}%, ${l}%)`;
};

Promise.prototype.thenA = function (onFulfilled, onRejected) {
    return this.then(f => f(onFulfilled), onRejected);
};

Number.prototype.saveDivide = function (x) {
    return x === 0 ? 0 : (this / x);
};

export function decorate(srcObj, dstObj) {
    const dstObjPrototype = Object.getPrototypeOf(dstObj);

    for (const property in srcObj) {
        if (!srcObj.hasOwnProperty(property)) continue;

        // ignore getters of dstObj
        const descriptor = Object.getOwnPropertyDescriptor(dstObjPrototype, property);
        const isGetter = descriptor && descriptor.get;

        if (!isGetter && !property.startsWith('_') && !dstObj[property]) {
            // console.log(property, srcObj[property]);
            dstObj[property] = srcObj[property];
        }
    }
    // copy methods
    Object.getOwnPropertyNames(
        Object.getPrototypeOf(srcObj)
    ).forEach(property => {
        if (property !== "constructor" && !property.startsWith('_') && !dstObj[property]) {
            // console.log(property, srcObj[property]);
            dstObj[property] = (...args) => srcObj[property](...args);
        }
    });
}

export function decorateView(scrView, dstView, viewName) {
    decorate(scrView, dstView);

    dstView.element = scrView.element;
    dstView.element[viewName] = dstView;
}

export function range(n) {
    return Array(n).fill();
}

Array.prototype.last = function () {
    return this[this.length - 1];
};

Array.prototype.skip = function (n) {
    return this.slice(n);
};

Array.prototype.take = function (n) {
    return this.slice(0, n);
};

Array.prototype.extendWith = function (otherArray) {
    if (!(otherArray instanceof Array)) {
        console.error(otherArray);
        throw new Error(`extend: an array was expected, ${otherArray} given`);
    }
    otherArray.forEach(x => this.push(x), this);
};

export function getNodesRange(fromEl, toEl = null) {
    let nodes = [];
    for (let el = fromEl; el && el.previousSibling !== toEl; el = el.nextSibling) {
        nodes.push(el);
    }
    return nodes;
}

export function tryDecodeURI(encodedURI) {
    try {
        return decodeURI(encodedURI);
    } catch (e) {
        return encodedURI;
    }
}

export function importScript(src) {
    return new Promise((resolve, reject) => {
        const scriptE = document.createElement("script");
        scriptE.src = src;
        scriptE.async = true;
        scriptE.onload = resolve;
        scriptE.onerror = reject;
        document.head.appendChild(scriptE);
    });
}

export function importCss(src) {
    return new Promise((resolve, reject) => {
        const linkE = document.createElement("link");
        linkE.rel = "stylesheet";
        linkE.href = src;
        linkE.onload = resolve;
        linkE.onerror = reject;
        document.head.appendChild(linkE);
    });
}

Object.filter = (obj, predicate) => Object.keys(obj)
    .filter(key => predicate(obj[key], key))
    .reduce((res, key) => {
        res[key] = obj[key];
        return res;
    }, {});


export function textToElement(text) {
    const textE = document.createElement('span');
    text.split('\n').forEach((line, idx) => {
        if (idx > 0) textE.appendChild(document.createElement('br'));
        textE.appendText(line);
    });
    return textE;
}

// quickly create a new element
export function newE(tag, attributes, ...body) {
    const element = document.createElement(tag);
    if (body) element.appendAny(body);

    Object.keys(attributes).forEach(name => element[name] = attributes[name]);

    return element;
}
