// file: chat_body_v.js
import * as repo from "../../repository";
import {ChatFormV} from "./chat_form_v";
import {ChatContentV} from "./chat_content_v";
import {ChatContentTrackNewD} from "./chat_content_track_new_d";
import {enableKatex} from "../../extra_services/katex_service";
import {enableHljs, highlightCode} from "../../extra_services/hljs_service";
import {addSettingsChangeEventListener} from "../../services/settings_service";
import {isChatHidden} from "../chat_container_v";

export class ChatBodyV {
    constructor() {
        const chatBodyE = $('#chat_body');

        this.element = chatBodyE;
        chatBodyE.chatBodyV = this;

        const recentPostsPromise = repo.getRecentPosts();
        const threadPromise = recentPostsPromise.then(rp => rp.chatThread || {});

        const chatContentV = new ChatContentTrackNewD(
            new ChatContentV().initEndlessScroll(() => highlightCode(this.element)),
            threadPromise,
        );

        this._chatContentV = chatContentV;
        chatBodyE.appendChild(chatContentV.element);

        const chatFormV = new ChatFormV(chatContentV.keepScrollBottomV);
        this._chatFromV = chatFormV;
        chatBodyE.appendChild(chatFormV.element);

        this._gapPrevention = new GapPrevention();
        this._gapPrevention.start();

        repo.addPostsListener((e, post) => {
            if (this._contextThreadId && post.thread.threadId !== this._contextThreadId) {
                return;
            }

            if (this._gapPrevention.isInProgress) {
                this._gapPrevention.addPost(post);
                return;
            }

            chatContentV.appendPosts([post]);
        });

        recentPostsPromise.then(rp => rp.posts).then(posts => {
            chatContentV.appendPosts(posts.concat(this._gapPrevention.getPosts()));

            this._gapPrevention.stop();
        }).then(() => this._afterRecentPostsLoaded());

        // listen to settings changes
        addSettingsChangeEventListener(this._onSettingsChangeHandler = e => {
            this._onSettingsChange(e.oldSettings, e.newSettings);
        });

        repo.addBoardDescriptionListener((_, board) => this._onBoardDescriptionChange(board));
        repo.addHiddenByOpListener((_, hiddenByOpEvent) => this._onUpdatedHiddenByOp(hiddenByOpEvent));
    }

    openThreadInChat(threadId) {
        if (threadId !== this._contextThreadId) {
            this._setContextReloadChat(threadId);
        }
    }

    leaveThread() {
        if (this._contextThreadId) {
            this._setContextReloadChat(null);
        }
    }

    _setContextReloadChat(threadId) {
        this._contextThreadId = threadId;
        this._chatContentV.setContextThreadId(this._contextThreadId);
        this._disableConnectedEventListener = true;
        this._chatContentV.clear();

        this._gapPrevention.start();

        repo.getRecentPosts(this._contextThreadId).then(rp => rp.posts).then(posts => {
            this._chatContentV.appendPosts(posts.concat(this._gapPrevention.getPosts()));

            this._gapPrevention.stop();
            this._disableConnectedEventListener = false;
        }).then(() => {
            this._chatContentV.afterRecentPostsLoaded();
            highlightCode(this.element);
        });
    }

    _afterRecentPostsLoaded() {
        // if EventSource fails, it reconnects and 'connected' event happens
        // we listen to this event and request posts after the last loaded one
        // and meanwhile we prevent gaps
        repo.addConnectedEventListener(() => {
            if (this._disableConnectedEventListener) return;

            this._gapPrevention.start();

            repo.getPostsAfter(
                this._chatContentV.getLastNonEphemeralPostV().post.postId,
                this._contextThreadId,
            ).then(posts => {
                this._chatContentV.appendPosts(posts.concat(this._gapPrevention.getPosts()));

                this._gapPrevention.stop();
                highlightCode(this.element);
            });
        });

        this._chatContentV.afterRecentPostsLoaded();
        highlightCode(this.element);
    }

    _onSettingsChange(oldSettings, settings) {

        return (oldSettings.isKatexEnabled !== settings.isKatexEnabled
                ? enableKatex(settings.isKatexEnabled) : Promise.resolve()
        ).then(() =>
            oldSettings.highlightCode !== settings.highlightCode
                ? enableHljs(settings.highlightCode).then() : null
        ).then(() => {
            this._chatContentV.reloadPosts();

            if (settings.highlightCode) highlightCode(this.element);

            if (oldSettings.trackNewPosts !== settings.trackNewPosts && !settings.trackNewPosts) {
                this._chatContentV.disableTracking();
            }

            if (oldSettings.isCompactTheme !== settings.isCompactTheme) {
                if (!isChatHidden()) {
                    this._chatFromV.tbxV.forceResize();
                }
            }
        });
    }

    _onBoardDescriptionChange(board) {
        this._chatContentV.onBoardDescriptionChange(board.name, board.description);
    }

    _onUpdatedHiddenByOp(hiddenByOpEvent) {
        this._chatContentV.onUpdatedHiddenByOp(hiddenByOpEvent);
    }
}

class GapPrevention {
    constructor() {
        // Posts awaiting to be added. These posts are from the event source only.
        this._posts = [];
        this._isInProgress = false;
    }

    get isInProgress() {
        return this._isInProgress;
    }

    addPost(post) {
        if (this._posts.some(p => p.postId === post.postId)) return;

        this._posts.push(post);
    }

    getPosts() {
        return this._posts.slice();
    }

    start() {
        this._posts = [];
        this._isInProgress = true;
    }

    stop() {
        this._posts = [];
        this._isInProgress = false;
    }
}