// file: chat_body_track_new_d.js
import {decorateView} from "../../utils";
import {NewPostsSeparatorV} from "./new_posts_separator_v";
import {isChatOrDocumentHidden} from "../chat_container_v";
import {config} from "../../config";
import * as repo from "../../repository";
import {getSettings} from "../../services/settings_service";
import * as titleService from "../../services/title_service";

/**
 * A chat content decorator that tracks which posts are new.
 */
export class ChatContentTrackNewD {
    constructor(chatContentV, threadPromise) {
        decorateView(chatContentV, this, 'chatContentV');

        this.chatContentV = chatContentV;
        this.element = chatContentV.element;

        this._newPostsSeparatorV = new NewPostsSeparatorV();

        this._lastSeenAt = null;
        this._storedLastSeenAt = null; // the value in the storage

        threadPromise.then(thread => {
            this._lastSeenAt = thread.lastSeenAt;
            this._storedLastSeenAt = thread.lastSeenAt;

            document.addEventListener(
                'chatVisibilityChange',
                e => this._onChatVisibilityChange(e), false
            );
        });

        this.appendPosts = this.chatContentV.appendPosts.bind(this);
        this.prependPosts = this.chatContentV.prependPosts.bind(this);
        this.prependPostV = this.chatContentV.prependPostV.bind(this);
        this.reloadPosts = this.chatContentV.reloadPosts.bind(this);

        window.addEventListener('keydown', e => titleService.setTitlePrefix(``), false);
        window.addEventListener('mousedown', e => titleService.setTitlePrefix(``), false);
    }

    appendPostV(prevV, postV) {
        if (getSettings().trackNewPosts && isChatOrDocumentHidden()
            && postV && postV.post.postDate > this._lastSeenAt) {
            titleService.setTitlePrefix("[new posts]");
        }
        return this.chatContentV.appendPostV.call(this, prevV, postV);
    }

    getLastPostV() {
        return this.chatContentV.getLastPostV();
    }

    getLastNonEphemeralPostV() {
        return this.chatContentV.getLastNonEphemeralPostV();
    }

    afterRecentPostsLoaded() {
        this.chatContentV.afterRecentPostsLoaded();
        this._updateLastSeenAtRepeatedly();
    }

    prependBetweenPosts(postV, nextV) {
        if (this._isNewPostsSeparatorBetween(postV, nextV)) {
            this._newPostsSeparatorV.element.detach();
            this.element.prependChild(this._newPostsSeparatorV.element);
        }

        this.chatContentV.prependBetweenPosts(postV, nextV);
    }

    appendBetweenPosts(prevV, postV) {
        this.chatContentV.appendBetweenPosts(prevV, postV);

        if (this._isNewPostsSeparatorBetween(prevV, postV)) {
            this._newPostsSeparatorV.element.detach();
            this.element.appendChild(this._newPostsSeparatorV.element);
        }
    }

    disableTracking() {
        this._lastSeenAt = null;
        this._storedLastSeenAt = null;
        this._newPostsSeparatorV.element.detach();
    }

    _updateLastSeenAt() {
        if (!getSettings().trackNewPosts) return;

        // do not do anything if chat is hidden
        if (isChatOrDocumentHidden()) return;

        // because when chat is visible
        // we want to prevent moving the separator lower
        // and also we want to prevent showing the separator if it's not shown now
        this._lastSeenAt = null;

        const lastPostV = this.getLastPostV();

        if (lastPostV && (!this._storedLastSeenAt || this._storedLastSeenAt < lastPostV.post.postDate)) {
            // make a request to store lastSeenAt as lastPostV.post.postDate
            repo.updateLastSeenAt(config.chat.threadId, lastPostV.post.postDate).then(() => {
                // success => change the local variable
                this._storedLastSeenAt = lastPostV.post.postDate;
            });
        }
    }

    _updateLastSeenAtRepeatedly() {
        this._updateLastSeenAt();

        setTimeout(() => this._updateLastSeenAtRepeatedly(), config.chat.updateLastSeenAtTimeout);
    }

    _isNewPostsSeparatorBetween(postV1, postV2) {
        if (!getSettings().trackNewPosts) return false;

        if (!postV1 || !postV2 || !this._lastSeenAt) return false;

        return (postV1.post.postDate <= this._lastSeenAt &&
            this._lastSeenAt < postV2.post.postDate);
    }

    _onChatVisibilityChange(e) {
        if (!getSettings().trackNewPosts) return;

        if (e.detail.isHidden) {
            this._newPostsSeparatorV.element.detach();

            const lastPostV = this.getLastPostV();
            if (lastPostV) {
                this._lastSeenAt = lastPostV.post.postDate;
            }

            titleService.setTitlePrefix(``);
        }
    }
}
