// file: chat_content_v.js
import {config} from "../../config";
import {KeepScrollBottomV} from "./keep_scroll_bottom_v";
import * as repo from "../../repository";
import {ChatPostV} from "./chat_post_v";
import {DaysSeparatorV} from "./days_separator_v";
import {getSettings} from "../../services/settings_service";

export class ChatContentV {
    constructor() {
        const contentE = document.createElement('div');
        contentE.id = 'chat_content';

        this.element = contentE;
        contentE.chatContentV = this;

        contentE.classList.add(`content`);

        this.chatPosts = new ChatPosts();

        this.keepScrollBottomV = new KeepScrollBottomV(
            contentE,
            config.chat.keepScrollBottomThreshold,
        );
    }

    initEndlessScroll(afterPrependPosts) {
        this.element.addEventListener('scroll', e => {
            if (this.element.scrollTop > 0 || !this.getFirstPostV()) return;

            repo.getPostsBefore(this.getFirstNonEphemeralPostV().post.postId, this._contextThreadId)
                .then(posts => {
                    this.prependPosts(posts);
                    afterPrependPosts()
                });
        }, false);

        return this;
    }

    afterRecentPostsLoaded() {
    }

    getFirstPostV() {
        return this._getPostsVs()[0];
    }

    getFirstNonEphemeralPostV() {
        return this._getPostsVs().filter(chatPostV => !chatPostV.post.ephemeral)[0];
    }

    getLastPostV() {
        return this._getPostsVs().last();
    }

    getLastNonEphemeralPostV() {
        return this._getPostsVs().filter(chatPostV => !chatPostV.post.ephemeral).last();
    }

    prependPosts(posts) {
        const prevScrollHeight = this.element.scrollHeight;

        this.chatPosts.addPosts(posts)
            .filter(post => !getSettings().hideEphemeralPosts || !post.ephemeral)
            .reverse()
            .map(post => new ChatPostV(post))
            .reduce((nextV, postV) => this.prependPostV(nextV, postV), this.getFirstPostV());

        this.element.scrollTop += (this.element.scrollHeight - prevScrollHeight);
    }

    appendPosts(posts) {
        this.chatPosts.addPosts(posts)
            .filter(post => !getSettings().hideEphemeralPosts || !post.ephemeral)
            .map(post => new ChatPostV(post))
            .reduce((prevV, postV) => this.appendPostV(prevV, postV), this.getLastPostV());

        this.keepScrollBottomV.keepScrollBottom();
    }

    prependPostV(nextV, postV) {
        this.prependBetweenPosts(postV, nextV);
        this.element.prependChild(postV.element); // NOTE: it's ok if postV.element = null

        return postV;
    }

    appendPostV(prevV, postV) {
        this.appendBetweenPosts(prevV, postV);
        this.element.appendChild(postV.element);

        return postV;
    }

    prependBetweenPosts(postV, nextV) {
        if (isDifferentDay(postV, nextV)) {
            const daysSeparatorV = new DaysSeparatorV(postV.post.postDate);
            this.element.prependChild(daysSeparatorV.element);
        }
    }

    appendBetweenPosts(prevV, postV) {
        if (isDifferentDay(prevV, postV)) {
            const daysSeparatorV = new DaysSeparatorV(postV.post.postDate);
            this.element.appendChild(daysSeparatorV.element);
        }
    }

    /**
     * Clears the chat content, i.e. removes all posts and day separators.
     */
    clear() {
        this.chatPosts.clear();
        this.element.empty();
    }

    /**
     * Gets the loaded posts in the order they are shown.
     */
    getPosts() {
        return this._getPostsVs().map(chatPostV => chatPostV.post);
    }

    /**
     * Reloads current posts, i.e. recreates chat post views.
     * This is needed after, for example, some user setting has been changed.
     */
    reloadPosts() {
        const posts = this.getPosts();
        const maxScrollTop = this.element.scrollHeight - this.element.offsetHeight;
        const scrollTop = this.element.scrollTop;
        const scrolledFromBottom = maxScrollTop - scrollTop;

        this.clear();
        this.appendPosts(posts);
        const newMaxScrollTop = this.element.scrollHeight - this.element.offsetHeight;
        this.element.scrollTop = newMaxScrollTop - scrolledFromBottom;
    }

    onBoardDescriptionChange(boardName, boardDescription) {
        this._getPostsVs().filter(
            chatPostV => chatPostV.post.thread.board.name === boardName
        ).forEach(
            chatPostV => chatPostV.changeBoardDescription(boardDescription)
        );
    }

    onUpdatedHiddenByOp(hiddenByOpEvent) {
        this.getPosts().filter(post => post.postId === hiddenByOpEvent.postId)
            .forEach(post => post.hiddenByOp = hiddenByOpEvent.hiddenByOp);

        this.reloadPosts();
    }

    setContextThreadId(threadId) {
        this._contextThreadId = threadId;
    }

    _getPostsVs() {
        return this.element.children.filter(element => element.chatPostV)
            .map(postE => postE.chatPostV);
    }
}

function isDifferentDay(postV1, postV2) {
    if (!postV1 || !postV2) return false;

    const dateX = postV1.post.postDate.toISOString().substr(0, 10);
    const dateY = postV2.post.postDate.toISOString().substr(0, 10);

    return dateX < dateY;
}

class ChatPosts {
    constructor() {
        // loaded posts. Key: postId, value: post.
        this._posts = {};
    }

    /**
     * Adds new (not loaded yet) posts to the loaded ones.
     * This function must guarantee that no post will be loaded more than once.
     * @param {Post[]} posts
     * @returns {Post[]} Added posts.
     */
    addPosts(posts) {
        return this._addLoadedPosts(this._skipLoadedPosts(posts));
    }

    clear(posts) {
        this._posts = {};
    }

    _skipLoadedPosts(posts) {
        return posts.filter(post => !(post.postId in this._posts));
    }

    _addLoadedPosts(posts) {
        return posts.map(post => this._posts[post.postId] = post);
    }
}
