// file: chat_post_v.js
import {PostBodyV} from "../post/post_body_v";
import {PostV} from "../post/post_v";
import {config} from "../../config";
import {ChatPostBodyD} from "./chat_post_body_d";
import {getSettings} from "../../services/settings_service";
import {containCyrillic, getShannonEntropy, parseColorsFilters, parseRegexpFilters} from "../filter_service";
import {toShortcutUuid, toSimpleUuid} from "../../services/uuid_service";

export class ChatPostV {
    constructor(post) {
        this.post = post;

        const postE = document.createElement('div');
        postE.classList.add('post');

        this.element = postE;
        postE.chatPostV = this;

        const infoE = document.createElement('div');
        this.infoE = infoE;
        infoE.classList.add('info');
        postE.appendChild(infoE);

        const infoTimeE = document.createElement('div');
        infoTimeE.classList.add('info_time');
        infoE.appendChild(infoTimeE);

        const infoIdE = document.createElement('div');
        infoIdE.classList.add('info_id');
        infoE.appendChild(infoIdE);

        const infoNameE = document.createElement('div');
        infoNameE.classList.add('info_name');
        infoE.appendChild(infoNameE);

        const idE = document.createElement('span');
        idE.classList.add('id');
        idE.style.color = post.fg.toHSL();
        if (post.ephemeral) {
            idE.style.backgroundImage = `repeating-linear-gradient(
            -45deg, transparent, transparent 1px, ${post.bg.toHSL()} 2px, ${post.bg.toHSL()} 10px)`
        } else {
            idE.style.backgroundColor = post.bg.toHSL();
        }
        idE.title = PostV.formatDate(post.postDate);
        idE.appendText(toShortcutUuid(post.postId.toString()));

        // const timeE = document.createElement('span');
        // timeE.classList.add('date');
        // timeE.appendText(ChatPostV.formatTime(post.postDate));

        const nameE = document.createElement('span');
        nameE.classList.add('name');
        if (post.user.banned) nameE.classList.add('banned');
        nameE.appendText(post.user.nickname);

        // infoTimeE.appendChild(timeE);
        infoNameE.appendChild(nameE);
        infoIdE.appendChild(idE);

        idE.addEventListener('click', e => this._reply(), false);

        if (this._isFromOtherBoard()) {
            postE.classList.add('from_other_board');
        }

        const isPostHidden = (
            this.post.hiddenByOp ||
            (getSettings().hideEphemeralPosts && this.post.ephemeral) ||
            (this._isFromOtherBoard() && getSettings().showOnlyChatPostsInChat) ||
            (getSettings().ignoreCyrillicShit && containCyrillic(post.text)) ||
            (getSettings().maxEntropy > 0 && getShannonEntropy(post.text) > getSettings().maxEntropy) ||
            parseRegexpFilters(getSettings().regexpFilters).some(re => re.test(post.text)) ||
            parseColorsFilters(getSettings().colorsFilters).some(({bg, fg}) => post.bg === bg && post.fg === fg)
        );

        if (isPostHidden) {
            this._hide();
            // avoid creating postBodyV in order to not waste time parsing
        } else {
            this._createPostBody();
        }
    }

    static formatTime(postDate) {
        return postDate.toISOString().slice(0, 19).split('T')[1];
    }

    changeBoardDescription(boardDescription) {
        this.post.thread.board.description = boardDescription;

        if (this.postBodyV) this.postBodyV.changeBoardDescription(boardDescription);
    }

    _hide() {
        if (!getSettings().displayHiddenPostsInChat) {
            this.element.classList.add('hidden');
        }
        this.element.classList.add('hidden_post');

        const buttonsContainerE = document.createElement('div');
        buttonsContainerE.classList.add('buttons_container');
        this.element.appendChild(buttonsContainerE);

        const unhideBtnE = document.createElement('i');
        unhideBtnE.classList.add('unhide_post_btn');
        unhideBtnE.classList.add('icon-eye');
        unhideBtnE.title = `Unhide: hidden by ${this.post.hiddenByOp ? `OP` : `you`}`;
        buttonsContainerE.appendChild(unhideBtnE);

        const hideBtnE = document.createElement('i');
        hideBtnE.classList.add('hide_post_btn');
        hideBtnE.classList.add('icon-eye-blocked');
        hideBtnE.title = 'Hide (only for you)';
        hideBtnE.hide();
        buttonsContainerE.appendChild(hideBtnE);

        unhideBtnE.addEventListener('click', e => {
            this._createPostBody();
            this.element.classList.remove('hidden');
            this.element.classList.remove('hidden_post');

            hideBtnE.show();
            unhideBtnE.hide();
        }, false);

        hideBtnE.addEventListener('click', e => {
            if (!getSettings().displayHiddenPostsInChat) {
                this.element.classList.add('hidden');
            }
            this.element.classList.add('hidden_post');

            unhideBtnE.show();
            hideBtnE.hide();
        }, false);
    }

    _createPostBody() {
        if (this.postBodyV) return;

        this.postBodyV = new ChatPostBodyD(new PostBodyV(this.post));
        this.postBodyV.element.appendAfter(this.infoE);
        // this.element.appendChild(this.postBodyV.element);
    }

    _isFromOtherBoard() {
        return this.post.thread.board.name !== config.chat.boardName;
    }

    _reply() {
        const chatFormV = $('#chat_form').chatFormV;
        const isAtBeginning = chatFormV.tbxV.getTextBeforeSelectionEnd().trim().length === 0;

        const postRef = `@${toSimpleUuid(this.post.postId)} `;
        const boardName = this.post.thread.board.name;
        const threadId = toSimpleUuid(this.post.thread.threadId);

        const text = (
            boardName !== config.chat.boardName && isAtBeginning
                ? `/${boardName}/${threadId} ${postRef}`
                : postRef
        );

        chatFormV.tbxV.insertText(text);
    }
}