// file: chat_textbox_d.js
import {decorateView} from "../../utils";
import {isAltEnter, isEnter} from "../../services/shortcut_service";
import {config} from "../../config";
import * as repo from "../../repository";
import {showNotification} from "../notifications/notification_v";
import {updateClientId} from "../../services/client_service";
import {NewPost, Post} from "../../models";
import {showHelpPopup} from "../menu/help_btn_v";
import {uuidPattern} from "../../services/uuid_service";
import {Board} from "../../models";
import {showSettingsPopup} from "../menu/settings_btn_v";
import {getSettings, saveSettings} from "../../services/settings_service";
import {parseColorsFilters} from "../filter_service";
import {isEphemeralPosting} from "../../services/ephemeral_posting_service";

export class ChatTextboxD {
    constructor(textboxV) {
        decorateView(textboxV, this);
        this.element = textboxV.element;

        this._boardRe = new RegExp(`^\\/(${config.boardNamePattern}) `);
        this._boardThreadRe = new RegExp(`^\\/(${config.boardNamePattern})\\/${uuidPattern} `);
        this._boardSubjectRe = new RegExp(`^\\/(${config.boardNamePattern})\\[(.+)\\] `);
        this._hideColorRe = new RegExp(`^\\/hide\\s*@${uuidPattern}`);
        this._unhideColorRe = new RegExp(`^\\/unhide\\s*@${uuidPattern}`);

        this.element.addEventListener("keydown", e => {
            if (isAltEnter(e)) {
                this.insertText("\n");
            }
            else if (isEnter(e)) {
                e.preventDefault();
                this._onSubmit();
            }
        }, false);
    }

    _onSubmit() {
        const text = this.element.value.trim();
        if (!text) return;

        this.clear();

        if (text === `/ep`) {
            this._toggleEphemeralPosting();
            return;
        }

        if (text === `/cc`) {
            this._changeColors();
            return;
        }

        if (text === `/help`) {
            showHelpPopup();
            return;
        }

        if (text === `/settings`) {
            showSettingsPopup();
            return;
        }

        if (text === `//`) {
            this._contextThreadId = null;
            this._getChatBodyV().leaveThread();
            return;
        }

        const hideColorMatch = this._hideColorRe.exec(text);
        if (hideColorMatch) {
            const postId = hideColorMatch[1];
            this._hideColor(postId);
            return;
        }

        const unhideColorMatch = this._unhideColorRe.exec(text);
        if (unhideColorMatch) {
            const postId = unhideColorMatch[1];
            this._unhideColor(postId);
            return;
        }

        const boardMatch = this._boardRe.exec(text);
        if (boardMatch) {
            const boardName = boardMatch[1];
            const boardDescription = text.substr(boardName.length + 2);

            this._updateBoardDescription(boardName, boardDescription);
            return;
        }

        const boardThreadMatch = this._boardThreadRe.exec(text);
        if (boardThreadMatch) {
            const boardName = boardThreadMatch[1];
            const threadId = boardThreadMatch[2];

            // tmp disabled todo
            // this._contextThreadId = toHyphenatedUuid(threadId);
            // this._getChatBodyV().openThreadInChat(this._contextThreadId);

            this._postToThread(
                text.substr(`/${boardName}/${threadId} `.length),
                threadId,
            );
            return;
        }

        const boardSubjectMatch = this._boardSubjectRe.exec(text);
        if (boardSubjectMatch) {
            const boardName = boardSubjectMatch[1];
            const subject = boardSubjectMatch[2];

            this._createThread(
                text.substr(`/${boardName}[${subject}] `.length),
                boardName,
                subject,
            );
            return;
        }

        if (this._contextThreadId) {
            this._postToThread(text, this._contextThreadId);
            return;
        }

        this._postToChat(text);
    }

    _updateBoardDescription(boardName, boardDescription) {
        if (boardName === config.chat.boardName) {
            showNotification(
                config.notification.type.warning,
                `/cht/ description cannot be changed.`
            );
            return;
        }

        repo.updateBoardDescription(new Board(boardName, boardDescription)).then(_ => {
            showNotification(config.notification.type.success, `The description was changed.`);
        }).catch(e => {
            showNotification(config.notification.type.error, `Failed to change the description.`);
        });
    }

    _changeColors() {
        updateClientId().then(() => {
            showNotification(config.notification.type.success, `Your colors were changed.`);
        }).catch(e => {
            showNotification(config.notification.type.error, `Failed to change your colors.`);
        });
    }

    _hideColor(postId) {
        repo.getPost(postId).then(post => {
            const settings = getSettings();
            settings.colorsFilters = settings.colorsFilters || "";

            const filters = parseColorsFilters(settings.colorsFilters);

            if (!filters.some(({bg, fg}) => post.bg === bg && post.fg === fg)) {
                filters.push({bg: post.bg, fg: post.fg});
            }

            settings.colorsFilters = filters.map(({bg, fg}) => `${bg}:${fg}`).join('\n');
            saveSettings(settings);
        }).catch(errorResponse => {
            showNotification(config.notification.type.error, errorResponse.message);
        });
    }

    _unhideColor(postId) {
        repo.getPost(postId).then(post => {
            const settings = getSettings();
            const filter = `${post.bg}:${post.fg}\n`;

            settings.colorsFilters = settings.colorsFilters || "";

            const filters = parseColorsFilters(settings.colorsFilters);
            settings.colorsFilters = filters.filter(({bg, fg}) => !(bg === post.bg && fg === post.fg))
                .map(({bg, fg}) => `${bg}:${fg}\n`).join('');

            saveSettings(settings);
        }).catch(errorResponse => {
            showNotification(config.notification.type.error, errorResponse.message);
        });
    }

    _toggleEphemeralPosting() {
        const settings = getSettings();
        settings.isEphemeralPostingOn = !settings.isEphemeralPostingOn;
        saveSettings(settings);
    }

    _postToChat(text) {
        const post = new Post(
            config.pendingPostId, new Date(), 0, 0, text, false, config.chat.threadId
        );

        const chatContentV = this._getChatContentV();
        // todo: finish the idea of pending posts: chatContentV contains PendingPostsContainerV
        // which contains PendingPostVs

        const ephemeral = isEphemeralPosting(text);

        repo.publish(new NewPost(null, text, config.chat.threadId, ephemeral)).then(post => {
            // todo: remove pending post
        }).catch(e => this._onPublishError(e));
    }

    _postToThread(text, threadId) {
        const ephemeral = isEphemeralPosting(text);

        repo.publish(new NewPost(null, text, threadId, ephemeral))
            .catch(e => this._onPublishError(e));
    }

    _createThread(text, boardName, subject) {
        const ephemeral = isEphemeralPosting(text);

        repo.publish(new NewPost(null, text, null, ephemeral, subject, boardName))
            .catch(e => this._onPublishError(e));
    }

    _getChatBodyV() {
        return $('#chat_body').chatBodyV;
    }

    _getChatContentV() {
        return $('#chat_content').chatContentV;
    }

    _onPublishError(error) {
        if (error.errorCode === 'E0401') {
            showNotification(config.notification.type.error, `${error.message}. Please log in.`);
            return;
        }

        showNotification(config.notification.type.error, `Message failed to send.`);
    }
}