// file: container_v.js
import * as repo from "../repository";
import {WarpPageV} from "./warp/warp_page_v";
import {ThreadPageV} from "./thread_page/thread_page_v";
import * as locationService from "../services/location_service";
import {LocationState} from "../location_state";
import {ThreadPageWithStateD} from "./thread_page/thread_page_with_state_d";
import {WarpPageWithStateD} from "./warp/warp_page_with_state_d";
import {SearchV} from "./search/search_v";
import {SearchWithStateD} from "./search/search_with_state_d";
import {highlightCode} from "../extra_services/hljs_service";
import {addSettingsChangeEventListener, getSettings} from "../services/settings_service";
import * as pcService from "../services/post_cache_service";
import {showNotification} from "./notifications/notification_v";
import {config} from "../config";

export class ContainerV {
    constructor() {
        const containerE = $("#container");
        this.element = containerE;
        containerE.containerV = this;

        this._locationState = new LocationState();
        this._currentV = null; // the loaded view at the moment

        if (locationService.getSelectedThreadId()) {
            this._loadThread(
                locationService.getSelectedThreadId(),
                locationService.getPage(),
                locationService.getPostIdAnchor(),
                false
            );
        } else if (locationService.isSearchPage()) {
            this._loadSearch(
                locationService.getSearchArgs(),
                false
            );
        } else {
            this._loadWarp(
                locationService.getSelectedBoardName(),
                locationService.getPage(),
                false
            );
        }

        document.addEventListener('selectBoard', e => this._onSelectBoard(e), false);
        document.addEventListener('selectThread', e => this._onSelectThread(e), false);
        document.addEventListener('selectGlobalPage', e => this._onSelectGlobalPage(e), false);
        document.addEventListener('selectSearch', e => this._onSelectSearch(e), false);
        document.addEventListener('setPostIdHash', e => this._onSetPostIdHash(e), false);
        document.addEventListener('authorizeChange', e => this._onAuthorizeChange(e), false);

        // listen to settings changes
        // NOTE: we listen here, because ContainerV is, unlike its children, created only once
        addSettingsChangeEventListener(this._onSettingsChangeHandler = e => {
            this._onSettingsChange(e.oldSettings, e.newSettings);
        });

        new ContainerPostsListener(() => this._currentV);
        new ContainerBoardDescriptionListener(() => this._currentV);
        new ContainerHiddenByOpListener(() => this._currentV);
    }

    _onSelectBoard(e) {
        const boardName = e.detail.boardName;
        const pageNumber = e.detail.pageNumber;
        const isPopState = e.detail.isPopState;

        this._loadWarp(boardName, pageNumber, isPopState);
    }

    _onSelectThread(e) {
        const threadId = e.detail.threadId;
        const pageNumber = e.detail.pageNumber;
        const postId = e.detail.postId;
        const isPopState = e.detail.isPopState;

        this._loadThread(threadId, pageNumber, postId, isPopState);
    }

    _onSelectGlobalPage(e) {
        const pageNumber = e.detail.pageNumber;
        const isPopState = e.detail.isPopState;

        this._loadWarp(null, pageNumber, isPopState);
    }

    _onSelectSearch(e) {
        const searchArgs = e.detail.searchArgs;
        const isPopState = e.detail.isPopState;

        this._loadSearch(searchArgs, isPopState);
    }

    _loadWarp(boardName, pageNumber, isPopState) {
        repo.getWarpPage(boardName, pageNumber).then(warpPage => {
            const warpPageV = new WarpPageWithStateD(
                new WarpPageV(warpPage)
            );
            this._loadContent(warpPageV, null, isPopState);
        });
    }

    _loadThread(threadId, pageNumber, postId, isPopState) {
        repo.getThreadPage(threadId, pageNumber).then(threadPage => {
            const threadPageV = new ThreadPageWithStateD(
                new ThreadPageV(threadPage)
            );
            this._loadContent(threadPageV, postId, isPopState);
        }).catch(e => {
            showNotification(config.notification.type.error, e.message);
        });
    }

    _loadSearch(searchArgs, isPopState) {
        repo.getSearch(searchArgs).then(searchResultPage => {
            const searchV = new SearchWithStateD(
                new SearchV(searchArgs, searchResultPage)
            );
            this._loadContent(searchV, null, isPopState);
        });
    }

    _loadContent(view, postId, isPopState) {
        if (this._locationState.isLocationSet()) {
            // save the state
            this._locationState.setState(this._getState());
        }
        // update the previous location href to the current location href
        this._locationState.updateLocation();

        this.element.replaceChildrenWith(view.element);
        this._currentV = view;

        highlightCode(this.element);

        if (postId) locationService.activateTarget();

        if (isPopState) {
            this._restoreState(this._locationState.getState());
            return;
        }

        if (!postId) this.element.scrollToTop();
    }

    _onSetPostIdHash(e) {
        // save the scroll position only (because the content is not changed.)
        this._locationState.setState({
            scrollTop: this.element.scrollTop
        });
        this._locationState.updateLocation();
    }

    _onAuthorizeChange(e) {
        $a(".new_thread_form").forEach(e => e.newThreadFormV.onAuthorizeChange());
        $a(".reply_form").forEach(e => e.replyFormV.onAuthorizeChange());
    }

    _onSettingsChange(oldSettings, settings) {
        const state = this._getState();

        return this._currentV.onSettingsChange(oldSettings, settings).then(() => {
            this._restoreState(state);
        });
    }

    _getState() {
        return new ContainerState(
            this.element.scrollTop,
            this._currentV.getState()
        );
    }

    _restoreState(state) {
        this._currentV.restoreState(state.viewState);
        this.element.scrollTop = state.scrollTop;
    }
}

class ContainerState {
    constructor(scrollTop, viewState) {
        this.scrollTop = scrollTop;
        this.viewState = viewState;
    }
}

class ContainerPostsListener {
    constructor(getCurrentView) {
        this._getCurrentView = getCurrentView;

        repo.addPostsListener((e, post) => this._handlePost(post));
    }

    _handlePost(post) {
        if (post.ephemeral && getSettings().hideEphemeralPosts) return;

        const currentV = this._getCurrentView();
        if (currentV.onNewPost) currentV.onNewPost(post);
    }
}

class ContainerBoardDescriptionListener {
    constructor(getCurrentView) {
        this._getCurrentView = getCurrentView;

        repo.addBoardDescriptionListener(
            (e, board) => this._onBoardDescriptionChange(board.name, board.description)
        );
    }

    _onBoardDescriptionChange(boardName, boardDescription) {
        const currentV = this._getCurrentView();

        if (currentV.onBoardDescriptionChange) {
            currentV.onBoardDescriptionChange(boardName, boardDescription);
        }
    }
}

class ContainerHiddenByOpListener {
    constructor(getCurrentView) {
        this._getCurrentView = getCurrentView;

        repo.addHiddenByOpListener((e, hiddenByOpEvent) => this._onUpdatedHiddenByOp(hiddenByOpEvent));
    }

    _onUpdatedHiddenByOp(hiddenByOpEvent) {
        pcService.updateHiddenByOp(hiddenByOpEvent);

        const currentV = this._getCurrentView();
        if (currentV.onUpdatedHiddenByOp) currentV.onUpdatedHiddenByOp(hiddenByOpEvent);
    }
}
