// file: post_body_v.js
import {LinkV} from "./link_v";
import {config} from "../../config";
import {PostReferenceV} from "./post_reference_v";
import {renderKatex} from "../../extra_services/katex_service";
import {
    parseBold, parseCode, parseItalic, parseLink, parseMono, parseNamedLink, parsePostReference,
    parseQuote, parserNewline, parseSpoiler, parseStrike, parseTeX
} from "../../services/text_parsing_service";

export class PostBodyV {
    constructor(post) {
        this.post = post;

        const bodyE = document.createElement('div');
        bodyE.classList.add('body');

        // more that 2 newlines replace with 2 newlines
        const text = post.text.replace(/\n{3,}/g, "\n\n");

        formatText(text, bodyE);

        this.element = bodyE;
        bodyE.postBodyV = this;
    }
}

function formatText(text, element, level = 0) {
    if (level > config.formatting.maxNestingLevel) {
        element.appendText(text);
        return;
    }

    for (let i = 0; i < text.length; i++) {
        const textToParse = text.slice(i);
        const markupFreePrefix = text.slice(0, i);

        const newline = parserNewline(textToParse);
        if (newline) {
            element.appendText(markupFreePrefix);
            element.appendChild(document.createElement('br'));
            formatText(newline.rest, element, level);
            return;
        }
        const quote = parseQuote(textToParse, markupFreePrefix, element, level);
        if (quote) {
            element.appendText(markupFreePrefix);
            if (quote.afterPostRef) element.appendChild(document.createElement('br'));
            const quoteE = document.createElement('span');
            quoteE.classList.add('quote');
            element.appendChild(quoteE);

            formatText(quote.match, quoteE, level + 1);
            formatText(quote.rest, element, level);
            return;
        }
        const bold = parseBold(textToParse);
        if (bold) {
            element.appendText(markupFreePrefix);
            const bE = document.createElement('b');
            element.appendChild(bE);

            formatText(bold.match, bE, level + 1);
            formatText(bold.rest, element, level);
            return;
        }
        const italic = parseItalic(textToParse);
        if (italic) {
            element.appendText(markupFreePrefix);
            const iE = document.createElement('i');
            element.appendChild(iE);

            formatText(italic.match, iE, level + 1);
            formatText(italic.rest, element, level);
            return;
        }
        const code = parseCode(textToParse);
        if (code) {
            element.appendText(markupFreePrefix);
            const preE = document.createElement('pre');
            element.appendChild(preE);

            const codeE = document.createElement('code');
            if (code.lang) codeE.classList.add(code.lang);
            codeE.appendText(code.match);
            preE.appendChild(codeE);

            formatText(code.rest, element, level);
            return;
        }
        const mono = parseMono(textToParse);
        if (mono) {
            element.appendText(markupFreePrefix);
            const sampE = document.createElement('samp');
            element.appendChild(sampE);

            formatText(mono.match, sampE, level + 1);
            formatText(mono.rest, element, level);
            return;
        }
        const strike = parseStrike(textToParse);
        if (strike) {
            element.appendText(markupFreePrefix);
            const sE = document.createElement('s');
            element.appendChild(sE);

            formatText(strike.match, sE, level + 1);
            formatText(strike.rest, element, level);
            return;
        }
        const tex = parseTeX(textToParse);
        if (tex) {
            element.appendText(markupFreePrefix);
            const texE = document.createElement('span');
            element.appendChild(texE);
            renderKatex(tex.match, texE);

            formatText(tex.rest, element, level);
            return;
        }
        const spoiler = parseSpoiler(textToParse);
        if (spoiler) {
            element.appendText(markupFreePrefix);
            const spanE = document.createElement('span');
            spanE.classList.add("spoiler");
            element.appendChild(spanE);

            formatText(spoiler.match, spanE, level + 1);
            formatText(spoiler.rest, element, level);
            return;
        }
        const namedLink = parseNamedLink(textToParse);
        if (namedLink) {
            element.appendText(markupFreePrefix);
            const linkV = new LinkV(namedLink.match, namedLink.name);
            element.appendChild(linkV.element);

            formatText(namedLink.rest, element, level);
            return;
        }
        const link = parseLink(textToParse);
        if (link) {
            element.appendText(markupFreePrefix);
            const linkV = new LinkV(link.match, null);
            element.appendChild(linkV.element);

            formatText(link.rest, element, level);
            return;
        }
        const postReference = parsePostReference(textToParse);
        if (postReference) {
            element.appendText(markupFreePrefix);
            const postReferenceV = new PostReferenceV(postReference.match);
            element.appendChild(postReferenceV.element);

            formatText(postReference.rest, element, level);
            return;
        }
    }
    element.appendText(text);
}
