// file: post_v.js
import {PostBodyV} from "./post_body_v";
import {getSettings} from "../../services/settings_service";
import {containCyrillic, getShannonEntropy, parseColorsFilters, parseRegexpFilters} from "../filter_service";
import {toShortcutUuid} from "../../services/uuid_service";
import {newE} from "../../utils";
import {config} from "../../config";
import * as repo from "../../repository";
import {UpdatedHiddenByOp} from "../../models";
import {showNotification} from "../notifications/notification_v";

export class PostV {
    constructor(post, isOriginalPost) {
        this.post = post;

        const postWrapperE = document.createElement('div');
        postWrapperE.classList.add('post_wrapper');

        this.element = postWrapperE;
        this.postV = this;

        const postE = document.createElement('div');
        this.postE = postE;
        postE.classList.add('post');
        if (isOriginalPost) {
            postE.classList.add('original');
        }
        postWrapperE.appendChild(postE);

        const headerE = document.createElement('div');
        this.headerE = headerE;
        headerE.classList.add('header');

        const idE = document.createElement('span');
        this.idE = idE;
        idE.classList.add('id');
        idE.style.color = post.fg.toHSL();
        if (post.ephemeral) {
            idE.style.backgroundImage = `repeating-linear-gradient(
            -45deg, transparent, transparent 1px, ${post.bg.toHSL()} 2px, ${post.bg.toHSL()} 10px)`
        } else {
            idE.style.backgroundColor = post.bg.toHSL();
        }
        idE.appendText(toShortcutUuid(post.postId));

        const dateE = document.createElement('span');
        dateE.classList.add('date');
        dateE.appendText(PostV.formatDate(post.postDate));

        const nameE = document.createElement('span');
        nameE.classList.add('name');
        if (post.user.banned) nameE.classList.add('banned');
        nameE.appendText(post.user.nickname);

        headerE.appendChild(idE);
        headerE.appendText(' ');
        headerE.appendChild(nameE);
        headerE.appendText(' ');
        headerE.appendChild(dateE);
        postE.appendChild(headerE);

        if (!isOriginalPost && !post.ephemeral && post.thread.threadId !== config.chat.threadId) {
            this._addHideByOpButtons();
        }

        const isPostHidden = (
            this.post.hiddenByOp ||
            (getSettings().hideEphemeralPosts && this.post.ephemeral) ||
            (getSettings().ignoreCyrillicShit && containCyrillic(post.text)) ||
            (getSettings().maxEntropy > 0 && getShannonEntropy(post.text) > getSettings().maxEntropy) ||
            parseRegexpFilters(getSettings().regexpFilters).some(re => re.test(post.text)) ||
            parseColorsFilters(getSettings().colorsFilters).some(({bg, fg}) => post.bg === bg && post.fg === fg)
        );

        this._addHideUnhideButtons(isPostHidden);
    }

    changeBoardDescription(boardDescription) {
        this.post.thread.board.description = boardDescription;
    }

    static formatDate(postDate) {
        return postDate.toISOString().slice(0, 19).replace('T', ' ');
    }

    _createPostBody() {
        if (this.postBodyV) return;

        this.postBodyV = new PostBodyV(this.post);
        this.postE.appendChild(this.postBodyV.element);
    }

    _addHideUnhideButtons(isPostHidden) {
        if (this.post.ephemeral) {
            if (getSettings().hideEphemeralPosts) {
                this.element.hide();
            } else {
                this.element.show();
            }
        }

        this.headerE.appendText(' ');

        this._unhideBtnE = newE('i', {
            className: 'unhide_post_btn icon-eye',
            title: `Unhide: hidden by ${this.post.hiddenByOp ? `OP` : `you`}`,
        });
        this._unhideBtnE.hide();
        this.headerE.appendChild(this._unhideBtnE);

        this._hideBtnE = newE('i', {
            className: 'hide_post_btn icon-eye-blocked',
            title: 'Hide (only for you)',
        });
        this._hideBtnE.hide();
        this.headerE.appendChild(this._hideBtnE);

        if (isPostHidden) {
            this.postE.classList.add('hidden_post');
            this._unhideBtnE.show();
        } else {
            this._createPostBody();
        }

        this._unhideBtnE.addEventListener('click', e => this._unhide(), false);

        this._hideBtnE.addEventListener('click', e => this._hide(), false);
    }

    _hide() {
        this.postE.classList.add('hidden_post');

        this._unhideBtnE.show();
        this._hideBtnE.hide();
    }

    _unhide() {
        this._createPostBody();
        this.postE.classList.remove('hidden_post');

        this._hideBtnE.show();
        this._unhideBtnE.hide();
    }

    _addHideByOpButtons() {
        this.headerE.appendText(' ');

        const unhideByOpBtnE = newE('i', {
            className: 'unhide_by_op_btn icon-plus-square',
            title: 'Unhide for all (you have to be OP)',
        });
        this.headerE.appendChild(unhideByOpBtnE);
        if (!this.post.hiddenByOp) unhideByOpBtnE.hide();

        const hideByOpBtnE = newE('i', {
            className: 'hide_by_op_btn icon-minus-square',
            title: 'Hide for all (you have to be OP)',
        });
        this.headerE.appendChild(hideByOpBtnE);
        if (this.post.hiddenByOp) hideByOpBtnE.hide();

        unhideByOpBtnE.addEventListener('click', e => {
            repo.updateHiddenByOp(new UpdatedHiddenByOp(null, this.post.postId, false)).then(_ => {
                this._unhide();
                unhideByOpBtnE.hide();
                hideByOpBtnE.show();
                showNotification(config.notification.type.success, `The post was unhidden.`);
            }).catch(e => {
                if (e.message === "Invalid clientId") {
                    showNotification(config.notification.type.error,
                        `You're not OP of this thread: invalid clientId`, 5000);
                } else {
                    showNotification(config.notification.type.error,
                        `Failed to unhide the post: ${e.message}`);
                }
            });
        }, false);

        hideByOpBtnE.addEventListener('click', e => {
            repo.updateHiddenByOp(new UpdatedHiddenByOp(null, this.post.postId, true)).then(_ => {
                this._hide();
                hideByOpBtnE.hide();
                unhideByOpBtnE.show();
                showNotification(config.notification.type.success, `The post was hidden.`);
            }).catch(e => {
                if (e.message === "Invalid clientId") {
                    showNotification(config.notification.type.error,
                        `You're not OP of this thread: invalid clientId`, 5000);
                } else {
                    showNotification(config.notification.type.error,
                        `Failed to hide the post: ${e.message}`);
                }
            });
        }, false);
    }
}