// file: reply_form_v.js
import {TextboxV} from "../textbox_v";
import {config} from "../../config";
import * as repo from "../../repository";
import {showNotification} from "../notifications/notification_v";
import * as locationService from "../../services/location_service";
import {isCtrlEnter} from "../../services/shortcut_service";
import {NewPost} from "../../models";
import {getAccessToken} from "../../services/access_token_service";
import {
    isEphemeralPosting, isEphemeralPostingOn,
    isReplyToEphemeralPost
} from "../../services/ephemeral_posting_service";
import {addSettingsChangeEventListener, getSettings} from "../../services/settings_service";

export class ReplyFormV {
    constructor(threadId, boardName) {
        const formE = document.createElement('form');
        formE.classList.add('reply_form');

        this.element = formE;
        formE.replyFormV = this;

        this._threadId = threadId;
        this._boardName = boardName;

        const formBodyE = document.createElement('div');
        formBodyE.classList.add('form_body');
        formE.appendChild(formBodyE);

        this.tbxV = new TextboxV();
        this.tbxV.autoResize(false);
        const tbxE = this.tbxV.element;
        tbxE.id = `${threadId}tbx`;
        tbxE.classList.add('reply_tbx');
        tbxE.maxLength = config.maxPostLength;
        formBodyE.appendChild(tbxE);

        const belowTbxE = document.createElement('div');
        belowTbxE.classList.add('below_tbx');
        formBodyE.appendChild(belowTbxE);

        const submitBtnE = document.createElement('button');
        submitBtnE.classList.add('form_btn', 'reply_btn');
        submitBtnE.type = 'submit';
        submitBtnE.title = 'Ctrl+Enter';

        const replyIconE = document.createElement('i');
        replyIconE.classList.add('icon-reply');
        submitBtnE.appendChild(replyIconE);
        submitBtnE.appendText('Reply');

        belowTbxE.appendChild(submitBtnE);

        const stayHereLabelE = document.createElement('label');
        stayHereLabelE.classList.add('stay_here_lbl');

        const stayHereCbxE = document.createElement('input');
        this._stayHereCbxE = stayHereCbxE;
        stayHereCbxE.classList.add('stay_here_cbx');
        stayHereCbxE.type = 'checkbox';
        stayHereLabelE.appendChild(stayHereCbxE);
        stayHereLabelE.appendText(`stay here`);

        belowTbxE.appendChild(stayHereLabelE);

        this.onAuthorizeChange();

        submitBtnE.addEventListener('click', e => {
            if (!this.element.reportValidity()) return;
            e.preventDefault();

            this._onSubmit(e);
        }, false);

        tbxE.addEventListener("keydown", e => {
            if (isCtrlEnter(e)) {
                if (!this.element.reportValidity()) return;
                e.preventDefault();

                this._onSubmit(e);
            }
        }, false);

        this.onEphemeralPostingModeChange();
        addSettingsChangeEventListener(e => {
            if (e.oldSettings.isEphemeralPostingOn !== e.newSettings.isEphemeralPostingOn) {
                this.onEphemeralPostingModeChange();
            }
        });
    }

    onAuthorizeChange() {
        const isAuthorized = getAccessToken();
        // this.element.elements.forEach(e => e.disabled = !isAuthorized);
        this.tbxV.element.placeholder = isAuthorized ? "" : config.ephemeralModeWarning;
    }

    onEphemeralPostingModeChange() {
        this.tbxV.element.placeholder = isEphemeralPostingOn()
            ? config.ephemeralModeWarning : "";
    }

    _onSubmit(e) {
        const text = this.tbxV.element.value;
        if (!text || !text.trim()) return;

        const stayHere = this._stayHereCbxE.checked;

        const ephemeral = isEphemeralPosting(text);

        if (stayHere || ephemeral) this.element.reset();

        repo.publish(new NewPost(null, text, this._threadId, ephemeral)).then(post => {
            if (stayHere || ephemeral) return;

            locationService.setThreadPage(this._boardName, this._threadId, null, post.postId);
        }).catch(e => {
            if (e.errorCode === 'E0401') {
                showNotification(config.notification.type.error, `${e.message}. Please log in.`);
                return;
            }

            showNotification(config.notification.type.error, `Message failed to send.`);
        });
    }
}