// file: settings_v.js
import {getSettings} from "../../services/settings_service";
import {TextboxV} from "../textbox_v";
import {config} from "../../config";
import {newE} from "../../utils";
import {getOrCreateClientId} from "../../services/client_service";
import {getAccessToken} from "../../services/access_token_service";

export class SettingsV {
    constructor() {
        const settingsE = document.createElement('form');
        settingsE.classList.add('settings');
        settingsE.onsubmit = 'return false';

        this.element = settingsE;
        settingsE.settingsV = this;

        this._createGroupTitle('General');

        if (getAccessToken()) {
            this._createBooleanSetting('isEphemeralPostingOn', 'Turn on ephemeral posting');
        }
        this._createBooleanSetting('isKatexEnabled', 'Enable KaTeX');
        this._createBooleanSetting('highlightCode', 'Highlight code');

        this._createBooleanSetting('showImages', newE('span', {},
            `Show 3rd party images (imgur, youtube, twitter, peertube: `,
            newE('a', {target: '_blank', href: 'https://libre.video'}, `libre.video`),
            `, `,
            newE('a', {target: '_blank', href: 'https://peertube.social'}, `peertube.social`),
            `, `,
            newE('a', {target: '_blank', href: 'https://video.ploud.fr'}, `ploud.fr`),
            `)`
        ));
        this._createBooleanSetting('embedBitchute', 'Embed bitchute videos');

        const trackNewPostsCbxE = this._createBooleanSetting('trackNewPosts', 'Track new posts');

        const trackNewPostsWarningE = this._createSettingWarning(
            'If you disable it, current information about which posts are new will be deleted'
        );
        trackNewPostsWarningE.hide();

        trackNewPostsCbxE.addEventListener('change', () => {
            if (!trackNewPostsCbxE.checked && getSettings().trackNewPosts) {
                trackNewPostsWarningE.show();
            } else {
                trackNewPostsWarningE.hide();
            }
        }, false);

        this._createClientIdSetting();

        this._createGroupTitle('Theme');
        this._createBooleanSetting('isCompactTheme', 'Compact');

        this._createGroupTitle('Filters');
        this._createBooleanSetting('hideEphemeralPosts', 'Hide ephemeral posts');
        this._createBooleanSetting('showOnlyChatPostsInChat', 'Show only chat posts in the chat');
        this._createBooleanSetting('displayHiddenPostsInChat', 'Display hidden posts in the chat');

        if (config.showIgnoreCyrillicShitFilter) {
            this._createBooleanSetting('ignoreCyrillicShit', 'Ignore cyrillic shit');
        }

        this._createNumberSetting('maxEntropy', `Max Shannon's entropy (0 - unlimited)`);
        this._createTextareaSetting('regexpFilters', 'Filter by regexps (each line is a regexp)');
        this._createTextareaSetting('colorsFilters', 'Filter by colors (each line is a bg:fg pair)');
    }

    getSettings() {
        const settings = {};

        for (let element of this.element.$a('input, select, textarea')) {
            const value = element instanceof HTMLInputElement && element.type === 'checkbox'
                ? element.checked
                : element.value;

            if (!element.id) throw new Error('The element must have an id');

            settings[element.id] = value;
        }

        return settings;
    }

    _createGroupTitle(groupName) {
        const groupTitleE = document.createElement('div');
        groupTitleE.classList.add('group_title');
        groupTitleE.appendText(groupName);
        this.element.appendChild(groupTitleE);
    }

    _createBooleanSetting(id, description) {
        const containerE = this._createSettingContainer();

        const cbxE = document.createElement('input');
        cbxE.id = id;
        cbxE.type = 'checkbox';
        cbxE.checked = getSettings()[id];
        containerE.appendChild(cbxE);

        containerE.appendChild(this._createLabel(id, description));
        return cbxE;
    }

    _createNumberSetting(id, description, size = 2, min = 0, step = 'any') {
        const containerE = this._createSettingContainer();
        containerE.appendChild(this._createLabel(id, description));

        const inputE = document.createElement('input');
        inputE.id = id;
        inputE.type = 'number';
        inputE.size = size;
        inputE.min = min;
        inputE.step = step;
        inputE.value = getSettings()[id];
        containerE.appendChild(inputE);
    }

    _createTextareaSetting(id, description) {
        const containerE = this._createSettingContainer();
        containerE.appendChild(this._createLabel(id, description));

        const textboxV = new TextboxV();
        textboxV.element.id = id;
        textboxV.insertText(getSettings()[id]);
        containerE.appendChild(textboxV.element);
    }

    _createClientIdSetting() {
        const id = "clientId";
        const containerE = this._createSettingContainer();
        containerE.appendChild(this._createLabel(id, "Client id:"));

        const inputE = document.createElement('input');
        inputE.id = id;
        inputE.required = true;
        inputE.maxLength = config.maxClientIdLength;
        inputE.disabled = true;
        getOrCreateClientId().then(clientId => {
            inputE.value = clientId;
            inputE.disabled = false;
        });
        containerE.appendChild(inputE);
    }

    _createSettingContainer() {
        const containerE = document.createElement('div');
        containerE.classList.add('setting_container');
        this.element.appendChild(containerE);
        return containerE;
    }

    _createLabel(id, description) {
        const labelE = document.createElement('label');
        labelE.htmlFor = id;
        labelE.appendAny(description);
        return labelE;
    }

    _createSettingWarning(message) {
        const warningE = document.createElement('div');
        warningE.classList.add('setting_warning');

        const iconE = document.createElement('i');
        iconE.classList.add('icon-warning');

        warningE.appendChild(iconE);
        warningE.appendText(' ');
        warningE.appendText(message);
        this.element.appendChild(warningE);
        return warningE;
    }
}