import {DragDependantV} from "../drag/drag_depentant_v";
import {WidthStoredDragDependantD} from "../drag/width_stored_drag_dependent_d";
import {VisibilityStoredDragDependantD} from "../drag/visibility_stored_drag_dependent_d";
import {SidebarDragDependantD} from "./sidebar_drag_dependant_d";
import {SidebarBoardV} from "./sidebar_board_v";
import * as locationService from "../../services/location_service";
import * as repo from "../../repository";
import {favoriteBoardsLsKey, getFavoriteBoards} from "../../services/favorites_service";

export class BoardsSidebarV {
    constructor() {
        const sidebarE = $('#boards_sidebar');
        this.element = sidebarE;
        sidebarE.view = this;

        const SIDEBAR_WIDTH = 'boards_sidebar_width';
        const SIDEBAR_HIDDEN = 'boards_sidebar_hidden';

        this.dragDependantV = new DragDependantV(sidebarE);
        this.dragDependantV = new WidthStoredDragDependantD(
            this.dragDependantV, SIDEBAR_WIDTH, 200
        );
        this.dragDependantV = new VisibilityStoredDragDependantD(
            this.dragDependantV,
            SIDEBAR_HIDDEN, true
        );
        this.dragDependantV = new SidebarDragDependantD(this.dragDependantV);

        this.favBoardsGroupE = sidebarE.$('#fav_boards_group');
        this.otherBoardsGroupE = sidebarE.$('#other_boards_group');

        this.favBoardsGroupE.addEventListener('scroll', e => this._onScroll(e), false);
        this.otherBoardsGroupE.addEventListener('scroll', e => this._onScroll(e), false);

        const boardName = locationService.getSelectedBoardName();
        this._load(true, boardName, 0);
        this._load(false, boardName, 0);

        document.addEventListener('selectBoard', e => this._onSelectBoard(e), false);
        document.addEventListener('selectThread', e => this._onSelectThread(e), false);
        document.addEventListener('selectGlobalPage', e => this._onSelectGlobalPage(e), false);
        document.addEventListener('selectSearch', e => this._onSelectSearch(e), false);

        repo.addBoardDescriptionListener(
            (e, board) => this._onBoardDescriptionChange(board.name, board.description)
        );

        document.addEventListener('lsChange', this._onLsChangeHandler = e => this._onLsChange(e), false);
    }

    toggleFavorite(boardV) {
        boardV.element.detach();
        this.addToGroup(
            boardV,
            boardV.board.isFavorite ? this.favBoardsGroupE : this.otherBoardsGroupE
        );
    }

    addToGroup(boardV, groupE) {
        for (let el of groupE.children) {
            if (boardV.board.name === el.view.board.name) {
                // skip it, we already have this board
                return;
            }

            if (boardV.board.name <= el.view.board.name) {
                groupE.insertBefore(boardV.element, el);
                return;
            }
        }
        groupE.appendChild(boardV.element);
    }

    _load(isFavorite, boardName, offset) {
        this._currentBoardName = boardName;

        const groupE = isFavorite ? this.favBoardsGroupE : this.otherBoardsGroupE;

        repo.getBoards(isFavorite, offset).then(boards => {

            const boardVs = boards.map(board => new SidebarBoardV(board, this));
            boardVs.forEach(boardV => this.addToGroup(boardV, groupE));

            if (boardName) this._selectBoard(boardName, this._getBoardsVsOfGroup(groupE));
        });
    }

    _onSelectBoard(e) {
        this._selectBoard(e.detail.boardName);
    }

    _onSelectThread(e) {
        this._selectBoard(e.detail.boardName);
    }

    _onSelectGlobalPage(e) {
        this._deselectAll();
    }

    _onSelectSearch(e) {
        this._deselectAll();
    }

    _selectBoard(boardName, boardsVs = this._getBoardsVs()) {
        boardsVs.forEach(boardV => {
            if (boardV.board.name === boardName) {
                boardV.selectableV.select();
            } else {
                boardV.selectableV.deselect();
            }
        });
    }

    _deselectAll() {
        this._getBoardsVs().forEach(boardV => boardV.selectableV.deselect());
    }

    _getBoardsVs() {
        return this._getBoardsVsOfGroup(this.favBoardsGroupE).concat(
            this._getBoardsVsOfGroup(this.otherBoardsGroupE)
        );
    }

    _getBoardsVsOfGroup(groupE) {
        return groupE.children.map(el => el.view);
    }

    _onScroll(e) {
        const groupE = e.target;
        if (groupE.scrollTop < groupE.getMaxScrollTop()) return;

        const isFavorite = groupE === this.favBoardsGroupE;

        this._load(isFavorite, this._currentBoardName, this._getBoardsVsOfGroup(groupE).length);
    }

    _onBoardDescriptionChange(boardName, boardDescription) {
        this._getBoardsVs()
            .filter(boardV => boardV.board.name === boardName)
            .forEach(boardV => boardV.setDescription(boardDescription));
    }

    _onLsChange(e) {
        if (e.detail.isFromForeignWindow && e.detail.key === favoriteBoardsLsKey) {
            let favoriteBoards = getFavoriteBoards();

            this._getBoardsVs().forEach(boardV => {
                const isFavorite = favoriteBoards.some(b => b.name === boardV.board.name);

                boardV.toggleFavoriteUiOnly(isFavorite);
            });
        }
    }
}