import {DragDependantV} from "../drag/drag_depentant_v";
import {WidthStoredDragDependantD} from "../drag/width_stored_drag_dependent_d";
import {VisibilityStoredDragDependantD} from "../drag/visibility_stored_drag_dependent_d";
import {SidebarDragDependantD} from "./sidebar_drag_dependant_d";
import {SidebarThreadV} from "./sidebar_thread_v";
import * as locationService from "../../services/location_service";
import * as repo from "../../repository";
import {favoriteThreadsLsKey, getFavoriteThreads} from "../../services/favorites_service";

export class ThreadsSidebarV {
    constructor() {
        const sidebarE = $('#threads_sidebar');
        this.element = sidebarE;
        sidebarE.view = this;

        const SIDEBAR_WIDTH = 'threads_sidebar_width';
        const SIDEBAR_HIDDEN = 'threads_sidebar_hidden';

        this.dragDependantV = new DragDependantV(sidebarE);
        this.dragDependantV = new WidthStoredDragDependantD(
            this.dragDependantV, SIDEBAR_WIDTH, 200
        );
        this.dragDependantV = new VisibilityStoredDragDependantD(
            this.dragDependantV, SIDEBAR_HIDDEN, true
        );
        this.dragDependantV = new SidebarDragDependantD(this.dragDependantV);

        this.favThreadsGroupE = $('#fav_threads_group');
        this.otherThreadsGroupE = $('#other_threads_group');

        this.favThreadsGroupE.addEventListener('scroll', e => this._onScroll(e), false);
        this.otherThreadsGroupE.addEventListener('scroll', e => this._onScroll(e), false);

        const boardName = (locationService.showAllThreads()
                ? null :
                locationService.getSelectedBoardName()
        );
        this._load(true, boardName, locationService.getSelectedThreadId(), 0);
        this._load(false, boardName, locationService.getSelectedThreadId(), 0);

        document.addEventListener('selectBoard', e => this._onSelectBoard(e), false);
        document.addEventListener('selectThread', e => this._onSelectThread(e), false);
        document.addEventListener('selectGlobalPage', e => this._onSelectGlobalPage(e), false);
        document.addEventListener('selectSearch', e => this._onSelectSearch(e), false);

        document.addEventListener('lsChange', this._onLsChangeHandler = e => this._onLsChange(e), false);
    }

    toggleFavorite(threadV) {
        threadV.element.detach();
        this.addToGroup(
            threadV,
            threadV.thread.isFavorite ? this.favThreadsGroupE : this.otherThreadsGroupE
        );
    }

    addToGroup(threadV, groupE) {
        for (let el of groupE.children) {
            if (threadV.thread.threadId === el.view.thread.threadId) {
                // skip it, we already have this thread
                return;
            }

            if (threadV.thread.lastPostDate >= el.view.thread.lastPostDate) {
                groupE.insertBefore(threadV.element, el);
                return;
            }
        }
        groupE.appendChild(threadV.element);
    }

    _load(isFavorite, boardName, threadId, offset) {
        this._currentBoardName = boardName;
        this._currentThreadId = threadId;

        const groupE = isFavorite ? this.favThreadsGroupE : this.otherThreadsGroupE;

        repo.getThreads(isFavorite, boardName, offset).then(threads => {
            if (offset === 0) groupE.empty();

            const threadVs = threads.map(thread => new SidebarThreadV(thread, this));
            threadVs.forEach(threadV => this.addToGroup(threadV, groupE));

            if (threadId) this._selectThread(threadId, this._getThreadVsOfGroup(groupE));
        });
    }

    _onSelectBoard(e) {
        const boardName = e.detail.boardName;
        if (this._currentBoardName === boardName) {
            this._deselectAll();
            return;
        }

        this._load(true, boardName, null, 0);
        this._load(false, boardName, null, 0);
    }

    _onSelectThread(e) {
        const boardName = locationService.showAllThreads() ? null : e.detail.boardName;
        const threadId = e.detail.threadId;

        if (this._currentBoardName === boardName) {
            this._selectThread(threadId, this._getThreadsVs());
            return;
        }

        this._load(true, boardName, threadId, 0);
        this._load(false, boardName, threadId, 0);
    }

    _onSelectGlobalPage(e) {
        if (!this._currentBoardName) {
            this._deselectAll();
            return;
        }

        this._load(true, null, null, 0);
        this._load(false, null, null, 0);
    }

    _onSelectSearch(e) {
        this._deselectAll();
    }

    _selectThread(threadId, threadsVs) {
        threadsVs.forEach(threadV => {
            if (threadV.thread.threadId === threadId) {
                threadV.selectableV.select();
            } else {
                threadV.selectableV.deselect();
            }
        });
    }

    _deselectAll() {
        this._getThreadsVs().forEach(threadV => threadV.selectableV.deselect());
    }

    _getThreadsVs() {
        return this._getThreadVsOfGroup(this.favThreadsGroupE).concat(
            this._getThreadVsOfGroup(this.otherThreadsGroupE)
        );
    }

    _getThreadVsOfGroup(groupE) {
        return groupE.children.map(el => el.view);
    }

    _onScroll(e) {
        const groupE = e.target;
        if (groupE.scrollTop < groupE.getMaxScrollTop()) return;

        const isFavorite = groupE === this.favThreadsGroupE;
        const threadsVs = this._getThreadVsOfGroup(groupE);

        this._load(isFavorite, this._currentBoardName, this._currentThreadId, threadsVs.length);
    }

    _onLsChange(e) {
        if (e.detail.isFromForeignWindow && e.detail.key === favoriteThreadsLsKey) {
            let favoriteThreads = getFavoriteThreads();

            this._getThreadsVs().forEach(threadV => {
                const isFavorite = favoriteThreads.some(t => t.threadId === threadV.thread.threadId);

                threadV.toggleFavoriteUiOnly(isFavorite);
            });
        }
    }
}