// file: thread_page_v.js
import {PostV} from "../post/post_v";
import {PaginationV} from "../pagination/pagination_v";
import {ThreadPageLinkF} from "./thread_page_link_f";
import {BoardBannerV} from "../banners/board_banner_v";
import {ThreadBannerV} from "../banners/thread_banner_v";
import {ThreadBannerQuotedSubjD} from "../banners/thread_banner_quoted_subj_d";
import {ReplyFormV} from "../reply_form/reply_form_v";
import {getThreadPageTitle, setTitle} from "../../services/title_service";
import {ThreadsSeparatorV} from "../warp/threads_separator_v";
import {BoardBannerAsLinkD} from "./board_banner_as_link_d";
import {PostWithReplyD} from "../post/post_with_reply_d";
import {PostWithIdD} from "../post/post_with_id_d";
import {config} from "../../config";
import {ReplyFormPositionedD} from "../reply_form/reply_form_positioned_d";
import {getSettings} from "../../services/settings_service";
import {PostTrackNewD} from "../post/post_track_new_d";
import * as repo from "../../repository";
import {enableKatex} from "../../extra_services/katex_service";
import {enableHljs, highlightCode} from "../../extra_services/hljs_service";
import * as trackNewPostsService from "../../services/track_new_posts_service";
import {getThreadPageNumber} from "../../services/page_number_service";
export class ThreadPageV {
    constructor(threadPage) {
        const thread = threadPage.data.thread;
        this._thread = thread;
        this._op = threadPage.data.op;
        this._posts = threadPage.data.posts;
        this._prevPost = threadPage.data.prevPost || {};

        const threadPageE = document.createElement('div');
        threadPageE.classList.add('thread_page');

        this.element = threadPageE;
        threadPageE.threadPageV = this;

        const boardBannerV = new BoardBannerAsLinkD(
            new BoardBannerV(thread.board.name, thread.board.description)
        );
        this._boardBannerV = boardBannerV;
        threadPageE.appendChild(boardBannerV.element);

        const threadsSeparatorV = new ThreadsSeparatorV();
        threadPageE.appendChild(threadsSeparatorV.element);

        const threadBannerV = new ThreadBannerQuotedSubjD(
            new ThreadBannerV(thread.threadId, thread.subject)
        );
        threadPageE.appendChild(threadBannerV.element);

        this.replyFormV = new ReplyFormPositionedD(
            new ReplyFormV(thread.threadId, thread.board.name)
        );

        threadPageE.appendChild(this.replyFormV.element);

        this._loadPosts();

        const pageLinkF = new ThreadPageLinkF(threadPage);
        this._paginationV = new PaginationV(
            threadPage.total - 1, // -1 to exclude OP
            getThreadPageNumber(threadPage.skipped),
            config.pageSize.general,
            config.pagination.siblingCount,
            pageLinkF,
        );
        threadPageE.appendChild(this._paginationV.element);

        setTitle(getThreadPageTitle(threadPage));

        if (getSettings().trackNewPosts) this._updateLastSeenAt();
    }

    onSettingsChange(oldSettings, settings) {
        return (oldSettings.isKatexEnabled !== settings.isKatexEnabled
                ? enableKatex(settings.isKatexEnabled) : Promise.resolve()
        ).then(() =>
            oldSettings.highlightCode !== settings.highlightCode
                ? enableHljs(settings.highlightCode).then() : null
        ).then(() => {
            this.reloadPosts();

            if (settings.highlightCode) highlightCode(this.element);
        });
    }

    onNewPost(post) {
        // ignore posts from other threads
        if (post.thread.threadId !== this._thread.threadId) return;

        this._thread.postCount++;
        this._paginationV.onTotalCountChange(this._thread.postCount - 1); // -1 to skip OP

        // quit if the current page is full
        if (this._posts.length === config.pageSize.general) return;

        // add the post to the current page
        this._posts.push(post);
        const postV = this._createThreadPostV(post, false);

        postV.element.appendAfter((this.postVs.last() || this.originalPostV).element);
        this.postVs.push(postV);
    }

    onUpdatedHiddenByOp(hiddenByOpEvent) {
        this._posts.filter(post => post.postId === hiddenByOpEvent.postId)
            .forEach(post => post.hiddenByOp = hiddenByOpEvent.hiddenByOp);

        this.reloadPosts();
    }

    onBoardDescriptionChange(boardName, boardDescription) {
        if (this._thread.board.name !== boardName) return;

        this._changeBoardDescription(boardDescription)
    }

    reloadPosts() {
        this.element.children.filter(e => e.postV).forEach(e => e.remove());
        this._loadPosts();
    }

    _changeBoardDescription(boardDescription) {
        this._boardBannerV.changeBoardDescription(boardDescription);
    }

    _loadPosts() {
        this.originalPostV = this._createThreadPostV(this._op, true);
        this.originalPostV.element.appendBefore(this.replyFormV.element);

        this.postVs = this._posts.map(post => this._createThreadPostV(post, false));

        this.postVs.forEach(postV => postV.element.appendBefore(this.replyFormV.element));
    }

    _createThreadPostV(post, isOriginalPost) {
        return createThreadPostV(post, isOriginalPost, this.replyFormV, this._thread.lastSeenAt);
    }

    _updateLastSeenAt() {
        const lastSeenAt = this._getLastSeenAt();
        this._thread.lastSeenAt = lastSeenAt;

        // quit if there're new posts on the previous page
        if (lastSeenAt && lastSeenAt < this._prevPost.postDate) return;

        const lastPostV = this.postVs.last();

        if (lastPostV && (!lastSeenAt || lastSeenAt < lastPostV.post.postDate)) {
            // make a request to store lastSeenAt as lastPostV.post.postDate
            repo.updateLastSeenAt(this._thread.threadId, lastPostV.post.postDate);
        }
    }

    _getLastSeenAt() {
        return config.isClientWebUI
            ? this._thread.lastSeenAt
            : trackNewPostsService.getLastSeenAt(this._thread.threadId);
    }
}

export function createThreadPostV(post, isOriginalPost, replyFormV, lastSeenAt) {
    const postV = new PostV(post, isOriginalPost);

    return new PostWithReplyD(
        new PostWithIdD(
            new PostTrackNewD(postV, lastSeenAt)
        ),
        replyFormV
    );
}