// file: new_thread_form_v.js
import {config} from "../../config";
import {TextboxV} from "../textbox_v";
import {isCtrlEnter} from "../../services/shortcut_service";
import * as repo from "../../repository";
import * as locationService from "../../services/location_service";
import {showNotification} from "../notifications/notification_v";
import {NewPost} from "../../models";
import {getAccessToken} from "../../services/access_token_service";
import {isEphemeralPosting, isEphemeralPostingOn} from "../../services/ephemeral_posting_service";
import {addSettingsChangeEventListener, getSettings} from "../../services/settings_service";

export class NewThreadFormV {
    constructor(boardName) {
        const formE = document.createElement('form');
        formE.classList.add('new_thread_form');

        this.element = formE;
        this.replyFormV = this;
        formE.newThreadFormV = this;

        this._boardName = boardName;

        const formBodyE = document.createElement('div');
        formBodyE.classList.add('form_body');
        formE.appendChild(formBodyE);

        const inputsContainerE = document.createElement('div');
        inputsContainerE.classList.add('inputs_container');
        formBodyE.appendChild(inputsContainerE);

        if (!boardName) {
            const boardE = document.createElement('input');
            this.boardE = boardE;
            boardE.id = 'board_name_input';
            boardE.type = 'text';
            boardE.required = true;
            boardE.placeholder = 'Board';
            boardE.title = '1 to 3 alphabetic characters';
            boardE.maxLength = config.maxBoardNameLength;
            boardE.pattern = config.boardNamePattern;
            boardE.classList.add('board');
            inputsContainerE.appendChild(boardE);
        }

        const subjectE = document.createElement('input');
        this.subjectE = subjectE;
        subjectE.id = 'subject_input';
        subjectE.classList.add('subject');
        subjectE.required = true;
        subjectE.placeholder = 'Subject';
        subjectE.maxLength = config.maxSubjectLength;
        inputsContainerE.appendChild(subjectE);

        this.tbxV = new TextboxV();
        this.tbxV.autoResize(false);
        const tbxE = this.tbxV.element;
        tbxE.id = `new_post_tbx`;
        tbxE.classList.add('post_tbx');
        tbxE.maxLength = config.maxPostLength;
        formBodyE.appendChild(tbxE);

        const belowTbxE = document.createElement('div');
        belowTbxE.classList.add('below_tbx');
        formBodyE.appendChild(belowTbxE);

        const submitBtnE = document.createElement('button');
        submitBtnE.classList.add('form_btn');
        submitBtnE.type = 'submit';
        submitBtnE.title = 'Ctrl+Enter';
        submitBtnE.appendText('Submit');

        belowTbxE.appendChild(submitBtnE);

        const stayHereLabelE = document.createElement('label');
        stayHereLabelE.classList.add('stay_here_lbl');

        const stayHereCbxE = document.createElement('input');
        this._stayHereCbxE = stayHereCbxE;
        stayHereCbxE.classList.add('stay_here_cbx');
        stayHereCbxE.type = 'checkbox';
        stayHereLabelE.appendChild(stayHereCbxE);
        stayHereLabelE.appendText(`stay here`);

        belowTbxE.appendChild(stayHereLabelE);

        this.onAuthorizeChange();

        submitBtnE.addEventListener('click', e => {
            if (!this.element.reportValidity()) return;
            e.preventDefault();

            this._onSubmit(e);
        }, false);

        tbxE.addEventListener("keydown", e => {
            if (isCtrlEnter(e)) {
                if (!this.element.reportValidity()) return;
                e.preventDefault();

                this._onSubmit(e);
            }
        }, false);

        this.onEphemeralPostingModeChange();
        addSettingsChangeEventListener(e => {
            if (e.oldSettings.isEphemeralPostingOn !== e.newSettings.isEphemeralPostingOn) {
                this.onEphemeralPostingModeChange();
            }
        });
    }

    onAuthorizeChange() {
        const isAuthorized = getAccessToken();
        this.tbxV.element.placeholder = isAuthorized ? "" : config.ephemeralModeWarning;
        // this.element.elements.forEach(e => e.disabled = !isAuthorized);
    }

    onEphemeralPostingModeChange() {
        this.tbxV.element.placeholder = isEphemeralPostingOn() ? config.ephemeralModeWarning : "";
    }

    _onSubmit(e) {
        const text = this.tbxV.element.value;
        if (!text || !text.trim()) return;

        const subject = this.subjectE.value;
        const boardName = this._boardName || this.boardE.value;
        const stayHere = this._stayHereCbxE.checked;

        const ephemeral = isEphemeralPosting(text);

        if (stayHere || ephemeral) this.element.reset();

        repo.publish(new NewPost(null, text, null, ephemeral, subject, boardName)).then(post => {
            if (stayHere || ephemeral) return;

            locationService.setThreadPage(
                post.thread.board.name, post.thread.threadId, null, post.postId
            );
        }).catch(e => {
            if (e.errorCode === 'E0401') {
                showNotification(config.notification.type.error, `${e.message}. Please log in.`);
                return;
            }

            showNotification(config.notification.type.error, `Message failed to send.`);
        });
    }
}