(function () {
    'use strict';

    // file: utils.js
    window.$ = function (selector) {
        return document.querySelector(selector);
    };
    window.$a = function (selector) {
        return document.querySelectorAll(selector);
    };
    Element.prototype.$ = function (selector) {
        return this.querySelector(selector);
    };
    Element.prototype.$a = function (selector) {
        return this.querySelectorAll(selector);
    };
    NodeList.prototype.each = function (fn) {
        return Array.prototype.forEach.call(this, fn);
    };
    NodeList.prototype.indexOf = function (el) {
        return Array.prototype.indexOf.call(this, el);
    };
    HTMLCollection.prototype.forEach = function (fn) {
        return Array.prototype.forEach.call(this, fn);
    };
    HTMLCollection.prototype.map = function (fn) {
        return Array.prototype.map.call(this, fn);
    };
    HTMLCollection.prototype.filter = function (fn) {
        return Array.prototype.filter.call(this, fn);
    };

    // adds the className if el doesn't have it, and removes it if el does have it.
    Element.prototype.toggleClass = function (className) {
        if (this.classList.contains(className)) {
            this.classList.remove(className);
        } else {
            this.classList.add(className);
        }
    };

    // adds the className if the condition is true and removes it otherwise
    Element.prototype.addOrRemoveClass = function (condition, className) {
        if (condition) {
            this.classList.add(className);
        } else {
            this.classList.remove(className);
        }
    };

    Element.prototype.appendText = function (x) {
        var text = x.toString();
        if (text) this.appendChild(document.createTextNode(text));
        return this;
    };
    Element.prototype.appendAny = function (x) {
        var _this = this;

        if (x instanceof Element) {
            this.appendChild(x);
        } else if (typeof x === 'string') {
            this.appendText(x);
        } else if (Array.isArray(x)) {
            x.forEach(function (item) {
                return _this.appendAny(item);
            });
        } else {
            console.error(this, x);
            throw Error("appendAny: invalid argument");
        }
        return this;
    };
    Element.prototype.appendBefore = function (x) {
        x.parentNode.insertBefore(this, x);
    };
    Element.prototype.appendAfter = function (x) {
        x.parentNode.insertBefore(this, x.nextSibling);
    };

    Element.prototype.prependChild = function (el) {
        if (!this.firstChild) {
            this.appendChild(el);
        } else {
            this.insertBefore(el, this.firstChild);
        }
    };
    Element.prototype.prependText = function (text) {
        this.prependChild(document.createTextNode(text));
    };

    Element.prototype.moveChildNodesTo = function (el) {
        while (this.childNodes.length > 0) {
            el.appendChild(this.childNodes[0]);
        }
    };

    Element.prototype.show = function () {
        this.classList.remove('hidden');
    };
    Element.prototype.hide = function () {
        this.classList.add('hidden');
    };
    Element.prototype.toggleHide = function () {
        this.toggleClass('hidden');
    };
    Element.prototype.empty = function () {
        while (this.lastChild) {
            this.removeChild(this.lastChild);
        }
    };
    Element.prototype.showOrHide = function (condition) {
        if (condition) this.show();else this.hide();
    };
    Element.prototype.isHidden = function () {
        return this.classList.contains('hidden');
    };
    Node.prototype.detach = function () {
        if (this.parentNode) this.parentNode.removeChild(this);
    };

    Element.prototype.scrollToBottom = function () {
        this.scrollTop = this.scrollHeight;
    };
    Element.prototype.scrollToTop = function () {
        return this.scrollTop = 0;
    };
    Element.prototype.getMaxScrollTop = function () {
        return this.scrollHeight - this.clientHeight;
    };

    Element.prototype.replaceChildrenWith = function (x) {
        this.empty();
        this.appendAny(x);
    };

    function parseCssDimension(str) {
        return str === "" ? 0 : parseFloat(str.slice(0, -2));
    }

    function getComputedWidth(el) {
        return parseCssDimension(getComputedStyle(el).width);
    }

    function getComputedHeight(el) {
        return parseCssDimension(getComputedStyle(el).height);
    }

    // export function getComputedVisibleWidth(el) { return (el.isHidden() ? 0 : getComputedWidth(el)); }

    Number.prototype.getNthByte = function (n) {
        var shift = (n - 1) * 8;
        return (this >> shift) % 0x100;
    };

    Number.prototype.toRGB = function () {
        return 'rgb(' + this.getNthByte(3) + ', ' + this.getNthByte(2) + ', ' + this.getNthByte(1) + ')';
    };

    Number.prototype.toHSL = function () {
        var h = (this >> 16) % 360;
        var s = this.getNthByte(2) % 101;
        var l = this.getNthByte(1) % 101;
        return 'hsl(' + h + ', ' + s + '%, ' + l + '%)';
    };

    Promise.prototype.thenA = function (onFulfilled, onRejected) {
        return this.then(function (f) {
            return f(onFulfilled);
        }, onRejected);
    };

    Number.prototype.saveDivide = function (x) {
        return x === 0 ? 0 : this / x;
    };

    function decorate(srcObj, dstObj) {
        var dstObjPrototype = Object.getPrototypeOf(dstObj);

        for (var property in srcObj) {
            if (!srcObj.hasOwnProperty(property)) continue;

            // ignore getters of dstObj
            var descriptor = Object.getOwnPropertyDescriptor(dstObjPrototype, property);
            var isGetter = descriptor && descriptor.get;

            if (!isGetter && !property.startsWith('_') && !dstObj[property]) {
                // console.log(property, srcObj[property]);
                dstObj[property] = srcObj[property];
            }
        }
        // copy methods
        Object.getOwnPropertyNames(Object.getPrototypeOf(srcObj)).forEach(function (property) {
            if (property !== "constructor" && !property.startsWith('_') && !dstObj[property]) {
                // console.log(property, srcObj[property]);
                dstObj[property] = function () {
                    return srcObj[property].apply(srcObj, arguments);
                };
            }
        });
    }

    function decorateView(scrView, dstView, viewName) {
        decorate(scrView, dstView);

        dstView.element = scrView.element;
        dstView.element[viewName] = dstView;
    }

    function range(n) {
        return Array(n).fill();
    }

    Array.prototype.last = function () {
        return this[this.length - 1];
    };

    Array.prototype.skip = function (n) {
        return this.slice(n);
    };

    Array.prototype.take = function (n) {
        return this.slice(0, n);
    };

    Array.prototype.extendWith = function (otherArray) {
        var _this2 = this;

        if (!(otherArray instanceof Array)) {
            console.error(otherArray);
            throw new Error('extend: an array was expected, ' + otherArray + ' given');
        }
        otherArray.forEach(function (x) {
            return _this2.push(x);
        }, this);
    };

    function getNodesRange(fromEl) {
        var toEl = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

        var nodes = [];
        for (var el = fromEl; el && el.previousSibling !== toEl; el = el.nextSibling) {
            nodes.push(el);
        }
        return nodes;
    }

    function tryDecodeURI(encodedURI) {
        try {
            return decodeURI(encodedURI);
        } catch (e) {
            return encodedURI;
        }
    }

    function importScript(src) {
        return new Promise(function (resolve, reject) {
            var scriptE = document.createElement("script");
            scriptE.src = src;
            scriptE.async = true;
            scriptE.onload = resolve;
            scriptE.onerror = reject;
            document.head.appendChild(scriptE);
        });
    }

    function importCss(src) {
        return new Promise(function (resolve, reject) {
            var linkE = document.createElement("link");
            linkE.rel = "stylesheet";
            linkE.href = src;
            linkE.onload = resolve;
            linkE.onerror = reject;
            document.head.appendChild(linkE);
        });
    }

    Object.filter = function (obj, predicate) {
        return Object.keys(obj).filter(function (key) {
            return predicate(obj[key], key);
        }).reduce(function (res, key) {
            res[key] = obj[key];
            return res;
        }, {});
    };

    function textToElement(text) {
        var textE = document.createElement('span');
        text.split('\n').forEach(function (line, idx) {
            if (idx > 0) textE.appendChild(document.createElement('br'));
            textE.appendText(line);
        });
        return textE;
    }

    // quickly create a new element
    function newE(tag, attributes) {
        var element = document.createElement(tag);

        for (var _len = arguments.length, body = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
            body[_key - 2] = arguments[_key];
        }

        if (body) element.appendAny(body);

        Object.keys(attributes).forEach(function (name) {
            return element[name] = attributes[name];
        });

        return element;
    }

    // file: storage_service.js
    function getItem(storage, key, defaultValue) {
        return parseValue(storage.getItem(key), defaultValue);
    }

    function getItems(storage, keyRe) {
        return Object.keys(storage).filter(function (key) {
            return keyRe.test(key);
        }).map(function (key) {
            return { key: key, value: getItem(storage, key) };
        });
    }

    function setItem(storage, key, value) {
        storage.setItem(key, serializeValue(value));
    }

    function toggle(storage, key) {
        setItem(storage, key, !getItem(storage, key, false));
    }

    function removeItem(storage, key) {
        storage.removeItem(key);
    }

    function parseValue(valueStr, defaultValue) {
        return valueStr ? JSON.parse(valueStr) : defaultValue;
    }

    function serializeValue(value) {
        return JSON.stringify(value);
    }

    // file: custom_events.js
    // todo: finish it later
    // const bc = new BroadcastChannel('customEvents');
    // bc.addEventListener('message', e => {
    //     if (e.data.type === 'authorizeChange') {
    //         authorizeChange(true);
    //     }
    // });

    function selectThread(boardName, threadId, pageNumber, postId) {
        var isPopState = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : false;

        document.dispatchEvent(new CustomEvent('selectThread', {
            detail: { boardName: boardName, threadId: threadId, pageNumber: pageNumber, postId: postId, isPopState: isPopState }
        }));
    }

    function selectBoard(boardName, pageNumber) {
        var isPopState = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

        document.dispatchEvent(new CustomEvent('selectBoard', {
            detail: { boardName: boardName, pageNumber: pageNumber, isPopState: isPopState }
        }));
    }

    function selectGlobalPage(pageNumber) {
        var isPopState = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

        document.dispatchEvent(new CustomEvent('selectGlobalPage', {
            detail: { pageNumber: pageNumber, isPopState: isPopState }
        }));
    }

    function selectSearch(searchArgs) {
        var isPopState = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

        document.dispatchEvent(new CustomEvent('selectSearch', {
            detail: { searchArgs: searchArgs, isPopState: isPopState }
        }));
    }

    function setPostIdHash(hash) {
        document.dispatchEvent(new CustomEvent('setPostIdHash', {
            detail: { hash: hash }
        }));
    }

    function chatVisibilityChange(isHidden) {
        document.dispatchEvent(new CustomEvent('chatVisibilityChange', {
            detail: { isHidden: isHidden }
        }));
    }

    function authorizeChange() {

        document.dispatchEvent(new CustomEvent('authorizeChange', {
            detail: {}
        }));
        // if (!isFromForeignWindow) {
        //     bc.postMessage({type: 'authorizeChange', detail: {}});
        // }
    }

    function lsChange(key, oldValue, newValue) {
        var isFromForeignWindow = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;

        document.dispatchEvent(new CustomEvent('lsChange', {
            detail: { key: key, oldValue: oldValue, newValue: newValue, isFromForeignWindow: isFromForeignWindow }
        }));
    }

    // file: config.js
    // we store here parameters that cannot be changed by a user
    var config = {
        version: "0.19.1",
        isClientWebUI: false, // false if this UI directly connects to a server
        maxPostLength: 9000,
        maxSubjectLength: 128,
        maxBoardNameLength: 3,
        maxClientIdLength: 16,
        boardNamePattern: '[a-zA-Z]{1,3}',
        pendingPostId: 'xxxxx',
        basePath: '',
        showIgnoreCyrillicShitFilter: false,
        supportSearching: false, // true if isClientWebUI or if the server supports it
        pageSize: {
            general: 100,
            warp: 20
        },
        formatting: {
            maxNestingLevel: 10
        },
        katex: {
            cssPath: "/static/katex/katex.min.css",
            scriptPath: "/static/katex/katex.min.js"
        },
        hljs: {
            cssPath: "/static/hljs/styles/vs.css",
            scriptPath: "/static/hljs/highlight.pack.js"
        },
        pagination: {
            siblingCount: 3
        },
        sse: {
            type: {
                post: "post",
                connected: "connected",
                boardDescription: "board-description",
                hiddenByOp: "hidden-by-op"
            },
            continueSubscriptionTimeout: 30000, // ms
            reconnectTimeout: 1000, // ms
            broadcastChannel: 'sse'
        },
        notification: {
            type: {
                success: 'success',
                error: 'error',
                warning: 'warning',
                info: 'info'
            },
            autoHideAfter: 2000 // ms
        },
        chat: {
            boardName: 'cht',
            threadId: 'a237b4ad-bdbd-4fa2-9abf-1f646fdcf40a',
            keepScrollBottomThreshold: 50, // px
            updateLastSeenAtTimeout: 2000 // ms
        },
        defaultSettings: {
            isEphemeralPostingOn: false,
            hideEphemeralPosts: false,
            showOnlyChatPostsInChat: false,
            displayHiddenPostsInChat: true,
            ignoreCyrillicShit: false,
            isKatexEnabled: false,
            highlightCode: false,
            showImages: true,
            embedBitchute: false,
            trackNewPosts: true,
            isCompactTheme: true,
            maxEntropy: 6,
            regexpFilters: '://imagetwist.com\n://cuts-url.com\n',
            colorsFilters: ''
        },
        tabHeartbeatTimeout: 500, // ms
        postingLockedMessage: "Log in to unlock posting.",
        ephemeralModeWarning: "Warning: ephemeral posting mode is on."
    };

    // file: local_storage_service.js

    // because of this bug https://bugzilla.mozilla.org/show_bug.cgi?id=1453699
    // which affects Tor Browser users
    // we need a temporary workaround: we ask other open tabs to send their localStorage to this tab
    // if we don't receive anything during 20ms we assume that there're no other open tabs.
    // we use fake localStorage object and synchronize it across tabs using BroadcastChannel.
    // When this bug is fixed in Tor Browser, this code will be removed.
    var isLsBugWorkaroundNeeded =  (navigator.userAgent === "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:68.0) Gecko/20100101 Firefox/68.0" || navigator.userAgent === "Mozilla/5.0 (Windows NT 10.0; rv:68.0) Gecko/20100101 Firefox/68.0" || navigator.userAgent === "Mozilla/5.0 (Windows NT 6.1; rv:60.0) Gecko/20100101 Firefox/60.0") || navigator.userAgent === "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:78.0) Gecko/20100101 Firefox/78.0";
    var wa = {}; // workaround

    function initLocalStorageService(callback) {
        var onInitialized = function onInitialized() {
            addStorageEventListener();
            callback();
        };

        if (isLsBugWorkaroundNeeded) {
            wa.fakeLocalStorage = {};
            Object.keys(localStorage).forEach(function (key) {
                return wa.fakeLocalStorage[key] = localStorage[key];
            });

            wa.bc = new BroadcastChannel('ls');
            wa.bc.postMessage({ type: 'send your ls' });

            var isLsReceived = false;
            wa.bc.addEventListener('message', function (e) {
                if (e.data.type === 'send your ls') {
                    wa.bc.postMessage({ type: 'ls', localStorage: wa.fakeLocalStorage });
                }

                if (e.data.type === 'ls') {
                    if (isLsReceived) return;

                    isLsReceived = true;
                    var receivedLs = e.data.localStorage;
                    Object.keys(receivedLs).forEach(function (key) {
                        if (receivedLs[key] !== undefined) {
                            wa.fakeLocalStorage[key] = receivedLs[key];
                            localStorage[key] = receivedLs[key];
                        }
                    });
                    onInitialized();
                }

                // storage event emulation
                if (e.data.type === 'storage') {
                    if (e.data.newValue === undefined) {
                        delete wa.fakeLocalStorage[e.data.key];
                        delete localStorage[e.data.key];
                    } else {
                        wa.fakeLocalStorage[e.data.key] = e.data.newValue;
                        localStorage[e.data.key] = e.data.newValue;
                    }
                    onStorageHandler(e.data);
                }
            });

            setTimeout(function () {
                if (!isLsReceived) {
                    console.log('timeout');
                    isLsReceived = true;
                    onInitialized();
                }
            }, 20);
        } else {
            onInitialized();
        }
    }

    function getItem$1(key, defaultValue) {
        if (isLsBugWorkaroundNeeded) {
            return parseValue(wa.fakeLocalStorage[key], defaultValue);
        }

        return getItem(localStorage, key, defaultValue);
    }

    function getItems$1(keyRe) {
        if (isLsBugWorkaroundNeeded) {
            return Object.keys(wa.fakeLocalStorage).filter(function (key) {
                return keyRe.test(key);
            }).map(function (key) {
                return { key: key, value: getItem$1(key) };
            });
        }

        return getItems(localStorage, keyRe);
    }

    function setItem$1(key, value, defaultValue) {
        var isImport = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;

        var oldValue = getItem$1(key, defaultValue);

        if (isLsBugWorkaroundNeeded) {
            var oldValueStr = serializeValue(oldValue);
            var newValueStr = serializeValue(value);
            wa.fakeLocalStorage[key] = newValueStr;

            if (oldValueStr !== newValueStr) {
                // console.log('send newVal', newValueStr);
                wa.bc.postMessage({
                    type: 'storage', key: key,
                    oldValue: oldValueStr,
                    newValue: newValueStr
                });
            }
        }

        setItem(localStorage, key, value);

        // todo: this won't work for objects
        if (oldValue !== value) lsChange(key, oldValue, value, isImport);
    }

    function toggle$1(key) {
        var defaultValue = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

        var oldValue = getItem$1(key, defaultValue);

        if (isLsBugWorkaroundNeeded) {
            var oldValueStr = serializeValue(oldValue);
            var newValueStr = serializeValue(!getItem$1(key, false));
            wa.fakeLocalStorage[key] = newValueStr;

            wa.bc.postMessage({
                type: 'storage', key: key,
                oldValue: oldValueStr,
                newValue: newValueStr
            });
        }

        toggle(localStorage, key);

        lsChange(key, oldValue, !oldValue);
    }

    function removeItem$1(key) {
        if (isLsBugWorkaroundNeeded) {
            var oldValueStr = wa.fakeLocalStorage[key];
            delete wa.fakeLocalStorage[oldValueStr];

            wa.bc.postMessage({
                type: 'storage', key: key,
                oldValue: oldValueStr
            });
        }

        removeItem(localStorage, key);
    }

    function addStorageEventListener() {
        if (isLsBugWorkaroundNeeded) return;

        window.addEventListener('storage', onStorageHandler, false);
    }

    function onStorageHandler(e) {
        // console.log('received newVal', e.newValue);
        // console.log('onStorage', e.key, e.oldValue, e.newValue); // todo: delete
        var oldValue = parseValue(e.oldValue, null);
        var newValue = parseValue(e.newValue, null);

        lsChange(e.key, oldValue, newValue, true);
    }

    // file: views.js

    function enableToggles() {
        $a(".toggle").each(function (toggleE) {
            var toggleCollapsedLsKey = toggleE.id + "_collapsed";

            toggleE.addOrRemoveClass(getItem$1(toggleCollapsedLsKey), 'collapsed');

            toggleE.addEventListener("click", function () {
                toggleE.toggleClass('collapsed');
                toggle$1(toggleCollapsedLsKey);
            }, false);

            var _onLsChangeHandler = void 0;
            document.addEventListener('lsChange', _onLsChangeHandler = function _onLsChangeHandler(e) {
                if (e.detail.isFromForeignWindow && e.detail.key === toggleCollapsedLsKey) {
                    e.detail.newValue ? toggleE.classList.add('collapsed') : toggleE.classList.remove('collapsed');
                }
            }, false);
        });
    }

    var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var DragDependantV = function () {
        function DragDependantV(element) {
            _classCallCheck(this, DragDependantV);

            this.element = element;
            this.element.dragDependantV = this;
        }

        _createClass(DragDependantV, [{
            key: 'setWidth',
            value: function setWidth(width) {
                this.element.style.width = width + 'px';
                this.element.style.minWidth = width + 'px';
            }
        }, {
            key: 'setHeight',
            value: function setHeight(height) {
                this.element.style.height = height + 'px';
                this.element.style.minHeight = height + 'px';
            }
        }, {
            key: 'toggleHide',
            value: function toggleHide() {
                this.element.toggleClass('hidden');
            }
        }]);

        return DragDependantV;
    }();

    function _classCallCheck$1(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var WidthStoredDragDependantD = function WidthStoredDragDependantD(dragDependantV, localStorageWidthKey, defaultWidth) {
        var _this = this;

        _classCallCheck$1(this, WidthStoredDragDependantD);

        decorateView(dragDependantV, this, 'dragDependantV');

        this.setWidth = function (width) {
            _this._width = width;
            dragDependantV.setWidth(width);
        };

        this.setWidth(getItem$1(localStorageWidthKey, defaultWidth));

        window.addEventListener('mouseup', function (e) {
            setItem$1(localStorageWidthKey, _this._width);
        }, false);

        document.addEventListener('lsChange', this._onLsChangeHandler = function (e) {
            if (e.detail.isFromForeignWindow && e.detail.key === localStorageWidthKey) {
                _this.setWidth(e.detail.newValue);
            }
        }, false);
    };

    function _classCallCheck$2(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var VisibilityStoredDragDependantD = function VisibilityStoredDragDependantD(dragDependantV, localStorageIsHiddenKey, isHiddenByDefault) {
        _classCallCheck$2(this, VisibilityStoredDragDependantD);

        decorateView(dragDependantV, this, 'dragDependantV');

        setItem$1(localStorageIsHiddenKey, getItem$1(localStorageIsHiddenKey, isHiddenByDefault));

        if (getItem$1(localStorageIsHiddenKey, isHiddenByDefault) !== isHiddenByDefault) {
            // if stored visibility state is different from the default value
            dragDependantV.toggleHide();
        }

        this.toggleHide = function () {
            dragDependantV.toggleHide();
            toggle$1(localStorageIsHiddenKey);
        };

        document.addEventListener('lsChange', this._onLsChangeHandler = function (e) {
            if (e.detail.isFromForeignWindow && e.detail.key === localStorageIsHiddenKey) {
                dragDependantV.toggleHide();
            }
        }, false);
    };

    var _createClass$1 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$3(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    // decorates DragDependantV for sidebars
    var SidebarDragDependantD = function () {
        function SidebarDragDependantD(dragDependantV) {
            _classCallCheck$3(this, SidebarDragDependantD);

            decorateView(dragDependantV, this, 'dragDependantV');

            this.element = dragDependantV.element;
            this.element.dragDependantV = this;

            this._dragDependantV = dragDependantV;
            this._containerE = $('#container');

            var boardsSidebarE = $('#boards_sidebar');
            var threadsSidebarE = $('#threads_sidebar');
            this._isBoardsSidebar = this.element === boardsSidebarE;
            this._otherSidebarE = this._isBoardsSidebar ? threadsSidebarE : boardsSidebarE;
        }

        _createClass$1(SidebarDragDependantD, [{
            key: 'setWidth',
            value: function setWidth(width) {
                var currentWidth = this._getWidth(this.element);
                var availableWidth = this._getAvailableWidth();
                var delta = currentWidth - width;

                // if there will be no space left
                if (availableWidth + delta < 0) {
                    // This is left here just in case somebody will cry about such feature.

                    // const otherSidebarWidth = getComputedWidth(this._otherSidebarE);
                    // // try to use other sidebar's width if it can be cut off yet
                    // if (this._isBoardsSidebar && otherSidebarWidth + delta >= 0) {
                    //     this._otherSidebarE.view.setWidth(otherSidebarWidth + delta);
                    // } else {
                    width = currentWidth + availableWidth;
                    // }
                }

                this._dragDependantV.setWidth(width);
            }
        }, {
            key: 'toggleHide',
            value: function toggleHide() {
                // in case of showing
                var totalWidth = this._getAvailableWidth() + this._getWidth(this._otherSidebarE);
                // console.log(getComputedWidth(this.element), totalWidth);
                if (this.element.isHidden() && getComputedWidth(this.element) > totalWidth) {
                    this._dragDependantV.setWidth(totalWidth);
                }

                this._dragDependantV.toggleHide();

                if (!this._otherSidebarE.isHidden() && this._getAvailableWidth() === 0) {
                    this._otherSidebarE.dragDependantV.setWidth(totalWidth - getComputedWidth(this.element));
                }
            }
        }, {
            key: '_getAvailableWidth',
            value: function _getAvailableWidth() {
                // return getComputedWidth(this._containerE);
                return this._containerE.offsetWidth;
            }
        }, {
            key: '_getWidth',
            value: function _getWidth(element) {
                // NOTE: if the element is hidden, use getComputedWidth() if you don't want 0.
                return element.offsetWidth;
            }
        }]);

        return SidebarDragDependantD;
    }();

    function _classCallCheck$4(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var Board = function Board(name, description, isFavorite) {
        _classCallCheck$4(this, Board);

        this.name = name;
        this.description = description;
        this.isFavorite = isFavorite;
    };

    var Thread = function Thread(threadId, subject, postCount, lastPostDate, board, isFavorite, lastSeenAt) {
        _classCallCheck$4(this, Thread);

        this.threadId = threadId;
        this.subject = subject;
        this.postCount = postCount;
        this.lastPostDate = lastPostDate;
        this.board = board;
        this.isFavorite = isFavorite;
        this.lastSeenAt = lastSeenAt;
    };

    var Post = function Post(postId, postDate, bg, fg, text, hiddenByOp, user, thread, index, ephemeral) {
        _classCallCheck$4(this, Post);

        this.postId = postId;
        this.postDate = postDate;
        this.bg = bg;
        this.fg = fg;
        this.text = text;
        this.hiddenByOp = hiddenByOp;
        this.user = user;
        this.thread = thread;
        this.index = index; // the index of the post in the thread
        this.ephemeral = ephemeral;

        if (this.ephemeral && this.thread && this.thread.threadId === config.chat.threadId) {
            this.thread.board.name = config.chat.boardName;
        }
    };

    var ServerInfo = function ServerInfo(version, powBits) {
        _classCallCheck$4(this, ServerInfo);

        this.version = version;
        this.powBits = powBits;
    };

    var ThreadSlice = function ThreadSlice(thread, op, posts, prevPost) {
        _classCallCheck$4(this, ThreadSlice);

        this.thread = thread;
        this.op = op;
        this.posts = posts;
        this.prevPost = prevPost;
    };

    var WarpSlice = function WarpSlice(board, shellThreads) {
        _classCallCheck$4(this, WarpSlice);

        this.board = board;
        this.shellThreads = shellThreads; // thread slices with less than 3 posts
    };

    var RecentPosts = function RecentPosts(chatThread, posts) {
        _classCallCheck$4(this, RecentPosts);

        this.chatThread = chatThread;
        this.posts = posts;
    };

    var ShortUserInfo = function ShortUserInfo(nickname, banned) {
        _classCallCheck$4(this, ShortUserInfo);

        this.nickname = nickname;
        this.banned = banned;
    };

    var NewPost = function NewPost(clientId, text, threadId, ephemeral, subject, boardName) {
        _classCallCheck$4(this, NewPost);

        this.clientId = clientId;
        this.text = text;
        this.threadId = threadId;
        this.subject = subject;
        this.boardName = boardName;
        this.ephemeral = ephemeral;
    };

    var NewUser = function NewUser(nickname, password, pow) {
        _classCallCheck$4(this, NewUser);

        this.nickname = nickname;
        this.password = password;
        this.pow = pow;
    };

    var Auth = function Auth(nickname, password) {
        _classCallCheck$4(this, Auth);

        this.nickname = nickname;
        this.password = password;
    };

    var AuthInfo = function AuthInfo(accessToken) {
        _classCallCheck$4(this, AuthInfo);

        this.accessToken = accessToken;
    };

    var Claims = function Claims(name, iat) {
        _classCallCheck$4(this, Claims);

        this.name = name;
        this.iat = iat;
    };

    var UpdatedHiddenByOp = function UpdatedHiddenByOp(clientId, postId, hiddenByOp) {
        _classCallCheck$4(this, UpdatedHiddenByOp);

        this.clientId = clientId;
        this.postId = postId;
        this.hiddenByOp = hiddenByOp;
    };

    var HiddenByOpEvent = function HiddenByOpEvent(postId, hiddenByOp) {
        _classCallCheck$4(this, HiddenByOpEvent);

        this.postId = postId;
        this.hiddenByOp = hiddenByOp;
    };

    var SuccessResponse = function SuccessResponse(data, skipped, taken, total) {
        _classCallCheck$4(this, SuccessResponse);

        this.data = data;
        this.skipped = skipped;
        this.taken = taken;
        this.total = total;
    };

    var RequestBody = function RequestBody(data) {
        _classCallCheck$4(this, RequestBody);

        this.data = data;
    };

    var ErrorResponse = function ErrorResponse(errorCode, message) {
        _classCallCheck$4(this, ErrorResponse);

        this.errorCode = errorCode;
        this.message = message;
    };

    var SearchArgs = function SearchArgs(query, range, orderDesc, boards, pageNumber) {
        _classCallCheck$4(this, SearchArgs);

        this.query = query;
        this.range = range;
        this.orderDesc = orderDesc;
        this.boards = boards;
        this.pageNumber = pageNumber;
    };

    // file: url_service.js

    var api =  "s_api";
    var prefix = config.basePath;

    function getRecentPostsApiUrl(threadId) {
        return "/" + api + "/posts?" + toQuery({
            includeThread: true,
            includeBoard: true,
            sortByDate: "desc",
            thread: threadId
        });
    }

    function getPostsBeforeApiUrl(postId, threadId) {
        return "/" + api + "/posts?" + toQuery({
            includeThread: true,
            includeBoard: true,
            sortByDate: "desc",
            thread: threadId,
            beforePost: postId
        });
    }

    function getPostsAfterApiUrl(postId, threadId) {
        return "/" + api + "/posts?" + toQuery({
            includeThread: true,
            includeBoard: true,
            sortByDate: "asc",
            thread: threadId,
            afterPost: postId
        });
    }

    function getPostApiUrl(postId, includeIndex) {
        return "/" + api + "/posts/" + postId + "?" + toQuery({
            includeThread: true,
            includeBoard: true,
            includeIndex: includeIndex
        });
    }

    function getBoardsApiUrl(isFavorite, skip) {
        return "/" + api + "/boards?" + toQuery({
            skip: skip,
            isFavorite: isFavorite
        });
    }

    function getThreadsApiUrl(isFavorite, boardName, skip) {
        return "/" + api + "/threads?" + toQuery({
            board: boardName,
            includeBoard: true,
            skip: skip,
            isFavorite: isFavorite
        });
    }

    function getBoardUrl(boardName) {
        return prefix + "/" + boardName + "/";
    }

    function getThreadUrl(boardName, threadId) {
        var pageNumber = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
        var postId = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : null;

        var path = prefix + "/" + boardName + "/" + threadId + "/";

        var pathAndQuery = pageNumber === null ? path : path + "?" + toQuery({ p: pageNumber });

        return postId === null ? pathAndQuery : pathAndQuery + "#p" + postId;
    }

    function getWarpUrl(boardName, pageNumber) {
        return prefix + ((boardName ? "/" + boardName + "/" : "/") + (pageNumber ? "?" + toQuery({ p: pageNumber }) : ""));
    }

    function getHomeUrl() {
        return prefix + "/";
    }

    function getPostLinkUrl(postId) {
        return prefix + "/@" + postId;
    }

    function getThreadPageApiUrl(threadId, pageNumber) {
        // null means the last page
        // + 1 because we want to skip OP

        return "/" + api + "/threadSlices/" + threadId + "?" + toQuery({
            skip: (pageNumber === null ? 0 : pageNumber * config.pageSize.general) + 1,
            lastSlice: pageNumber === null,
            includeBoard: true
        });
    }

    function getWarpPageApiUrl(boardName, pageNumber) {
        return "/" + api + "/warpSlices?" + toQuery({
            board: boardName,
            skip: pageNumber * config.pageSize.warp,
            includeBoard: !boardName
        });
    }

    function getPublishApiUrl() {
        return "/" + api + "/posts";
    }

    function getUpdateHiddenByOpApiUrl() {
        return "/" + api + "/posts";
    }

    function getUpdateBoardDescriptionApiUrl() {
        return "/" + api + "/boards";
    }

    function getSubscribeApiUrl() {
        return "/" + api + "/subscribe";
    }

    function getContinueSubscriptionApiUrl(subscriptionId) {
        return "/" + api + "/continueSubscription/" + subscriptionId;
    }

    function getAuthorizeApiUrl() {
        return "/" + api + "/authorize";
    }

    function getCreateUserApiUrl() {
        return "/" + api + "/users";
    }

    function getSearchBaseUrl() {
        return prefix + "/search";
    }

    function getSearchUrl(searchArgs) {
        if (!searchArgs) return getSearchBaseUrl();

        return getSearchBaseUrl() + "?" + toQuery({
            q: searchArgs.query,
            range: searchArgs.range,
            order_desc: searchArgs.orderDesc,
            boards: searchArgs.boards,
            p: searchArgs.pageNumber
        });
    }

    function getServerInfoApiUrl() {
        return "/" + api + "/serverInfo";
    }

    function getSearchApiUrl(searchArgs) {
        return "/" + api + getSearchUrl(searchArgs);
    }

    function parseSearchArgs(queryStrings) {
        if (!queryStrings.q) return null;

        return new SearchArgs(queryStrings.q, queryStrings.range, queryStrings.order_desc ? JSON.parse(queryStrings.order_desc) : null, queryStrings.boards, queryStrings.p ? parseInt(queryStrings.p) : null);
    }

    function toQuery(obj) {
        return Object.keys(obj).filter(function (k) {
            return obj[k] !== null && obj[k] !== undefined;
        }).map(function (k) {
            return encodeURIComponent(k) + "=" + encodeURIComponent(obj[k]);
        }).join('&');
    }

    // file: uuid_service.js
    function toSimpleUuid(x) {
        return x.replace(/-/g, "").toLowerCase();
    }

    function toShortcutUuid(uuid) {
        return uuid.substring(0, 5).toLowerCase();
    }

    function toHyphenatedUuid(uuid) {
        uuid = toSimpleUuid(uuid);
        return [uuid.substring(0, 8), uuid.substring(8, 12), uuid.substring(12, 16), uuid.substring(16, 20), uuid.substring(20, 32)].join("-");
    }

    var hyphenatedUuidPattern = '[\\da-f]{8}-[\\da-f]{4}-4[\\da-f]{3}-[89ab][\\da-f]{3}-[\\da-f]{12}';
    var simpleUuidPattern = '[\\da-f]{8}[\\da-f]{4}4[\\da-f]{3}[89ab][\\da-f]{3}[\\da-f]{12}';
    var uuidPattern = "(" + simpleUuidPattern + "|" + hyphenatedUuidPattern + ")";

    // file: text_parsing_service.js

    function parsePostReference(text) {
        if (!text.startsWith('@')) return null;

        var uuid = parseUuid(text.slice(1));
        if (!uuid) return null;
        uuid.match = toHyphenatedUuid(uuid.match);
        return uuid;
    }

    function parseUuid(text) {
        var simpleUuid = parseSimpleUuid(text);
        if (simpleUuid) return simpleUuid;

        var hyphenatedUuid = parseHyphenatedUuid(text);
        if (hyphenatedUuid) return hyphenatedUuid;

        return null;
    }

    function parseSimpleUuid(text) {
        var simpleUuidRe = /^[\da-f]{8}[\da-f]{4}4[\da-f]{3}[89ab][\da-f]{3}[\da-f]{12}/i;
        return parseByRe(simpleUuidRe, text);
    }

    function parseHyphenatedUuid(text) {
        var hyphenatedUuidRe = /^[\da-f]{8}-[\da-f]{4}-4[\da-f]{3}-[89ab][\da-f]{3}-[\da-f]{12}/i;
        return parseByRe(hyphenatedUuidRe, text);
    }

    function parseByRe(re, text) {
        var match = re.exec(text);
        if (!match) return null;
        return { match: match[0], rest: text.slice(match[0].length) };
    }

    function parserNewline(text) {
        if (text[0] === '\n') {
            return { rest: text.slice(1) };
        }
        return null;
    }

    function parseQuote(text, markupFreePrefix, element, level) {
        if (level !== 0 || text[0] !== '>') return null;
        // quit if the prefix contains non-whitespace chars
        if (/\S/.test(markupFreePrefix)) return null;

        var prevE = element.lastChild;
        var afterPostRef = prevE && prevE.className === 'post_ref';
        if (prevE && prevE.tagName !== 'BR' && !afterPostRef) {
            return;
        }
        var idx = text.indexOf('\n', 1);
        return {
            match: idx < 0 ? text : text.slice(0, idx),
            rest: idx < 0 ? '' : text.slice(idx),
            afterPostRef: afterPostRef
        };
    }

    function parseNamedLink(text) {
        var match = /^\[([^\n\r\]]+)\]\(([^\)]+)\)/.exec(text);
        if (!match) return null;
        var name = match[1];
        // const urlMatch = urlRe.exec(match[2]);
        // if (!urlMatch) return null;
        // const url = urlMatch[0];
        var url = match[2];
        var lengthToSkip = match[0].length;
        return { match: url, name: name, rest: text.slice(lengthToSkip) };
    }

    function parseLink(text) {
        var match = urlRe.exec(text);
        if (!match) return null;
        var url = match[0];
        return { match: url, rest: text.slice(url.length) };
    }

    function parseBold(text) {
        return parseSimpleMarkup("**", text);
    }
    function parseItalic(text) {
        return parseSimpleMarkup("//", text);
    }
    function parseMono(text) {
        return parseSimpleMarkup("`", text);
    }
    function parseStrike(text) {
        return parseSimpleMarkup("~~", text);
    }
    function parseTeX(text) {
        return parseSimpleMarkup("$", text);
    }
    function parseSpoiler(text) {
        return parseSimpleMarkup("%%", text);
    }
    function parseCode(text) {
        var code = parseSimpleMarkup("```", text);
        if (!code) return code;
        var langMatch = /^[^\s]*/.exec(code.match);
        if (!langMatch) return code;
        code.lang = langMatch[0];
        code.match = code.match.slice(code.lang.length).replace(/^\s/, "");
        return code;
    }

    function parseSimpleMarkup(markup, text) {
        if (!text.startsWith(markup)) return null;
        var idx = text.indexOf(markup, markup.length);
        if (idx < 0) return null;
        var match = text.substring(markup.length, idx);
        return { match: match, rest: text.slice(idx + markup.length) };
    }

    var urlRe = /^(?:(?:(?:https?|ftp):\/\/)(?:\S+(?::\S*)?@)?(?:(?!10(?:\.\d{1,3}){3})(?!127(?:\.\d{1,3}){3})(?!169\.254(?:\.\d{1,3}){2})(?!192\.168(?:\.\d{1,3}){2})(?!172\.(?:1[6-9]|2\d|3[0-1])(?:\.\d{1,3}){2})(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))|(?:(?:[a-z\u00a1-\uffff0-9]+-?)*[a-z\u00a1-\uffff0-9]+)(?:\.(?:[a-z\u00a1-\uffff0-9]+-?)*[a-z\u00a1-\uffff0-9]+)*(?:\.(?:[a-z\u00a1-\uffff0-9]{2,})))(?::\d{2,5})?(?:\/[^\s]*)?|magnet:\?(?:[^\s\]\)]+))/;

    function getPostReferences(text) {
        var ids = [];

        for (var i = 0; i < text.length; i++) {
            var textToParse = text.slice(i);

            var postReference = parsePostReference(textToParse);
            if (postReference) {
                ids.push(postReference.match);
            }
        }
        return ids;
    }

    // file: location_service.js

    function getHref() {
        return location.href;
    }

    function getPathName() {
        return location.pathname.substring(config.basePath.length);
    }

    function getHrefWithoutHost() {
        return getPathName() + location.search + location.hash;
    }

    function getPathParts() {
        return getPathName().split('/');
    }

    function showAllThreads() {
        return "allt" in getQueryStrings() || !getSelectedBoardName();
    }

    function isSearchPage() {
        return location.pathname.startsWith(getSearchBaseUrl());
    }

    function getSearchArgs() {
        return parseSearchArgs(getQueryStrings());
    }

    function getSelectedBoardName() {
        var name = getPathParts()[1].toLowerCase();
        return (/^[a-z]{1,3}$/.test(name) ? name : null
        );
    }

    function getSelectedThreadId() {
        if (!getSelectedBoardName()) return null;

        return getThreadId();
    }

    function getThreadId() {
        var threadId = parseUuid(getPathParts()[2]);
        return threadId ? threadId.match : null;
    }

    function getPage() {
        var pageNumber = parseInt(getQueryStrings().p);
        return isNaN(pageNumber) ? null : pageNumber;
    }

    function getPostId() {
        var match = /(@.+)/.exec(getPathName());
        if (!match) return null;
        return parsePostReference(match[1]).match;
    }

    function getPostIdAnchor() {
        var postId = parseUuid(location.hash.slice(2));
        return postId ? postId.match : null;
    }

    function activateTarget() {
        if (location.hash) {
            // noinspection SillyAssignmentJS
            location.hash = location.hash;
        }
    }

    function setPostIdHash$1(hash) {
        pushEmptyState(location.pathname + location.search + hash);

        setPostIdHash(hash);
        activateTarget();
    }

    function setHome() {
        pushEmptyState(getHomeUrl());
        selectGlobalPage(0);
    }

    function setBoard(boardName) {
        setWarp(boardName, 0);
    }

    function setWarp(boardName, pageNumber) {
        pushEmptyState(getWarpUrl(boardName, pageNumber));

        selectBoard(boardName, pageNumber);
    }

    function setThread(boardName, threadId) {
        var queryString = showAllThreads() ? "?allt" : "";
        var url = getThreadUrl(boardName, threadId) + queryString;

        pushEmptyState(url);

        selectThread(boardName, threadId, null);
    }

    function setThreadPage(boardName, threadId, pageNumber) {
        var postId = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : null;

        var url = getThreadUrl(boardName, threadId, pageNumber) + (showAllThreads() ? (pageNumber !== null ? '&' : '?') + "allt" : "") + (postId === null ? "" : "#p" + postId);

        pushEmptyState(url);

        selectThread(boardName, threadId, pageNumber, postId);
    }

    function setSearch(searchArgs) {
        pushEmptyState(getSearchUrl(searchArgs));
        selectSearch(searchArgs);
    }

    function replaceByThreadPage(boardName, threadId, pageNumber, postId) {
        var url = getThreadUrl(boardName, threadId, pageNumber, postId);
        replaceEmptyState(url);
    }

    function pushEmptyState(url) {
        // do not push the same state twice
        if (url === getHrefWithoutHost()) return;

        history.pushState({}, "", url);
    }

    function replaceEmptyState(url) {
        history.replaceState({}, "", url);
    }

    function getQueryStrings() {
        var result = {};
        location.search.substring(1).split('&').forEach(function (x) {
            var idx = x.indexOf('=');
            if (idx > 0) {
                result[decodeURIComponent(x.substring(0, idx))] = decodeURIComponent(x.substring(idx + 1));
            } else {
                result[decodeURIComponent(x)] = null;
            }
        });
        return result;
    }

    var _createClass$2 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$5(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    // file: selectable_v.js
    var SelectableV = function () {
        function SelectableV(element) {
            _classCallCheck$5(this, SelectableV);

            this.element = element;
            element.selectableV = this;
        }

        _createClass$2(SelectableV, [{
            key: 'select',
            value: function select() {
                this.element.classList.add('selected');
            }
        }, {
            key: 'deselect',
            value: function deselect() {
                this.element.classList.remove('selected');
            }
        }]);

        return SelectableV;
    }();

    // file: parsers.js

    function parseBoard(obj) {
        return new Board(obj.name, obj.description, obj.isFavorite);
    }

    function parseThread(obj) {
        return new Thread(obj.threadId, obj.subject, obj.postCount, obj.lastPostDate ? new Date(obj.lastPostDate) : null, obj.board ? parseBoard(obj.board) : null, obj.isFavorite, obj.lastSeenAt);
    }

    function parsePost(obj) {
        return new Post(obj.postId, new Date(obj.postDate), obj.bg, obj.fg, obj.text, obj.hiddenByOp, parseShortUserInfo(obj.user), obj.thread ? parseThread(obj.thread) : null, obj.index, obj.ephemeral);
    }

    function parseServerInfo(obj) {
        return new ServerInfo(obj.version, obj.powBits);
    }

    function parseThreadSlice(obj) {
        return new ThreadSlice(parseThread(obj.thread), parsePost(obj.op), obj.posts.map(parsePost), obj.prevPost ? parsePost(obj.prevPost) : null);
    }

    function parseWarpSlice(obj) {
        return new WarpSlice(obj.board ? parseBoard(obj.board) : null, obj.shellThreads.map(parseThreadSlice));
    }

    function parseShortUserInfo(obj) {
        return new ShortUserInfo(obj.nickname, obj.banned);
    }

    function parseAuthInfo(obj) {
        return new AuthInfo(obj.accessToken);
    }

    function parseClaims(accessToken) {
        var obj = JSON.parse(atob(accessToken.split('.')[1]));
        return new Claims(obj.name, obj.iat);
    }

    function parseSuccessResponse(obj) {
        return function (dataParser) {
            return new SuccessResponse(dataParser(obj.data), obj.skipped, obj.taken, obj.total);
        };
    }

    function parseErrorResponse(obj) {
        return new ErrorResponse(obj.errorCode, obj.message);
    }

    function parseHiddenByOpEvent(obj) {
        return new HiddenByOpEvent(obj.postId, obj.hiddenByOp);
    }

    // file: cookie_service.js
    function setCookie(name, value, maxAge) {
        var path = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : "/";

        document.cookie = [name + "=" + value, "path=" + path, maxAge ? "max-age=" + maxAge : "", "samesite=strict"].join("; ");
    }

    function getCookie(name) {
        return document.cookie.split(";").map(function (kv) {
            return kv.trim().split('=');
        }).filter(function (kv) {
            return kv[0] === name;
        }).map(function (kv) {
            return kv[1];
        })[0];
    }

    function deleteCookie(name) {
        var path = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "/";

        setCookie(name, '', '-1', path);
    }

    // file: access_token_service.js

    var cookieKey = "accessToken";
    var path = "/";

    function setAccessToken(accessToken) {
        setCookie(cookieKey, accessToken, null, path);
    }

    function deleteAccessToken() {
        deleteCookie(cookieKey, path);
    }

    function getAccessToken() {
        return getCookie(cookieKey);
    }

    function getClaims() {
        return parseClaims(getAccessToken());
    }

    var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

    function getRecentPosts(threadId) {
        return getJson(getRecentPostsApiUrl(threadId)).thenA(function (obj) {
            return  new RecentPosts(null, obj.map(parsePost).reverse());
        });
    }

    function getPostsBefore(postId, threadId) {
        return getJson(getPostsBeforeApiUrl(postId, threadId)).thenA(function (ps) {
            return ps.map(parsePost).reverse();
        });
    }

    function getPostsAfter(postId, threadId) {
        return getJson(getPostsAfterApiUrl(postId, threadId)).thenA(function (ps) {
            return ps.map(parsePost);
        });
    }

    function getBoards(isFavorite, skip) {
        return getJson(getBoardsApiUrl(isFavorite, skip)).thenA(function (bs) {
            return bs.map(parseBoard);
        });
    }

    function getThreads(isFavorite, boardName, skip) {
        return getJson(getThreadsApiUrl(isFavorite, boardName, skip)).thenA(function (ts) {
            return ts.map(parseThread);
        });
    }

    function getPost(postId, includeIndex) {
        return getJson(getPostApiUrl(postId, includeIndex)).thenA(parsePost);
    }

    function getThreadPage(threadId, pageNumber) {
        // null means the last page

        return getJson(getThreadPageApiUrl(threadId, pageNumber)).thenA(parseThreadSlice);
    }

    function getWarpPage(boardName, pageNumber) {
        return getJson(getWarpPageApiUrl(boardName, pageNumber)).thenA(parseWarpSlice);
    }

    function getServerInfo() {
        return getJson(getServerInfoApiUrl()).thenA(parseServerInfo);
    }

    function getSearch(searchArgs) {
        return getJson(getSearchApiUrl(searchArgs)).thenA(function (ps) {
            return ps.map(parsePost);
        });
    }

    function updateBoardDescription(board) {
        return patchJson(getUpdateBoardDescriptionApiUrl(), new RequestBody(board)).thenA(function (message) {
            return message;
        });
    }

    function updateHiddenByOp(updatedHiddenByOp) {
        return patchJson(getUpdateHiddenByOpApiUrl(), new RequestBody(updatedHiddenByOp)).thenA(function (message) {
            return message;
        });
    }

    function publish(newPost) {
        return postJson(getPublishApiUrl(), new RequestBody(newPost)).thenA(function (p) {
            return parsePost(p);
        });
    }

    function authorize(auth) {
        return postJson(getAuthorizeApiUrl(), new RequestBody(auth)).thenA(parseAuthInfo);
    }

    function createUser(newUser) {
        return postJson(getCreateUserApiUrl(), new RequestBody(newUser)).thenA(function (message) {
            return message;
        });
    }

    function continueSubscription(subscriptionId) {
        return postJson(getContinueSubscriptionApiUrl(subscriptionId)).thenA(function (message) {
            return message;
        });
    }

    function getJson(url) {
        return handleFetchPromise(fetch(url));
    }

    function postJson(url, data) {
        return httpRequestJson(url, 'POST', data);
    }

    function patchJson(url, data) {
        return httpRequestJson(url, 'PATCH', data);
    }

    function httpRequestJson(url, method, data) {
        var headers = {
            'Accept': 'application/json',
            'Content-Type': 'application/json'
        };
        if (['POST', 'PATCH', 'DELETE'].includes(method)) {
            var accessToken = getAccessToken();
            if (accessToken) headers['Authorization'] = accessToken;
        }
        return handleFetchPromise(fetch(url, {
            method: method,
            headers: headers,
            body: JSON.stringify(data)
        }));
    }

    function handleFetchPromise(fetchPromise) {
        return fetchPromise.then(function (response) {
            return Promise.all([Promise.resolve(response), response.json()]);
        }).then(function (_ref) {
            var _ref2 = _slicedToArray(_ref, 2),
                response = _ref2[0],
                json = _ref2[1];

            if (response.ok) return json;

            throw parseErrorResponse(json);
        }).then(function (json) {
            return parseSuccessResponse(json);
        });
    }

    // file: post_cache_service.js

    // key: postId, value: PostModel
    var cachedPosts = {};

    function get(postId) {
        // console.log(cachedPosts);
        return cachedPosts[toSimpleUuid(postId)];
    }

    function saveForRecentPosts(recentPosts) {
        saveMany(recentPosts.posts);
        return recentPosts;
    }

    function saveForThreadSlice(threadSlice) {
        save(threadSlice.op);
        saveMany(threadSlice.posts);
        save(threadSlice.prevPost);
        return threadSlice;
    }

    function saveForWarpSlice(warpSlice) {
        warpSlice.shellThreads.forEach(saveForThreadSlice);
        return warpSlice;
    }

    function save(post) {
        if (!post) return post;

        // do not cache if thread or board not available
        if (!post.thread || !post.thread.board) return post;

        cachedPosts[toSimpleUuid(post.postId)] = post;
        return post;
    }

    function saveMany(posts) {
        posts.forEach(save);
        return posts;
    }

    function updateHiddenByOp$1(hiddenByOpEvent) {
        var post = get(hiddenByOpEvent.postId);

        if (post) {
            post.hiddenByOp = hiddenByOpEvent.hiddenByOp;
            save(post);
        }
        return post;
    }

    // file: alive_tabs_service.js

    var tabCreatedAt = new Date().getTime();
    var tabId = "tab_" + tabCreatedAt;
    var isEventSourceTab = false;
    var isEventSourceListeners = [];

    function heartbeat() {
        var aliveTabs = getOtherAliveTabs();
        var oldIsEventSourceTab = isEventSourceTab;

        // this tab will be used to connect to SSE if
        // there's no other connected tab at the moment and it's the oldest tab
        isEventSourceTab = !aliveTabs.some(function (x) {
            return x.value.isEventSource;
        }) && aliveTabs.every(function (x) {
            return x.value.createdAt > tabCreatedAt;
        });

        setItem$1(tabId, {
            createdAt: tabCreatedAt,
            aliveAt: new Date().getTime(),
            isEventSource: isEventSourceTab
        });

        if (isEventSourceTab !== oldIsEventSourceTab) {
            isEventSourceListeners.forEach(function (listener) {
                return listener(isEventSourceTab);
            });
        }
    }

    function initAliveTabsService() {
        cleanOldTabs();
        heartbeat();
        var intervalId = setInterval(function () {
            return heartbeat();
        }, config.tabHeartbeatTimeout);

        addEventListener('beforeunload', function (e) {
            clearInterval(intervalId);
            removeItem$1(tabId);
        });
    }

    function getTabs() {
        var tabs = getItems$1(/^tab_/);
        // const now = new Date().getTime();
        // console.log(tabs.map(t => {
        //     return {tab: t.key, at: t.value.aliveAt, dur: (now - t.value.aliveAt)};
        // }));
        return tabs;
    }

    function getAliveTabs() {
        var now = new Date().getTime();

        return getTabs().filter(function (x) {
            return now - x.value.aliveAt < config.tabHeartbeatTimeout * 4;
        });
    }

    function getOtherAliveTabs() {
        return getAliveTabs().filter(function (x) {
            return x.key !== tabId;
        });
    }

    function cleanOldTabs() {
        var now = new Date().getTime();

        getTabs().filter(function (x) {
            return now - x.value.aliveAt >= config.tabHeartbeatTimeout * 4;
        }).forEach(function (x) {
            return removeItem$1(x.key);
        });
    }

    function isCurrentTabEventSource() {
        return isEventSourceTab;
    }

    function addIsEventSourceListener(listener) {
        isEventSourceListeners.push(listener);
    }

    var _createClass$3 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$6(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var AppEventSource = function () {
        function AppEventSource(url, configuration) {
            var _this = this;

            _classCallCheck$6(this, AppEventSource);

            this._url = url;
            this._configuration = configuration;

            this._sseListeners = [];
            this._onErrorEventListeners = [];

            this._sseChannel = new BroadcastChannel(config.sse.broadcastChannel);

            this._isEventSourceTab = isCurrentTabEventSource();

            if (this._isEventSourceTab) {
                console.log("Current tab must connect!"); // todo: delete
                this._connect();
            } else {
                console.log("Do not connect. Listen to other tab!"); // todo: delete
            }

            addIsEventSourceListener(function (isEventSourceTab) {
                _this._isEventSourceTab = isEventSourceTab;

                if (_this._isEventSourceTab) {
                    console.log("Current tab must connect!"); // todo: delete
                    _this._connect();
                } else {
                    console.log("Disconnect. listen to other tab"); // todo: delete
                    _this._closeEventSource();
                }
            });

            this.addConnectedEventListener(function (e, data) {
                return _this._onConnected(e, data);
            });

            this.addConnectedEventListener(function (e) {
                if (!_this._isEventSourceTab) return;
                _this._sseChannel.postMessage({ type: e.type, data: e.data });
            });

            this.addPostsListener(function (e) {
                if (!_this._isEventSourceTab) return;
                _this._sseChannel.postMessage({ type: e.type, data: e.data });
            });

            this.addBoardDescriptionListener(function (e) {
                if (!_this._isEventSourceTab) return;
                _this._sseChannel.postMessage({ type: e.type, data: e.data });
            });

            this.addHiddenByOpListener(function (e) {
                if (!_this._isEventSourceTab) return;
                _this._sseChannel.postMessage({ type: e.type, data: e.data });
            });

            this.addErrorEventListener(function (e) {
                if (!_this._isEventSourceTab) return;
                _this._sseChannel.postMessage({ type: e.type, readyState: _this._eventSource.readyState });
            });

            // Listen to other tab.
            this._sseChannel.addEventListener('message', function (e) {
                return _this._sseChannelListener(e);
            });

            addEventListener('beforeunload', function (e) {
                return _this._close();
            });
        }

        _createClass$3(AppEventSource, [{
            key: "addErrorEventListener",
            value: function addErrorEventListener(listener) {
                this._onErrorEventListeners.push(listener);
            }
        }, {
            key: "addPostsListener",
            value: function addPostsListener(listener) {
                this._addJsonEventListener(config.sse.type.post, function (e, data) {
                    return listener(e, parsePost(data));
                });
            }
        }, {
            key: "addBoardDescriptionListener",
            value: function addBoardDescriptionListener(listener) {
                this._addJsonEventListener(config.sse.type.boardDescription, function (e, data) {
                    return listener(e, parseBoard(data));
                });
            }
        }, {
            key: "addHiddenByOpListener",
            value: function addHiddenByOpListener(listener) {
                this._addJsonEventListener(config.sse.type.hiddenByOp, function (e, data) {
                    return listener(e, parseHiddenByOpEvent(data));
                });
            }
        }, {
            key: "addConnectedEventListener",
            value: function addConnectedEventListener(listener) {
                if (!this._isEventSourceTab) listener({ type: config.sse.type.connected }, {});

                this._addJsonEventListener(config.sse.type.connected, listener);
            }
        }, {
            key: "_close",
            value: function _close() {
                this._sseChannel.close();
                this._closeEventSource();
            }
        }, {
            key: "_closeEventSource",
            value: function _closeEventSource() {
                if (this._eventSource) this._eventSource.close();
            }
        }, {
            key: "_connect",
            value: function _connect() {
                var _this2 = this;

                this._eventSource = new EventSource(this._url, this._configuration);
                this._eventSource.onerror = function (e) {
                    return _this2._onError(e);
                };

                this._sseListeners.forEach(function (sseListener) {
                    return _this2._restoreEventListener(sseListener.type, sseListener.listener);
                });
            }
        }, {
            key: "_onConnected",
            value: function _onConnected(e, data) {
                if (!this._isEventSourceTab) return;

                this._subscriptionId = data.subscriptionId;
                this._keepSubscriptionAlive();
                console.log(config.sse.type.connected, this._subscriptionId);
            }
        }, {
            key: "_keepSubscriptionAlive",
            value: function _keepSubscriptionAlive() {
                var _this3 = this;

                if (!this._isEventSourceTab) return;

                clearTimeout(this._keepAliveTimeoutId);

                if (this._eventSource.readyState !== EventSource.OPEN) return;

                console.log('keep alive', this._subscriptionId);

                this._keepAliveTimeoutId = setTimeout(function () {
                    continueSubscription$1(_this3._subscriptionId).then(function () {
                        return _this3._keepSubscriptionAlive();
                    });
                }, config.sse.continueSubscriptionTimeout);
            }
        }, {
            key: "_addJsonEventListener",
            value: function _addJsonEventListener(type, listener) {
                this._addEventListener(type, function (e) {
                    return listener(e, JSON.parse(e.data));
                });
            }
        }, {
            key: "_addEventListener",
            value: function _addEventListener(type, listener) {
                this._sseListeners.push(new SseListener(type, listener));
                if (this._eventSource) this._eventSource.addEventListener(type, listener);
            }
        }, {
            key: "_restoreEventListener",
            value: function _restoreEventListener(type, listener) {
                this._eventSource.addEventListener(type, listener);
            }
        }, {
            key: "_onError",
            value: function _onError(e) {
                var _this4 = this;

                console.log('Event Source failed, readyState = ', this._eventSource.readyState);

                this._onErrorEventListeners.forEach(function (listener) {
                    return listener(e, _this4._eventSource.readyState);
                });

                if (this._eventSource.readyState !== EventSource.CLOSED) return;

                // firefox won't try to reconnect after CLOSED state, so we have to do it ourselves
                clearTimeout(this._reconnectTimeoutId);
                this._reconnectTimeoutId = setTimeout(function () {
                    return _this4._connect();
                }, config.sse.reconnectTimeout);
            }
        }, {
            key: "_sseChannelListener",
            value: function _sseChannelListener(e) {
                if (this._isEventSourceTab) return;

                var sse = e.data;
                this._sseListeners.filter(function (sseListener) {
                    return sseListener.type === sse.type;
                }).forEach(function (sseListener) {
                    return sseListener.listener(sse, sse.data);
                });

                if (sse.type === 'error') {
                    this._onErrorEventListeners.forEach(function (listener) {
                        listener(sse, sse.readyState);
                    });
                }
            }
        }]);

        return AppEventSource;
    }();

    var SseListener = function SseListener(type, listener) {
        _classCallCheck$6(this, SseListener);

        this.type = type;
        this.listener = listener;
    };

    // file: client_service.js

    var clientIdLsKey = 'betaClientId';

    function getOrCreateClientId() {
        var clientId = getItem$1(clientIdLsKey, null);

        if (clientId) return Promise.resolve(clientId);

        clientId = newClientId();
        setItem$1(clientIdLsKey, clientId);

        return Promise.resolve(clientId);
    }

    function updateClientId() {

        var clientId = newClientId();
        setItem$1(clientIdLsKey, clientId);

        return Promise.resolve(clientId);
    }

    function setClientId(clientId) {

        setItem$1(clientIdLsKey, clientId);

        return Promise.resolve(clientId);
    }

    function newClientId() {
        return Math.random().toString(36).substring(2);
    }

    // file: favorites_service.js

    var favoriteBoardsLsKey = 'favorite_boards';

    function getFavoriteBoards() {
        return getItem$1(favoriteBoardsLsKey, []).map(parseBoard);
    }

    function addOrRemoveFavoriteBoard(board) {
        var boards = getFavoriteBoards();

        if (board.isFavorite) {
            // add todo: there's probably a bug: you don't push in the correct order
            if (!boards.some(function (b) {
                return b.name === board.name;
            })) boards.push(board);
        } else {
            // remove
            boards = boards.filter(function (b) {
                return b.name !== board.name;
            });
        }

        setItem$1(favoriteBoardsLsKey, boards);
    }

    var favoriteThreadsLsKey = 'favorite_threads';

    function getFavoriteThreads() {
        return getItem$1(favoriteThreadsLsKey, []).map(parseThread);
    }

    function addOrRemoveFavoriteThread(thread) {
        var threads = getFavoriteThreads();

        if (thread.isFavorite) {
            // add todo: there's probably a bug: you don't push in the correct order
            if (!threads.some(function (t) {
                return t.threadId === thread.threadId;
            })) threads.push(thread);
        } else {
            // remove
            threads = threads.filter(function (t) {
                return t.threadId !== thread.threadId;
            });
        }

        setItem$1(favoriteThreadsLsKey, threads);
    }

    function initFavoritesService() {
        addBoardDescriptionListener(function (e, board) {
            // change board description in LS
            setItem$1(favoriteBoardsLsKey, getFavoriteBoards().map(function (b) {
                if (b.name === board.name) b.description = board.description;
                return b;
            }));
        });
    }

    // file: track_new_posts_service.js

    var lsKey = 'last_seen_at_data';

    function getLastSeenAt(threadId) {
        var lastSeenAtStr = getLastSeenAtData()[threadId];
        return lastSeenAtStr ? new Date(lastSeenAtStr) : lastSeenAtStr;
    }

    function updateLastSeenAt(threadId, lastSeenAt) {
        var data = getLastSeenAtData();
        data[threadId] = lastSeenAt;

        setItem$1(lsKey, data);
        return Promise.resolve();
    }

    function deleteAllLastSeenAt() {
        removeItem$1(lsKey);
        return Promise.resolve();
    }

    function getLastSeenAtData() {
        return getItem$1(lsKey, {});
    }

    // file: repository.js

    function getRecentPosts$1(threadId) {
        return getRecentPosts(threadId).then(function (r) {
            return r.data;
        }).then(saveForRecentPosts);
    }

    function getPostsBefore$1(postId, threadId) {
        return getPostsBefore(postId, threadId).then(function (r) {
            return r.data;
        }).then(saveMany);
    }

    function getPostsAfter$1(postId, threadId) {
        return getPostsAfter(postId, threadId).then(function (r) {
            return r.data;
        }).then(saveMany);
    }

    function getPost$1(postId) {
        var includeIndex = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

        var cachedPost = get(postId);
        if (cachedPost && !includeIndex) return Promise.resolve(cachedPost);

        return getPost(postId, includeIndex).then(function (r) {
            return r.data;
        }).then(save);
    }

    function getBoards$1(isFavorite) {
        var skip = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

        {
            var favoriteBoards = getFavoriteBoards();

            if (isFavorite) {
                // return favorite boards from local storage
                return Promise.resolve(favoriteBoards.skip(skip).take(config.pageSize.general));
            }

            var favoriteBoardsNames = favoriteBoards.map(function (b) {
                return b.name;
            });

            return getBoards(null, skip).then(function (r) {
                var boards = r.data;
                // todo: here's a potential bug if a user added all boards as favorites for this page
                return boards.filter(function (b) {
                    return !favoriteBoardsNames.includes(b.name);
                });
            });
        }
    }

    function getThreads$1(isFavorite) {
        var boardName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
        var skip = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;

        {
            var favoriteThreads = getFavoriteThreads().filter(function (t) {
                return !boardName || t.board.name === boardName;
            });

            if (isFavorite) {
                // return favorite threads from local storage
                return Promise.resolve(favoriteThreads.skip(skip).take(config.pageSize.general));
            }

            var favoriteThreadsIds = favoriteThreads.map(function (t) {
                return t.threadId;
            });

            return getThreads(null, boardName, skip).then(function (r) {
                var threads = r.data;
                // todo: here's a potential bug if a user added all threads as favorites for this page
                return threads.filter(function (t) {
                    return !favoriteThreadsIds.includes(t.threadId);
                });
            });
        }
    }

    function getThreadPage$1(threadId) {
        var pageNumber = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

        // null means the last page

        return getThreadPage(threadId, pageNumber).then(function (r) {
            r.data.op.thread = r.data.thread;

            r.data.posts = r.data.posts.map(function (p) {
                p.thread = r.data.thread;
                return p;
            });
            saveForThreadSlice(r.data);
            return r;
        });
    }

    function getWarpPage$1() {
        var boardName = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
        var pageNumber = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

        return getWarpPage(boardName, pageNumber).then(function (r) {
            r.data.shellThreads = r.data.shellThreads.map(function (st) {
                st.thread.board = st.thread.board || r.data.board;
                st.op.thread = st.thread;

                st.posts = st.posts.map(function (p) {
                    p.thread = st.thread;
                    return p;
                });
                return st;
            });
            saveForWarpSlice(r.data);
            return r;
        });
    }

    function getServerInfo$1() {
        return getServerInfo().then(function (r) {
            return r.data;
        });
    }

    function getSearch$1(searchArgs) {
        var defaultValue = new SuccessResponse([], 0, 0, 0);

        if (!searchArgs) {
            return Promise.resolve(defaultValue);
        }

        return getSearch(searchArgs).then(function (r) {
            return  defaultValue;
        });
    }

    function createUser$1(newUser) {
        return createUser(newUser).then(function (r) {
            return r.data;
        });
    }

    function updateBoardDescription$1(board) {
        return updateBoardDescription(board).then(function (r) {
            return r.data;
        });
    }

    function updateHiddenByOp$2(updatedHiddenByOp) {
        return getOrCreateClientId().then(function (clientId) {
            updatedHiddenByOp.clientId = clientId;

            return updateHiddenByOp(updatedHiddenByOp).then(function (r) {
                return r.data;
            });
        });
    }

    function publish$1(newPost) {
        return getOrCreateClientId().then(function (clientId) {
            newPost.clientId = clientId;

            return publish(newPost).then(function (r) {
                return r.data;
            });
        });
    }

    function updateBoardIsFavorite(board) {
        return Promise.resolve(addOrRemoveFavoriteBoard(board));
    }

    function updateThreadIsFavorite(thread) {
        return Promise.resolve(addOrRemoveFavoriteThread(thread));
    }

    function updateLastSeenAt$1(threadId, lastSeenAt) {
        return updateLastSeenAt(threadId, lastSeenAt);
    }

    function deleteAllLastSeenAt$1() {
        return deleteAllLastSeenAt();
    }

    function resetAllLastSeenAt() {
        return deleteAllLastSeenAt();
    }

    function authorize$1(auth) {
        return authorize(auth).then(function (r) {
            return r.data;
        });
    }

    var eventSource = void 0;

    function subscribe() {
        eventSource = new AppEventSource(getSubscribeApiUrl());
    }

    function addErrorEventListener(listener) {
        return eventSource.addErrorEventListener(listener);
    }

    function addPostsListener(listener) {
        return eventSource.addPostsListener(function (e, post) {
            return listener(e, save(post));
        });
    }

    function addBoardDescriptionListener(listener) {
        return eventSource.addBoardDescriptionListener(listener);
    }

    function addHiddenByOpListener(listener) {
        return eventSource.addHiddenByOpListener(listener);
    }

    function addConnectedEventListener(listener) {
        return eventSource.addConnectedEventListener(listener);
    }

    function continueSubscription$1(subscriptionId) {
        return continueSubscription(subscriptionId);
    }

    var _createClass$4 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$7(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var SidebarBoardV = function () {
        function SidebarBoardV(board, sidebarV) {
            var _this = this;

            _classCallCheck$7(this, SidebarBoardV);

            this.board = board;
            this.sidebarV = sidebarV;

            var itemE = document.createElement('div');
            itemE.classList.add('item');

            itemE.view = this;
            this.element = itemE;

            this.selectableV = new SelectableV(itemE);

            var favBtnE = document.createElement('div');
            this.favBtnE = favBtnE;
            itemE.appendChild(favBtnE);
            favBtnE.classList.add('fav_btn');
            favBtnE.classList.add(board.isFavorite ? 'icon-fav_remove' : 'icon-fav_add');
            favBtnE.title = board.isFavorite ? 'Remove from favorites' : 'Add to favorites';
            favBtnE.addEventListener('click', function (e) {
                e.preventDefault();
                _this.toggleFavorite();
            }, false);

            var linkE = document.createElement('a');
            this._linkE = linkE;
            itemE.appendChild(linkE);
            linkE.classList.add('link');
            linkE.title = board.description;
            linkE.href = getBoardUrl(board.name);
            linkE.appendText(this._getLinkText());
            linkE.addEventListener('click', function (e) {
                e.preventDefault();
                setBoard(_this.board.name);
            }, false);
        }

        _createClass$4(SidebarBoardV, [{
            key: "toggleFavorite",
            value: function toggleFavorite() {
                var _this2 = this;

                this.board.isFavorite = !this.board.isFavorite;

                updateBoardIsFavorite(this.board).then(function () {
                    _this2.toggleFavoriteUiOnly(_this2.board.isFavorite);
                });
            }
        }, {
            key: "toggleFavoriteUiOnly",
            value: function toggleFavoriteUiOnly(isFavorite) {
                this.board.isFavorite = isFavorite;
                this.favBtnE.classList.remove(this.board.isFavorite ? 'icon-fav_remove' : 'icon-fav_add');
                this.favBtnE.classList.add(this.board.isFavorite ? 'icon-fav_remove' : 'icon-fav_add');
                this.favBtnE.title = this.board.isFavorite ? 'Remove from favorites' : 'Add to favorites';

                this.sidebarV.toggleFavorite(this);
            }
        }, {
            key: "setDescription",
            value: function setDescription(description) {
                this.board.description = description;
                this._linkE.replaceChildrenWith(this._getLinkText());
                this._linkE.title = description;
            }
        }, {
            key: "_getLinkText",
            value: function _getLinkText() {
                return "/" + this.board.name + "/" + (this.board.description ? " \u2013 " + this.board.description : "");
            }
        }]);

        return SidebarBoardV;
    }();

    var _createClass$5 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$8(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var BoardsSidebarV = function () {
        function BoardsSidebarV() {
            var _this = this;

            _classCallCheck$8(this, BoardsSidebarV);

            var sidebarE = $('#boards_sidebar');
            this.element = sidebarE;
            sidebarE.view = this;

            var SIDEBAR_WIDTH = 'boards_sidebar_width';
            var SIDEBAR_HIDDEN = 'boards_sidebar_hidden';

            this.dragDependantV = new DragDependantV(sidebarE);
            this.dragDependantV = new WidthStoredDragDependantD(this.dragDependantV, SIDEBAR_WIDTH, 200);
            this.dragDependantV = new VisibilityStoredDragDependantD(this.dragDependantV, SIDEBAR_HIDDEN, true);
            this.dragDependantV = new SidebarDragDependantD(this.dragDependantV);

            this.favBoardsGroupE = sidebarE.$('#fav_boards_group');
            this.otherBoardsGroupE = sidebarE.$('#other_boards_group');

            this.favBoardsGroupE.addEventListener('scroll', function (e) {
                return _this._onScroll(e);
            }, false);
            this.otherBoardsGroupE.addEventListener('scroll', function (e) {
                return _this._onScroll(e);
            }, false);

            var boardName = getSelectedBoardName();
            this._load(true, boardName, 0);
            this._load(false, boardName, 0);

            document.addEventListener('selectBoard', function (e) {
                return _this._onSelectBoard(e);
            }, false);
            document.addEventListener('selectThread', function (e) {
                return _this._onSelectThread(e);
            }, false);
            document.addEventListener('selectGlobalPage', function (e) {
                return _this._onSelectGlobalPage(e);
            }, false);
            document.addEventListener('selectSearch', function (e) {
                return _this._onSelectSearch(e);
            }, false);

            addBoardDescriptionListener(function (e, board) {
                return _this._onBoardDescriptionChange(board.name, board.description);
            });

            document.addEventListener('lsChange', this._onLsChangeHandler = function (e) {
                return _this._onLsChange(e);
            }, false);
        }

        _createClass$5(BoardsSidebarV, [{
            key: "toggleFavorite",
            value: function toggleFavorite(boardV) {
                boardV.element.detach();
                this.addToGroup(boardV, boardV.board.isFavorite ? this.favBoardsGroupE : this.otherBoardsGroupE);
            }
        }, {
            key: "addToGroup",
            value: function addToGroup(boardV, groupE) {
                var _iteratorNormalCompletion = true;
                var _didIteratorError = false;
                var _iteratorError = undefined;

                try {
                    for (var _iterator = groupE.children[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                        var el = _step.value;

                        if (boardV.board.name === el.view.board.name) {
                            // skip it, we already have this board
                            return;
                        }

                        if (boardV.board.name <= el.view.board.name) {
                            groupE.insertBefore(boardV.element, el);
                            return;
                        }
                    }
                } catch (err) {
                    _didIteratorError = true;
                    _iteratorError = err;
                } finally {
                    try {
                        if (!_iteratorNormalCompletion && _iterator.return) {
                            _iterator.return();
                        }
                    } finally {
                        if (_didIteratorError) {
                            throw _iteratorError;
                        }
                    }
                }

                groupE.appendChild(boardV.element);
            }
        }, {
            key: "_load",
            value: function _load(isFavorite, boardName, offset) {
                var _this2 = this;

                this._currentBoardName = boardName;

                var groupE = isFavorite ? this.favBoardsGroupE : this.otherBoardsGroupE;

                getBoards$1(isFavorite, offset).then(function (boards) {

                    var boardVs = boards.map(function (board) {
                        return new SidebarBoardV(board, _this2);
                    });
                    boardVs.forEach(function (boardV) {
                        return _this2.addToGroup(boardV, groupE);
                    });

                    if (boardName) _this2._selectBoard(boardName, _this2._getBoardsVsOfGroup(groupE));
                });
            }
        }, {
            key: "_onSelectBoard",
            value: function _onSelectBoard(e) {
                this._selectBoard(e.detail.boardName);
            }
        }, {
            key: "_onSelectThread",
            value: function _onSelectThread(e) {
                this._selectBoard(e.detail.boardName);
            }
        }, {
            key: "_onSelectGlobalPage",
            value: function _onSelectGlobalPage(e) {
                this._deselectAll();
            }
        }, {
            key: "_onSelectSearch",
            value: function _onSelectSearch(e) {
                this._deselectAll();
            }
        }, {
            key: "_selectBoard",
            value: function _selectBoard(boardName) {
                var boardsVs = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this._getBoardsVs();

                boardsVs.forEach(function (boardV) {
                    if (boardV.board.name === boardName) {
                        boardV.selectableV.select();
                    } else {
                        boardV.selectableV.deselect();
                    }
                });
            }
        }, {
            key: "_deselectAll",
            value: function _deselectAll() {
                this._getBoardsVs().forEach(function (boardV) {
                    return boardV.selectableV.deselect();
                });
            }
        }, {
            key: "_getBoardsVs",
            value: function _getBoardsVs() {
                return this._getBoardsVsOfGroup(this.favBoardsGroupE).concat(this._getBoardsVsOfGroup(this.otherBoardsGroupE));
            }
        }, {
            key: "_getBoardsVsOfGroup",
            value: function _getBoardsVsOfGroup(groupE) {
                return groupE.children.map(function (el) {
                    return el.view;
                });
            }
        }, {
            key: "_onScroll",
            value: function _onScroll(e) {
                var groupE = e.target;
                if (groupE.scrollTop < groupE.getMaxScrollTop()) return;

                var isFavorite = groupE === this.favBoardsGroupE;

                this._load(isFavorite, this._currentBoardName, this._getBoardsVsOfGroup(groupE).length);
            }
        }, {
            key: "_onBoardDescriptionChange",
            value: function _onBoardDescriptionChange(boardName, boardDescription) {
                this._getBoardsVs().filter(function (boardV) {
                    return boardV.board.name === boardName;
                }).forEach(function (boardV) {
                    return boardV.setDescription(boardDescription);
                });
            }
        }, {
            key: "_onLsChange",
            value: function _onLsChange(e) {
                if (e.detail.isFromForeignWindow && e.detail.key === favoriteBoardsLsKey) {
                    var favoriteBoards = getFavoriteBoards();

                    this._getBoardsVs().forEach(function (boardV) {
                        var isFavorite = favoriteBoards.some(function (b) {
                            return b.name === boardV.board.name;
                        });

                        boardV.toggleFavoriteUiOnly(isFavorite);
                    });
                }
            }
        }]);

        return BoardsSidebarV;
    }();

    var _createClass$6 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$9(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var SidebarThreadV = function () {
        function SidebarThreadV(thread, sidebarV) {
            var _this = this;

            _classCallCheck$9(this, SidebarThreadV);

            this.thread = thread;
            this.sidebarV = sidebarV;

            var itemE = document.createElement('div');
            itemE.classList.add('item');

            itemE.view = this;
            this.element = itemE;

            this.selectableV = new SelectableV(itemE);

            this.favBtnE = document.createElement('div');
            itemE.appendChild(this.favBtnE);
            this.favBtnE.classList.add('fav_btn');
            this.favBtnE.classList.add(thread.isFavorite ? 'icon-fav_remove' : 'icon-fav_add');
            this.favBtnE.title = thread.isFavorite ? 'Remove from favorites' : 'Add to favorites';
            this.favBtnE.addEventListener('click', function (e) {
                e.preventDefault();
                _this.toggleFavorite();
            }, false);

            var linkE = document.createElement('a');
            itemE.appendChild(linkE);
            linkE.classList.add('link');
            linkE.title = thread.subject;
            linkE.href = getThreadUrl(thread.board.name, thread.threadId);

            var boardPrefix = showAllThreads() ? "/" + thread.board.name + "/" : "";

            linkE.appendText("" + boardPrefix + toShortcutUuid(thread.threadId) + " \u2013 " + thread.subject);

            linkE.addEventListener('click', function (e) {
                e.preventDefault();
                setThread(_this.thread.board.name, _this.thread.threadId);
            }, false);
        }

        _createClass$6(SidebarThreadV, [{
            key: "toggleFavorite",
            value: function toggleFavorite() {
                var _this2 = this;

                this.thread.isFavorite = !this.thread.isFavorite;

                updateThreadIsFavorite(this.thread).then(function () {
                    _this2.toggleFavoriteUiOnly(_this2.thread.isFavorite);
                });
            }
        }, {
            key: "toggleFavoriteUiOnly",
            value: function toggleFavoriteUiOnly(isFavorite) {
                this.thread.isFavorite = isFavorite;
                this.favBtnE.classList.remove(this.thread.isFavorite ? 'icon-fav_remove' : 'icon-fav_add');
                this.favBtnE.classList.add(this.thread.isFavorite ? 'icon-fav_remove' : 'icon-fav_add');
                this.favBtnE.title = this.thread.isFavorite ? 'Remove from favorites' : 'Add to favorites';

                this.sidebarV.toggleFavorite(this);
            }
        }]);

        return SidebarThreadV;
    }();

    var _createClass$7 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$a(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ThreadsSidebarV = function () {
        function ThreadsSidebarV() {
            var _this = this;

            _classCallCheck$a(this, ThreadsSidebarV);

            var sidebarE = $('#threads_sidebar');
            this.element = sidebarE;
            sidebarE.view = this;

            var SIDEBAR_WIDTH = 'threads_sidebar_width';
            var SIDEBAR_HIDDEN = 'threads_sidebar_hidden';

            this.dragDependantV = new DragDependantV(sidebarE);
            this.dragDependantV = new WidthStoredDragDependantD(this.dragDependantV, SIDEBAR_WIDTH, 200);
            this.dragDependantV = new VisibilityStoredDragDependantD(this.dragDependantV, SIDEBAR_HIDDEN, true);
            this.dragDependantV = new SidebarDragDependantD(this.dragDependantV);

            this.favThreadsGroupE = $('#fav_threads_group');
            this.otherThreadsGroupE = $('#other_threads_group');

            this.favThreadsGroupE.addEventListener('scroll', function (e) {
                return _this._onScroll(e);
            }, false);
            this.otherThreadsGroupE.addEventListener('scroll', function (e) {
                return _this._onScroll(e);
            }, false);

            var boardName = showAllThreads() ? null : getSelectedBoardName();
            this._load(true, boardName, getSelectedThreadId(), 0);
            this._load(false, boardName, getSelectedThreadId(), 0);

            document.addEventListener('selectBoard', function (e) {
                return _this._onSelectBoard(e);
            }, false);
            document.addEventListener('selectThread', function (e) {
                return _this._onSelectThread(e);
            }, false);
            document.addEventListener('selectGlobalPage', function (e) {
                return _this._onSelectGlobalPage(e);
            }, false);
            document.addEventListener('selectSearch', function (e) {
                return _this._onSelectSearch(e);
            }, false);

            document.addEventListener('lsChange', this._onLsChangeHandler = function (e) {
                return _this._onLsChange(e);
            }, false);
        }

        _createClass$7(ThreadsSidebarV, [{
            key: "toggleFavorite",
            value: function toggleFavorite(threadV) {
                threadV.element.detach();
                this.addToGroup(threadV, threadV.thread.isFavorite ? this.favThreadsGroupE : this.otherThreadsGroupE);
            }
        }, {
            key: "addToGroup",
            value: function addToGroup(threadV, groupE) {
                var _iteratorNormalCompletion = true;
                var _didIteratorError = false;
                var _iteratorError = undefined;

                try {
                    for (var _iterator = groupE.children[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                        var el = _step.value;

                        if (threadV.thread.threadId === el.view.thread.threadId) {
                            // skip it, we already have this thread
                            return;
                        }

                        if (threadV.thread.lastPostDate >= el.view.thread.lastPostDate) {
                            groupE.insertBefore(threadV.element, el);
                            return;
                        }
                    }
                } catch (err) {
                    _didIteratorError = true;
                    _iteratorError = err;
                } finally {
                    try {
                        if (!_iteratorNormalCompletion && _iterator.return) {
                            _iterator.return();
                        }
                    } finally {
                        if (_didIteratorError) {
                            throw _iteratorError;
                        }
                    }
                }

                groupE.appendChild(threadV.element);
            }
        }, {
            key: "_load",
            value: function _load(isFavorite, boardName, threadId, offset) {
                var _this2 = this;

                this._currentBoardName = boardName;
                this._currentThreadId = threadId;

                var groupE = isFavorite ? this.favThreadsGroupE : this.otherThreadsGroupE;

                getThreads$1(isFavorite, boardName, offset).then(function (threads) {
                    if (offset === 0) groupE.empty();

                    var threadVs = threads.map(function (thread) {
                        return new SidebarThreadV(thread, _this2);
                    });
                    threadVs.forEach(function (threadV) {
                        return _this2.addToGroup(threadV, groupE);
                    });

                    if (threadId) _this2._selectThread(threadId, _this2._getThreadVsOfGroup(groupE));
                });
            }
        }, {
            key: "_onSelectBoard",
            value: function _onSelectBoard(e) {
                var boardName = e.detail.boardName;
                if (this._currentBoardName === boardName) {
                    this._deselectAll();
                    return;
                }

                this._load(true, boardName, null, 0);
                this._load(false, boardName, null, 0);
            }
        }, {
            key: "_onSelectThread",
            value: function _onSelectThread(e) {
                var boardName = showAllThreads() ? null : e.detail.boardName;
                var threadId = e.detail.threadId;

                if (this._currentBoardName === boardName) {
                    this._selectThread(threadId, this._getThreadsVs());
                    return;
                }

                this._load(true, boardName, threadId, 0);
                this._load(false, boardName, threadId, 0);
            }
        }, {
            key: "_onSelectGlobalPage",
            value: function _onSelectGlobalPage(e) {
                if (!this._currentBoardName) {
                    this._deselectAll();
                    return;
                }

                this._load(true, null, null, 0);
                this._load(false, null, null, 0);
            }
        }, {
            key: "_onSelectSearch",
            value: function _onSelectSearch(e) {
                this._deselectAll();
            }
        }, {
            key: "_selectThread",
            value: function _selectThread(threadId, threadsVs) {
                threadsVs.forEach(function (threadV) {
                    if (threadV.thread.threadId === threadId) {
                        threadV.selectableV.select();
                    } else {
                        threadV.selectableV.deselect();
                    }
                });
            }
        }, {
            key: "_deselectAll",
            value: function _deselectAll() {
                this._getThreadsVs().forEach(function (threadV) {
                    return threadV.selectableV.deselect();
                });
            }
        }, {
            key: "_getThreadsVs",
            value: function _getThreadsVs() {
                return this._getThreadVsOfGroup(this.favThreadsGroupE).concat(this._getThreadVsOfGroup(this.otherThreadsGroupE));
            }
        }, {
            key: "_getThreadVsOfGroup",
            value: function _getThreadVsOfGroup(groupE) {
                return groupE.children.map(function (el) {
                    return el.view;
                });
            }
        }, {
            key: "_onScroll",
            value: function _onScroll(e) {
                var groupE = e.target;
                if (groupE.scrollTop < groupE.getMaxScrollTop()) return;

                var isFavorite = groupE === this.favThreadsGroupE;
                var threadsVs = this._getThreadVsOfGroup(groupE);

                this._load(isFavorite, this._currentBoardName, this._currentThreadId, threadsVs.length);
            }
        }, {
            key: "_onLsChange",
            value: function _onLsChange(e) {
                if (e.detail.isFromForeignWindow && e.detail.key === favoriteThreadsLsKey) {
                    var favoriteThreads = getFavoriteThreads();

                    this._getThreadsVs().forEach(function (threadV) {
                        var isFavorite = favoriteThreads.some(function (t) {
                            return t.threadId === threadV.thread.threadId;
                        });

                        threadV.toggleFavoriteUiOnly(isFavorite);
                    });
                }
            }
        }]);

        return ThreadsSidebarV;
    }();

    // file: history_navigation_service.js

    function handlePopState(state) {

        if (getSelectedThreadId()) {

            selectThread(getSelectedBoardName(), getSelectedThreadId(), getPage(), getPostIdAnchor(), true);
        } else if (getSelectedBoardName()) {

            selectBoard(getSelectedBoardName(), getPage(), true);
        } else if (isSearchPage()) {

            selectSearch(getSearchArgs(), true);
        } else {
            selectGlobalPage(getPage(), true);
        }
    }

    function _classCallCheck$b(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var HeightStoredDragDependantD = function HeightStoredDragDependantD(dragDependantV, localStorageHeightKey) {
        var _this = this;

        _classCallCheck$b(this, HeightStoredDragDependantD);

        decorateView(dragDependantV, this, 'dragDependantV');

        this.setHeight = function (height) {
            _this._height = height;
            dragDependantV.setHeight(height);
        };

        window.addEventListener('mouseup', function (e) {
            if (_this._height >= 0) setItem$1(localStorageHeightKey, _this._height);
        }, false);

        document.addEventListener('lsChange', this._onLsChangeHandler = function (e) {
            if (e.detail.isFromForeignWindow && e.detail.key === localStorageHeightKey) {
                _this.setHeight(e.detail.newValue);
            }
        }, false);
    };

    var _createClass$8 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$c(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ChatContainerDragDependantD = function () {
        function ChatContainerDragDependantD(dragDependantV) {
            _classCallCheck$c(this, ChatContainerDragDependantD);

            decorateView(dragDependantV, this, 'dragDependantV');
            this.element = dragDependantV.element;

            this._dragDependantV = dragDependantV;
            this._containerE = $('#container');
        }

        _createClass$8(ChatContainerDragDependantD, [{
            key: 'setWidth',
            value: function setWidth(width) {
                if (this.element.view.isOnTop()) return;

                var currentWidth = this._getWidth();
                var availableWidth = this._getAvailableWidth();
                var delta = currentWidth - width;

                // if there will be no space left
                if (availableWidth + delta < 0) {
                    width = currentWidth + availableWidth;
                }

                this._dragDependantV.setWidth(width);
            }
        }, {
            key: 'setHeight',
            value: function setHeight(height) {
                if (this.element.view.isOnTop()) {
                    this._dragDependantV.setHeight(height);
                }
            }
        }, {
            key: 'toggleHide',
            value: function toggleHide() {
                // in case of showing
                var availableWidth = this._getAvailableWidth();
                if (this.element.isHidden() && getComputedWidth(this.element) > availableWidth) {
                    this.element.view.setWidth(availableWidth);
                }

                this._dragDependantV.toggleHide();
            }
        }, {
            key: '_getAvailableWidth',
            value: function _getAvailableWidth() {
                return this._containerE.offsetWidth;
            }
        }, {
            key: '_getWidth',
            value: function _getWidth() {
                return this.element.offsetWidth;
            }
        }, {
            key: '_getHeight',
            value: function _getHeight() {
                return this.element.clientHeight;
            }
        }]);

        return ChatContainerDragDependantD;
    }();

    var _createClass$9 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$d(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var DraggableWindowV = function () {
        function DraggableWindowV(element) {
            var _this = this;

            _classCallCheck$d(this, DraggableWindowV);

            this.element = element;
            this.isDisabled = false;

            this.titleBarE = element.$('.title_bar');
            if (!this.titleBarE) throw new Error("There must be a title bar!");

            this.titleBarE.addEventListener("mousedown", this._onMouseDownHandler = function (e) {
                return _this.onMouseDown(e);
            }, false);
        }

        _createClass$9(DraggableWindowV, [{
            key: "onMouseDown",
            value: function onMouseDown(e) {
                var _this2 = this;

                if (this.isDisabled) return;
                e.preventDefault();

                this.xBefore = e.clientX;
                this.yBefore = e.clientY;

                document.addEventListener("mouseup", this._onMouseUpHandler = function (e) {
                    return _this2.onMouseUp(e);
                }, false);

                document.addEventListener("mousemove", this._onMouseMoveHandler = function (e) {
                    return _this2.onMouseMove(e);
                }, false);
            }
        }, {
            key: "onMouseUp",
            value: function onMouseUp(e) {
                document.removeEventListener("mouseup", this._onMouseUpHandler, false);
                document.removeEventListener("mousemove", this._onMouseMoveHandler, false);
            }
        }, {
            key: "onMouseMove",
            value: function onMouseMove(e) {
                e.preventDefault();

                var xDelta = e.clientX - this.xBefore;
                var yDelta = e.clientY - this.yBefore;

                var left = Math.min(Math.max(0, this.element.offsetLeft + xDelta), window.innerWidth - this.element.offsetWidth);
                var top = Math.min(Math.max(0, this.element.offsetTop + yDelta), window.innerHeight - this.titleBarE.offsetHeight);

                this.element.style.left = left + "px";
                if (!this._isTopFixed) this.element.style.top = top + "px";

                this.xBefore = e.clientX;
                if (!this._isTopFixed) this.yBefore = e.clientY;
            }
        }, {
            key: "setTopFixed",
            value: function setTopFixed(isFixed) {
                this._isTopFixed = isFixed;
            }
        }]);

        return DraggableWindowV;
    }();

    var _createClass$a = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$e(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ChatContainerTrackVisibilityD = function () {
        function ChatContainerTrackVisibilityD(dragDependantV) {
            _classCallCheck$e(this, ChatContainerTrackVisibilityD);

            decorateView(dragDependantV, this, 'dragDependantV');
            this.element = dragDependantV.element;
            this._dragDependantV = dragDependantV;

            document.addEventListener("visibilitychange", function (e) {
                // if the chat is hidden then chat visibility hasn't been changed
                if (isChatHidden()) return;

                chatVisibilityChange(isChatOrDocumentHidden());
            }, false);
        }

        _createClass$a(ChatContainerTrackVisibilityD, [{
            key: "setWidth",
            value: function setWidth(width) {
                this._dragDependantV.setWidth(width);
            }
        }, {
            key: "toggleHide",
            value: function toggleHide() {
                this._dragDependantV.toggleHide();
                chatVisibilityChange(isChatOrDocumentHidden());
            }
        }]);

        return ChatContainerTrackVisibilityD;
    }();

    var _createClass$b = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$f(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ChatContainerV = function () {
        function ChatContainerV() {
            var _this = this;

            _classCallCheck$f(this, ChatContainerV);

            var chatContainerE = $('#chat_container');
            chatContainerE.view = this;
            this.element = chatContainerE;

            this.dragE = $('#chat_container_drag');

            this.CHAT_CONTAINER_DETACHED = 'chat_container_detached';
            this.DETACHED_CHAT_CONTAINER_WIDTH = 'detached_chat_container_width';
            this.DETACHED_CHAT_CONTAINER_HEIGHT = 'detached_chat_container_height';
            this.CHAT_CONTAINER_LEFT = 'chat_container_left';
            this.CHAT_CONTAINER_TOP = 'chat_container_top';
            this.ATTACHED_CHAT_CONTAINER_WIDTH = 'attached_chat_container_width';
            this.ATTACHED_CHAT_CONTAINER_HEIGHT = 'attached_chat_container_height';
            this.CHAT_CONTAINER_HIDDEN = 'chat_container_hidden';
            this.CHAT_CONTAINER_ON_TOP = 'chat_container_on_top';
            this.CHAT_CONTAINER_COLLAPSED = 'chat_container_collapsed';

            this.dragDependantV = new DragDependantV(chatContainerE);
            this.dragDependantV = new ChatContainerDragDependantD(this.dragDependantV);
            this.dragDependantV = new VisibilityStoredDragDependantD(this.dragDependantV, this.CHAT_CONTAINER_HIDDEN, false);
            this.dragDependantV = new WidthStoredDragDependantD(this.dragDependantV, this.ATTACHED_CHAT_CONTAINER_WIDTH, window.innerWidth / 2);
            this.dragDependantV = new HeightStoredDragDependantD(this.dragDependantV, this.ATTACHED_CHAT_CONTAINER_HEIGHT, window.innerHeight / 2);
            this.dragDependantV = new ChatContainerTrackVisibilityD(this.dragDependantV);

            this.draggableWindowV = new DraggableWindowV(chatContainerE);

            this.titleBarE = $('#chat_title_bar');
            this.detachBtnE = this.titleBarE.$('.detach_btn');
            this.detachBtnE.addEventListener('click', function (e) {
                return _this.toggleDetach(e);
            }, false);

            this.moveUpBtnE = this.titleBarE.$('.move_up_btn');
            this.moveRightBtnE = this.titleBarE.$('.move_right_btn');
            this.contentFrameE = $('#content_frame');
            this.moveUpBtnE.addEventListener('click', function (e) {
                return _this.moveUp();
            }, false);
            this.moveRightBtnE.addEventListener('click', function (e) {
                return _this.moveRight();
            }, false);

            this.collapseBtnE = this.titleBarE.$('.collapse_btn');
            this.expandBtnE = this.titleBarE.$('.expand_btn');
            this.collapseBtnE.addEventListener('click', function (e) {
                return _this.collapse();
            }, false);
            this.expandBtnE.addEventListener('click', function (e) {
                return _this.expand();
            }, false);

            if (this.isDetached()) this.detach();else this.attach();

            this.saveDetachedStateOnMouseUp();

            document.addEventListener('lsChange', this._onLsChangeHandler = function (e) {
                return _this._onLsChange(e.detail);
            }, false);
        }

        _createClass$b(ChatContainerV, [{
            key: "toggleDetach",
            value: function toggleDetach(e) {
                if (getItem$1(this.CHAT_CONTAINER_DETACHED)) this.attach();else this.detach();
            }
        }, {
            key: "detach",
            value: function detach() {
                setItem$1(this.CHAT_CONTAINER_DETACHED, true);

                this.element.style.minWidth = "";
                this.element.style.minHeight = "";
                this.element.style.width = this._getWidth() + "px";
                this.element.style.height = this._getHeight() + "px";

                this.element.style.left = this._getLeft() + "px";
                this.element.style.top = this._getTop() + "px";

                this.element.classList.add('detached');
                this.dragE.hide();

                this.draggableWindowV.isDisabled = false;
                this.detachBtnE.title = "attach";

                // change the icon
                var icon = this.detachBtnE.$(".icon");
                icon.classList.remove('icon-popout');
                icon.classList.add('icon-popin');

                this.moveUpBtnE.hide();
                this.moveRightBtnE.hide();

                if (this.isCollapsed()) this.collapse();else this.expand();
            }
        }, {
            key: "attach",
            value: function attach() {
                setItem$1(this.CHAT_CONTAINER_DETACHED, false);

                if (this.isOnTop()) this.moveUp();else this.moveRight();

                this.element.classList.remove('detached');
                this.dragE.show();

                this.draggableWindowV.isDisabled = true;
                this.detachBtnE.title = "detach";

                // change the icon
                var icon = this.detachBtnE.$(".icon");
                icon.classList.remove('icon-popin');
                icon.classList.add('icon-popout');

                var isOnTop = getItem$1(this.CHAT_CONTAINER_ON_TOP, false);
                this.moveUpBtnE.showOrHide(!isOnTop);
                this.moveRightBtnE.showOrHide(isOnTop);
            }
        }, {
            key: "saveDetachedStateOnMouseUp",
            value: function saveDetachedStateOnMouseUp() {
                var _this2 = this;

                document.addEventListener("mouseup", function () {
                    if (!_this2.isDetached()) return;

                    var style = getComputedStyle(_this2.element);

                    setItem$1(_this2.DETACHED_CHAT_CONTAINER_WIDTH, parseCssDimension(style.width));
                    setItem$1(_this2.DETACHED_CHAT_CONTAINER_HEIGHT, parseCssDimension(style.height));
                    setItem$1(_this2.CHAT_CONTAINER_LEFT, parseCssDimension(style.left));

                    if (!_this2.isCollapsed()) {
                        setItem$1(_this2.CHAT_CONTAINER_TOP, parseCssDimension(style.top));
                    }
                }, false);
            }
        }, {
            key: "moveUp",
            value: function moveUp() {
                setItem$1(this.CHAT_CONTAINER_ON_TOP, true);

                // this.contentFrameE.style.flexDirection = "column-reverse";
                this.contentFrameE.classList.add('chat_on_top');

                var height = getItem$1(this.ATTACHED_CHAT_CONTAINER_HEIGHT, null);
                this.element.style.height = height ? height + "px" : "40vh";
                this.element.style.minHeight = this.element.style.height.repeat(1);
                this.element.style.minWidth = "";
                this.element.style.width = "";

                this.moveUpBtnE.hide();
                this.moveRightBtnE.show();

                var icon = this.dragE.$(".icon");
                icon.classList.remove("icon-vdots");
                icon.classList.add("icon-hdots");

                this.dragE.dragV.isVertical = false;
                this.dragE.dragV.direction = 1; // because flexDirection = "column-reverse"
            }
        }, {
            key: "moveRight",
            value: function moveRight() {
                setItem$1(this.CHAT_CONTAINER_ON_TOP, false);

                // this.contentFrameE.style.flexDirection = "row";
                this.contentFrameE.classList.remove('chat_on_top');

                var width = getItem$1(this.ATTACHED_CHAT_CONTAINER_WIDTH, window.innerWidth / 2);
                this.dragDependantV.setWidth(width);
                this.element.style.height = "";
                this.element.style.minHeight = "";

                this.moveUpBtnE.show();
                this.moveRightBtnE.hide();

                var icon = this.dragE.$(".icon");
                icon.classList.remove("icon-hdots");
                icon.classList.add("icon-vdots");

                this.dragE.dragV.isVertical = true;
                this.dragE.dragV.direction = -1;
            }
        }, {
            key: "collapse",
            value: function collapse() {
                setItem$1(this.CHAT_CONTAINER_COLLAPSED, true);

                this.element.classList.add('chat_collapsed');
                this.element.style.top = "calc(100% - var(--menu-height))";

                this.draggableWindowV.setTopFixed(true);
                chatVisibilityChange(true);
            }
        }, {
            key: "expand",
            value: function expand() {
                setItem$1(this.CHAT_CONTAINER_COLLAPSED, false);

                this.element.classList.remove('chat_collapsed');
                this.element.style.top = this._getTop() + "px";

                this.draggableWindowV.setTopFixed(false);
                chatVisibilityChange(false);
            }
        }, {
            key: "isDetached",
            value: function isDetached() {
                return getItem$1(this.CHAT_CONTAINER_DETACHED, false);
            }
        }, {
            key: "isOnTop",
            value: function isOnTop() {
                return getItem$1(this.CHAT_CONTAINER_ON_TOP, false);
            }
        }, {
            key: "isCollapsed",
            value: function isCollapsed() {
                return this.isDetached() && getItem$1(this.CHAT_CONTAINER_COLLAPSED, false);
            }
        }, {
            key: "_getWidth",
            value: function _getWidth() {
                return getItem$1(this.DETACHED_CHAT_CONTAINER_WIDTH, window.innerWidth / 2);
            }
        }, {
            key: "_getHeight",
            value: function _getHeight() {
                return getItem$1(this.DETACHED_CHAT_CONTAINER_HEIGHT, window.innerHeight / 2);
            }
        }, {
            key: "_getLeft",
            value: function _getLeft() {
                return getItem$1(this.CHAT_CONTAINER_LEFT, Math.max(0, window.innerWidth - this._getWidth()) / 2);
            }
        }, {
            key: "_getTop",
            value: function _getTop() {
                return getItem$1(this.CHAT_CONTAINER_TOP, Math.max(0, window.innerHeight - this._getHeight()) / 2);
            }
        }, {
            key: "_onLsChange",
            value: function _onLsChange(e) {
                if (!e.isFromForeignWindow) return;

                if (e.key === this.CHAT_CONTAINER_DETACHED) {
                    e.newValue ? this.detach() : this.attach();
                }

                if (e.key === this.CHAT_CONTAINER_ON_TOP) {
                    e.newValue ? this.moveUp() : this.moveRight();
                }

                if (e.key === this.CHAT_CONTAINER_COLLAPSED) {
                    e.newValue ? this.collapse() : this.expand();
                }

                if (e.key === this.DETACHED_CHAT_CONTAINER_WIDTH || e.key === this.DETACHED_CHAT_CONTAINER_HEIGHT || e.key === this.CHAT_CONTAINER_LEFT || e.key === this.CHAT_CONTAINER_TOP) {
                    if (this.element.classList.contains('detached')) {
                        this.detach();
                    }
                }
            }
        }]);

        return ChatContainerV;
    }();

    function isChatOrDocumentHidden() {
        return document.hidden || isChatHidden();
    }

    function isChatHidden() {
        var chatContainerE = $('#chat_container');
        return chatContainerE.isHidden() || chatContainerE.view.isCollapsed();
    }

    var _createClass$c = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$g(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ServiceLoader = function () {
        function ServiceLoader(cssUrl, scriptUrl) {
            _classCallCheck$g(this, ServiceLoader);

            this._cssUrl = cssUrl;
            this._scriptUrl = scriptUrl;

            this._isServiceLoaded = false;
            this._isServiceEnabled = false;
            this._serviceLoadPromise = null;
        }

        _createClass$c(ServiceLoader, [{
            key: "enableService",
            value: function enableService(isEnabled) {
                var _this = this;

                this._isServiceEnabled = isEnabled;

                // load it if enabled
                if (!isEnabled) {
                    return Promise.resolve();
                }

                if (this._isServiceLoaded) {
                    // already loaded
                    return Promise.resolve();
                }

                if (this._serviceLoadPromise) {
                    // it is loading at the moment
                    return this._serviceLoadPromise;
                }

                this._serviceLoadPromise = this._loadService().then(function () {
                    return _this._isServiceLoaded = true;
                });
                return this._serviceLoadPromise;
            }
        }, {
            key: "_loadService",
            value: function _loadService() {
                return Promise.all([importCss(this._cssUrl), importScript(this._scriptUrl)]);
            }
        }, {
            key: "isServiceEnabled",
            get: function get() {
                return this._isServiceEnabled;
            }
        }]);

        return ServiceLoader;
    }();

    // file: katex_service.js

    var serviceLoader = new ServiceLoader(config.katex.cssPath, config.katex.scriptPath);

    function enableKatex(isEnabled) {
        return serviceLoader.enableService(isEnabled);
    }

    function renderKatex(formula, element) {
        var plain = "$" + formula + "$";

        if (!serviceLoader.isServiceEnabled) {
            element.appendText(plain);
            return;
        }

        try {
            katex.render(formula, element, { throwOnError: false });
        } catch (e) {
            // console.warn('renderKatex exception:', e);
            element.appendText(plain);
        }
    }

    function _classCallCheck$h(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    // file: thumbnail_v.js
    var ThumbnailV = function ThumbnailV(url) {
        _classCallCheck$h(this, ThumbnailV);

        var pictureE = document.createElement("picture");

        this.element = pictureE;
        pictureE.thumbnailV = this;

        this.imgE = document.createElement("img");
        this.imgE.referrerPolicy = "no-referrer";
        this.imgE.title = url;
        this.imgE.alt = url;

        pictureE.appendChild(this.imgE);
    };

    function getYoutubeThumbnailV(url) {
        var videoId = getYoutubeVideoId(url);
        if (!videoId) return null;

        var thumbnailV = new ThumbnailV(url);
        var imgE = thumbnailV.imgE;
        imgE.classList.add("youtube_thumb");
        imgE.src = "https://img.youtube.com/vi/" + videoId + "/mqdefault.jpg";
        return thumbnailV;
    }

    function getYoutubeVideoId(url) {
        var regExp = /^https?:\/\/(?:www\.|m\.)?(?:youtube\.com|youtu\.be)\/(?:[\w\-]+\?v=|embed\/|v\/)?([\w\-]+)/;
        var match = url.match(regExp);
        return match && match[1].length === 11 ? match[1] : null;
    }

    function getPloudfrThumbnailV(url) {
        var regExp = /^https?:\/\/(?:www\.)?video.ploud.fr\/videos\/(?:watch|embed)\/([\w\-]+)/;
        return getPeerTubeThumbnailV(url, 'video.ploud.fr', 'ploudfr_thumb', regExp);
    }

    function getLibreVideoThumbnailV(url) {
        var regExp = /^https?:\/\/(?:www\.)?libre.video\/videos\/(?:watch|embed)\/([\w\-]+)/;
        return getPeerTubeThumbnailV(url, 'libre.video', 'librevideo_thumb', regExp);
    }

    function getPeertubeSocialThumbnailV(url) {
        var regExp = /^https?:\/\/(?:www\.)?peertube.social\/videos\/(?:watch|embed)\/([\w\-]+)/;
        return getPeerTubeThumbnailV(url, 'peertube.social', 'peertubesocial_thumb', regExp);
    }

    function getPeerTubeThumbnailV(url, domain, thumbClass, regExp) {
        var match = url.match(regExp);
        if (!match) return null;

        var videoId = match[1];
        var thumbnailV = new ThumbnailV(url);
        var imgE = thumbnailV.imgE;
        imgE.classList.add(thumbClass);
        imgE.src = "https://" + domain + "/static/thumbnails/" + videoId + ".jpg";
        return thumbnailV;
    }

    function getImgurThumbnailV(url) {
        var regExp = /^.*imgur.com\/(\w{6,7})\w?\.(\w{3})/;
        var match = regExp.exec(url);
        if (!match) return null;

        var imageId = match[1];
        var imageExt = match[2];
        var thumbId = imageId + "m";
        var thumbnailV = new ThumbnailV(url);
        var imgE = thumbnailV.imgE;
        imgE.classList.add("imgur_thumb");
        imgE.src = "https://i.imgur.com/" + thumbId + "." + imageExt;
        return thumbnailV;
    }

    function getTwitterThumbnailV(url) {
        var regExp = /^(.*pbs.twimg.com\/media\/[\w_-]+)(?:\.(\w{3,4})|\?format=(\w{3,4}))/;
        var match = regExp.exec(url);
        if (!match) return null;

        var thumbnailV = new ThumbnailV(url);
        var imgE = thumbnailV.imgE;
        imgE.classList.add("twitter_thumb");
        imgE.src = match[1] + "." + (match[2] || match[3]) + ":thumb";
        return thumbnailV;
    }

    // file: settings_service.js

    var currentSettings = null;

    function getSettings() {
        if (!currentSettings) {
            currentSettings = getItem$1('settings', config.defaultSettings);
        }
        return currentSettings;
    }

    function saveSettings(settings) {
        setItem$1('settings', settings);
        currentSettings = settings;
    }

    function addSettingsChangeEventListener(onSettingsChange) {
        document.addEventListener('lsChange', function (e) {
            if (e.detail.key === 'settings') {
                var oldSettings = e.detail.oldValue || config.defaultSettings;
                var newSettings = e.detail.newValue;
                currentSettings = newSettings;

                onSettingsChange({ oldSettings: oldSettings, newSettings: newSettings });
            }
        }, false);
    }

    function _classCallCheck$i(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    // file: embed_v.js
    var EmbedV = function EmbedV(url) {
        _classCallCheck$i(this, EmbedV);

        var iframeE = document.createElement("iframe");
        iframeE.setAttribute('allowFullScreen', '');

        this.element = iframeE;
        iframeE.thumbnailV = this;
    };

    function getBitchuteEmbedV(url) {
        var regExp = /^https?:\/\/(?:www\.)?bitchute.com\/(?:video|embed|torrent\/[^\/]+)\/([^\/?#&]+)/;
        var match = regExp.exec(url);
        if (!match) return null;

        var videoId = match[1];
        var embedV = new EmbedV(url);
        var iframeE = embedV.element;
        iframeE.classList.add("bitchute_thumb");

        iframeE.src = 'https://www.bitchute.com/embed/' + videoId;
        iframeE.title = url;

        return embedV;
    }

    function _classCallCheck$j(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var LinkV = function LinkV(url, name) {
        _classCallCheck$j(this, LinkV);

        var aE = document.createElement("a");
        aE.target = '_blank';
        aE.href = url;
        aE.referrerPolicy = "no-referrer";
        aE.rel = "noreferrer";
        aE.textContent = name ? name : tryDecodeURI(url);

        this.element = aE;
        aE.linkV = this;

        formatLink(aE, url);
    };

    function formatLink(aE, url) {
        var settings = getSettings();

        if (settings.embedBitchute) {
            var _thumbV = getBitchuteEmbedV(url);
            if (_thumbV) {
                addThumb(_thumbV, "bitchute_link");
                aE.appendChild(document.createElement("br"));
                aE.appendText(tryDecodeURI(url));
                return;
            }
        }

        if (!settings.showImages) {
            return;
        }

        var thumbV = getYoutubeThumbnailV(url);
        if (thumbV) {
            addThumb(thumbV, "youtube_link");
            return;
        }
        thumbV = getImgurThumbnailV(url);
        if (thumbV) {
            // todo: refactor this block
            addThumb(thumbV, "imgur_link");
            // aE.id = Date.now();

            // thumbV.onload = function () {
            //     aE = document.getElementById(aE.id);
            //     var msgDiv = getClosestByClassName(aE, "msg");
            //     var msgHeightBefore = parseInt(getComputedStyle(msgDiv).height);

            //     addThumb(thumbV, "imgur-link");

            //     var msgHeightAfter = parseInt(getComputedStyle(msgDiv).height);
            //     var addedHeight = msgHeightAfter - msgHeightBefore;

            //     var chatFrameDiv = document.getElementById("chatframe");
            //     chatFrameDiv.scrollTop += addedHeight;
            // };
            return;
        }
        thumbV = getTwitterThumbnailV(url);
        if (thumbV) {
            addThumb(thumbV, "twitter_link");
            return;
        }
        thumbV = getPloudfrThumbnailV(url);
        if (thumbV) {
            addThumb(thumbV, "ploudfr_link");
            return;
        }
        thumbV = getLibreVideoThumbnailV(url);
        if (thumbV) {
            addThumb(thumbV, "librevideo_link");
            return;
        }
        thumbV = getPeertubeSocialThumbnailV(url);
        if (thumbV) {
            addThumb(thumbV, "peertubesocial_link");
            return;
        }

        function addThumb(thumbV, className) {
            aE.textContent = "";
            aE.classList.add(className);
            aE.appendChild(thumbV.element);
        }
    }

    // file: page_number_service.js

    function getThreadPageNumber(skipped) {
        var pageSize = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : config.pageSize.general;

        // Note: page number starts form 0
        // Note: skipped starts from 1 because we always skip OP

        return getPageNumber(skipped - 1, pageSize);
    }

    function getPageNumber(skipped) {
        var pageSize = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : config.pageSize.general;

        // Note: page number starts form 0

        return skipped <= 0 ? 0 : Math.floor(skipped / pageSize);
    }

    var _createClass$d = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$k(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var NotificationV = function () {
        function NotificationV(notificationType, content, autoHideAfter) {
            var _this = this;

            _classCallCheck$k(this, NotificationV);

            if (!(notificationType in config.notification.type)) {
                throw new Error('Unsupported notification type: ' + notificationType);
            }

            var notificationE = document.createElement('div');

            this.element = notificationE;
            notificationE.notificationV = this;

            notificationE.classList.add('notification');
            notificationE.classList.add('notification_' + notificationType);

            var notificationContentE = document.createElement('div');
            notificationContentE.classList.add('notification_content');
            notificationContentE.appendAny(content);

            notificationE.appendChild(notificationContentE);

            var closeBtnE = document.createElement('button');
            closeBtnE.classList.add('close_btn');
            closeBtnE.classList.add('icon-close');

            notificationE.appendChild(closeBtnE);

            closeBtnE.addEventListener('click', function (e) {
                return _this._close();
            }, false);

            setTimeout(function () {
                return _this._close();
            }, autoHideAfter);
        }

        _createClass$d(NotificationV, [{
            key: '_close',
            value: function _close() {
                this.element.remove();
            }
        }]);

        return NotificationV;
    }();

    function showNotification(notificationType, content) {
        var autoHideAfter = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : config.notification.autoHideAfter;

        var notificationContainerE = $("#notification_container");

        var notificationV = new NotificationV(notificationType, content, autoHideAfter);
        notificationContainerE.appendChild(notificationV.element);
    }

    var _createClass$e = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$l(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var PostLinkV = function () {
        function PostLinkV(postId, linkText, title) {
            var _this = this;

            _classCallCheck$l(this, PostLinkV);

            this.postId = postId;

            var linkE = document.createElement("a");
            linkE.href = getPostLinkUrl(postId);
            if (title) linkE.title = title;
            linkE.appendText(linkText);

            this.element = linkE;
            linkE.postLinkV = this;

            linkE.addEventListener('click', function (e) {
                return _this._onClick(e);
            }, false);
        }

        _createClass$e(PostLinkV, [{
            key: "_onClick",
            value: function _onClick(e) {
                var _this2 = this;

                e.preventDefault();

                var hash = "#p" + this.postId;
                var postE = $(hash);
                if (postE && getThreadId()) {
                    setPostIdHash$1(hash);
                    return;
                }

                getPost$1(this.postId, true).then(function (post) {
                    return setThreadPage(post.thread.board.name, post.thread.threadId, getThreadPageNumber(post.index), _this2.postId);
                }).catch(function (e) {
                    return showNotification(config.notification.type.error, e.message);
                });
            }
        }]);

        return PostLinkV;
    }();

    var _createClass$f = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$m(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var PostReferenceV = function () {
        function PostReferenceV(postId) {
            var _this = this;

            _classCallCheck$m(this, PostReferenceV);

            this.postId = postId;
            this._loadPostPreviewTimeout = 100;
            this._requestTimetoutId = null;

            var refE = document.createElement("span");
            refE.classList.add("post_ref");

            this.element = refE;
            refE.postReferenceV = this;

            var postLinkV = new PostLinkV(postId, "@" + toShortcutUuid(postId));
            this._linkE = postLinkV.element;
            refE.appendChild(this._linkE);

            refE.addEventListener('mouseenter', function (e) {
                return _this._onMouseEnter(e);
            }, false);
            refE.addEventListener('mouseleave', function (e) {
                return _this._onMouseLeave(e);
            }, false);
        }

        _createClass$f(PostReferenceV, [{
            key: "_onMouseEnter",
            value: function _onMouseEnter(e) {
                var _this2 = this;

                if (this._postPreviewExists() || this._requestTimetoutId) return;

                this._linkE.style.cursor = 'wait';

                this._requestTimetoutId = setTimeout(function () {
                    getPost$1(_this2.postId).then(function (p) {
                        if (!_this2._requestTimetoutId) return;

                        _this2._showPostPreview(p);
                        _this2._requestTimetoutId = null;
                        _this2._linkE.style.cursor = "";
                    }).catch(function (e) {
                        _this2._linkE.style.cursor = "";
                        showNotification(config.notification.type.error, e.message);
                    });
                }, this._loadPostPreviewTimeout);
            }
        }, {
            key: "_onMouseLeave",
            value: function _onMouseLeave(e) {
                clearTimeout(this._requestTimetoutId);
                this._requestTimetoutId = null;

                if (!this._postPreviewExists() || !e.relatedTarget) return;

                var relatedPostPreviewE = e.relatedTarget.closest('.post_preview');
                if (relatedPostPreviewE === this._postPreviewV.element) return;

                this._removePostPreview();
            }
        }, {
            key: "_showPostPreview",
            value: function _showPostPreview(post) {
                var rect = this.element.getBoundingClientRect();

                this._postPreviewV = new PostPreviewV(post, rect);
            }
        }, {
            key: "_removePostPreview",
            value: function _removePostPreview() {
                if (!this._postPreviewExists()) return;

                this._postPreviewV.remove();
                this._postPreviewV = null;
            }
        }, {
            key: "_postPreviewExists",
            value: function _postPreviewExists() {
                return this._postPreviewV && this._postPreviewV.element;
            }
        }]);

        return PostReferenceV;
    }();

    function _classCallCheck$n(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var PostBodyV = function PostBodyV(post) {
        _classCallCheck$n(this, PostBodyV);

        this.post = post;

        var bodyE = document.createElement('div');
        bodyE.classList.add('body');

        // more that 2 newlines replace with 2 newlines
        var text = post.text.replace(/\n{3,}/g, "\n\n");

        formatText(text, bodyE);

        this.element = bodyE;
        bodyE.postBodyV = this;
    };

    function formatText(text, element) {
        var level = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;

        if (level > config.formatting.maxNestingLevel) {
            element.appendText(text);
            return;
        }

        for (var i = 0; i < text.length; i++) {
            var textToParse = text.slice(i);
            var markupFreePrefix = text.slice(0, i);

            var newline = parserNewline(textToParse);
            if (newline) {
                element.appendText(markupFreePrefix);
                element.appendChild(document.createElement('br'));
                formatText(newline.rest, element, level);
                return;
            }
            var quote = parseQuote(textToParse, markupFreePrefix, element, level);
            if (quote) {
                element.appendText(markupFreePrefix);
                if (quote.afterPostRef) element.appendChild(document.createElement('br'));
                var quoteE = document.createElement('span');
                quoteE.classList.add('quote');
                element.appendChild(quoteE);

                formatText(quote.match, quoteE, level + 1);
                formatText(quote.rest, element, level);
                return;
            }
            var bold = parseBold(textToParse);
            if (bold) {
                element.appendText(markupFreePrefix);
                var bE = document.createElement('b');
                element.appendChild(bE);

                formatText(bold.match, bE, level + 1);
                formatText(bold.rest, element, level);
                return;
            }
            var italic = parseItalic(textToParse);
            if (italic) {
                element.appendText(markupFreePrefix);
                var iE = document.createElement('i');
                element.appendChild(iE);

                formatText(italic.match, iE, level + 1);
                formatText(italic.rest, element, level);
                return;
            }
            var code = parseCode(textToParse);
            if (code) {
                element.appendText(markupFreePrefix);
                var preE = document.createElement('pre');
                element.appendChild(preE);

                var codeE = document.createElement('code');
                if (code.lang) codeE.classList.add(code.lang);
                codeE.appendText(code.match);
                preE.appendChild(codeE);

                formatText(code.rest, element, level);
                return;
            }
            var mono = parseMono(textToParse);
            if (mono) {
                element.appendText(markupFreePrefix);
                var sampE = document.createElement('samp');
                element.appendChild(sampE);

                formatText(mono.match, sampE, level + 1);
                formatText(mono.rest, element, level);
                return;
            }
            var strike = parseStrike(textToParse);
            if (strike) {
                element.appendText(markupFreePrefix);
                var sE = document.createElement('s');
                element.appendChild(sE);

                formatText(strike.match, sE, level + 1);
                formatText(strike.rest, element, level);
                return;
            }
            var tex = parseTeX(textToParse);
            if (tex) {
                element.appendText(markupFreePrefix);
                var texE = document.createElement('span');
                element.appendChild(texE);
                renderKatex(tex.match, texE);

                formatText(tex.rest, element, level);
                return;
            }
            var spoiler = parseSpoiler(textToParse);
            if (spoiler) {
                element.appendText(markupFreePrefix);
                var spanE = document.createElement('span');
                spanE.classList.add("spoiler");
                element.appendChild(spanE);

                formatText(spoiler.match, spanE, level + 1);
                formatText(spoiler.rest, element, level);
                return;
            }
            var namedLink = parseNamedLink(textToParse);
            if (namedLink) {
                element.appendText(markupFreePrefix);
                var linkV = new LinkV(namedLink.match, namedLink.name);
                element.appendChild(linkV.element);

                formatText(namedLink.rest, element, level);
                return;
            }
            var link = parseLink(textToParse);
            if (link) {
                element.appendText(markupFreePrefix);
                var _linkV = new LinkV(link.match, null);
                element.appendChild(_linkV.element);

                formatText(link.rest, element, level);
                return;
            }
            var postReference = parsePostReference(textToParse);
            if (postReference) {
                element.appendText(markupFreePrefix);
                var postReferenceV = new PostReferenceV(postReference.match);
                element.appendChild(postReferenceV.element);

                formatText(postReference.rest, element, level);
                return;
            }
        }
        element.appendText(text);
    }

    // file: filter_service.js
    function containCyrillic(str) {
        for (var i = 0; i < str.length; i++) {
            var charCode = str.charCodeAt(i);
            if (charCode > 1023 && charCode < 1280) return true;
        }
        return false;
    }

    function getShannonEntropy(text) {
        var set = {};

        text.split('').forEach(function (c) {
            return set[c] ? set[c]++ : set[c] = 1;
        });

        return Object.keys(set).reduce(function (acc, c) {
            var p = set[c] / text.length;
            return acc - p * (Math.log(p) / Math.log(2));
        }, 0);
    }

    var regexpFiltersStr = '';
    var filtersRegexps = []; // todo: use Set
    function parseRegexpFilters(str) {
        // we memorize the last result because we don't want to parse the same regexps for each post
        if (str === regexpFiltersStr) return filtersRegexps;

        filtersRegexps = [];
        regexpFiltersStr = str || '';

        regexpFiltersStr.split('\n').filter(function (x) {
            return x;
        }).forEach(function (x) {
            try {
                filtersRegexps.push(new RegExp(x, 'i'));
            } catch (e) {
                console.warn("Invalid regexp: ", x, e);
            }
        });

        return filtersRegexps;
    }

    var colorsFiltersStr = '';
    var colorsFilters = [];
    function parseColorsFilters(str) {
        if (str === colorsFiltersStr) return colorsFilters;

        colorsFilters = [];
        colorsFiltersStr = str || '';

        var colorsFiltersSet = new Set();

        colorsFiltersStr.split('\n').filter(function (x) {
            return x;
        }).forEach(function (x) {
            return colorsFiltersSet.add(x);
        });

        colorsFilters = Array.from(colorsFiltersSet).map(function (x) {
            var match = /^([0-9]+):([0-9]+)$/.exec(x);
            if (!match) return;

            return {
                bg: Number.parseInt(match[1]),
                fg: Number.parseInt(match[2])
            };
        }).filter(function (x) {
            return x;
        });

        return colorsFilters;
    }

    var _createClass$g = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$o(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var PostV = function () {
        function PostV(post, isOriginalPost) {
            _classCallCheck$o(this, PostV);

            this.post = post;

            var postWrapperE = document.createElement('div');
            postWrapperE.classList.add('post_wrapper');

            this.element = postWrapperE;
            this.postV = this;

            var postE = document.createElement('div');
            this.postE = postE;
            postE.classList.add('post');
            if (isOriginalPost) {
                postE.classList.add('original');
            }
            postWrapperE.appendChild(postE);

            var headerE = document.createElement('div');
            this.headerE = headerE;
            headerE.classList.add('header');

            var idE = document.createElement('span');
            this.idE = idE;
            idE.classList.add('id');
            idE.style.color = post.fg.toHSL();
            if (post.ephemeral) {
                idE.style.backgroundImage = "repeating-linear-gradient(\n            -45deg, transparent, transparent 1px, " + post.bg.toHSL() + " 2px, " + post.bg.toHSL() + " 10px)";
            } else {
                idE.style.backgroundColor = post.bg.toHSL();
            }
            idE.appendText(toShortcutUuid(post.postId));

            var dateE = document.createElement('span');
            dateE.classList.add('date');
            dateE.appendText(PostV.formatDate(post.postDate));

            var nameE = document.createElement('span');
            nameE.classList.add('name');
            if (post.user.banned) nameE.classList.add('banned');
            nameE.appendText(post.user.nickname);

            headerE.appendChild(idE);
            headerE.appendText(' ');
            headerE.appendChild(nameE);
            headerE.appendText(' ');
            headerE.appendChild(dateE);
            postE.appendChild(headerE);

            if (!isOriginalPost && !post.ephemeral && post.thread.threadId !== config.chat.threadId) {
                this._addHideByOpButtons();
            }

            var isPostHidden = this.post.hiddenByOp || getSettings().hideEphemeralPosts && this.post.ephemeral || getSettings().ignoreCyrillicShit && containCyrillic(post.text) || getSettings().maxEntropy > 0 && getShannonEntropy(post.text) > getSettings().maxEntropy || parseRegexpFilters(getSettings().regexpFilters).some(function (re) {
                return re.test(post.text);
            }) || parseColorsFilters(getSettings().colorsFilters).some(function (_ref) {
                var bg = _ref.bg,
                    fg = _ref.fg;
                return post.bg === bg && post.fg === fg;
            });

            this._addHideUnhideButtons(isPostHidden);
        }

        _createClass$g(PostV, [{
            key: "changeBoardDescription",
            value: function changeBoardDescription(boardDescription) {
                this.post.thread.board.description = boardDescription;
            }
        }, {
            key: "_createPostBody",
            value: function _createPostBody() {
                if (this.postBodyV) return;

                this.postBodyV = new PostBodyV(this.post);
                this.postE.appendChild(this.postBodyV.element);
            }
        }, {
            key: "_addHideUnhideButtons",
            value: function _addHideUnhideButtons(isPostHidden) {
                var _this = this;

                if (this.post.ephemeral) {
                    if (getSettings().hideEphemeralPosts) {
                        this.element.hide();
                    } else {
                        this.element.show();
                    }
                }

                this.headerE.appendText(' ');

                this._unhideBtnE = newE('i', {
                    className: 'unhide_post_btn icon-eye',
                    title: "Unhide: hidden by " + (this.post.hiddenByOp ? "OP" : "you")
                });
                this._unhideBtnE.hide();
                this.headerE.appendChild(this._unhideBtnE);

                this._hideBtnE = newE('i', {
                    className: 'hide_post_btn icon-eye-blocked',
                    title: 'Hide (only for you)'
                });
                this._hideBtnE.hide();
                this.headerE.appendChild(this._hideBtnE);

                if (isPostHidden) {
                    this.postE.classList.add('hidden_post');
                    this._unhideBtnE.show();
                } else {
                    this._createPostBody();
                }

                this._unhideBtnE.addEventListener('click', function (e) {
                    return _this._unhide();
                }, false);

                this._hideBtnE.addEventListener('click', function (e) {
                    return _this._hide();
                }, false);
            }
        }, {
            key: "_hide",
            value: function _hide() {
                this.postE.classList.add('hidden_post');

                this._unhideBtnE.show();
                this._hideBtnE.hide();
            }
        }, {
            key: "_unhide",
            value: function _unhide() {
                this._createPostBody();
                this.postE.classList.remove('hidden_post');

                this._hideBtnE.show();
                this._unhideBtnE.hide();
            }
        }, {
            key: "_addHideByOpButtons",
            value: function _addHideByOpButtons() {
                var _this2 = this;

                this.headerE.appendText(' ');

                var unhideByOpBtnE = newE('i', {
                    className: 'unhide_by_op_btn icon-plus-square',
                    title: 'Unhide for all (you have to be OP)'
                });
                this.headerE.appendChild(unhideByOpBtnE);
                if (!this.post.hiddenByOp) unhideByOpBtnE.hide();

                var hideByOpBtnE = newE('i', {
                    className: 'hide_by_op_btn icon-minus-square',
                    title: 'Hide for all (you have to be OP)'
                });
                this.headerE.appendChild(hideByOpBtnE);
                if (this.post.hiddenByOp) hideByOpBtnE.hide();

                unhideByOpBtnE.addEventListener('click', function (e) {
                    updateHiddenByOp$2(new UpdatedHiddenByOp(null, _this2.post.postId, false)).then(function (_) {
                        _this2._unhide();
                        unhideByOpBtnE.hide();
                        hideByOpBtnE.show();
                        showNotification(config.notification.type.success, "The post was unhidden.");
                    }).catch(function (e) {
                        if (e.message === "Invalid clientId") {
                            showNotification(config.notification.type.error, "You're not OP of this thread: invalid clientId", 5000);
                        } else {
                            showNotification(config.notification.type.error, "Failed to unhide the post: " + e.message);
                        }
                    });
                }, false);

                hideByOpBtnE.addEventListener('click', function (e) {
                    updateHiddenByOp$2(new UpdatedHiddenByOp(null, _this2.post.postId, true)).then(function (_) {
                        _this2._hide();
                        hideByOpBtnE.hide();
                        unhideByOpBtnE.show();
                        showNotification(config.notification.type.success, "The post was hidden.");
                    }).catch(function (e) {
                        if (e.message === "Invalid clientId") {
                            showNotification(config.notification.type.error, "You're not OP of this thread: invalid clientId", 5000);
                        } else {
                            showNotification(config.notification.type.error, "Failed to hide the post: " + e.message);
                        }
                    });
                }, false);
            }
        }], [{
            key: "formatDate",
            value: function formatDate(postDate) {
                return postDate.toISOString().slice(0, 19).replace('T', ' ');
            }
        }]);

        return PostV;
    }();

    var _createClass$h = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$p(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var PostPreviewV = function () {
        function PostPreviewV(post, rect) {
            var _this = this;

            _classCallCheck$p(this, PostPreviewV);

            var postPreviewE = document.createElement('div');
            postPreviewE.classList.add('post_preview');

            this.element = postPreviewE;
            postPreviewE.postPreviewV = this;

            var postV = new PostV(post);
            postPreviewE.appendChild(postV.element);
            document.body.appendChild(postPreviewE);

            var totalWidth = document.documentElement.clientWidth;
            var totalHeight = document.documentElement.clientHeight;

            var onLeft = rect.left + rect.width / 2 <= totalWidth / 2;
            var onTop = rect.top + rect.height / 2 <= totalHeight / 2;

            if (onLeft) {
                postPreviewE.style.maxWidth = Math.floor(totalWidth - rect.left) + "px";
                postPreviewE.style.left = rect.left + "px";
            } else {
                postPreviewE.style.maxWidth = Math.floor(rect.right) + "px";
                postPreviewE.style.right = totalWidth - rect.right + "px";
            }

            if (onTop) {
                postPreviewE.style.maxHeight = Math.floor(totalHeight - rect.bottom) + "px";
                postPreviewE.style.top = rect.bottom + "px";
            } else {
                postPreviewE.style.maxHeight = Math.floor(rect.top) + "px";
                postPreviewE.style.top = rect.top - postPreviewE.offsetHeight + "px";
            }

            postPreviewE.addEventListener('mouseleave', function (e) {
                return _this._onMouseLeave(e);
            }, false);
        }

        _createClass$h(PostPreviewV, [{
            key: 'remove',
            value: function remove() {
                this.element.remove();
                this.element = null;
            }
        }, {
            key: '_onMouseLeave',
            value: function _onMouseLeave(e) {
                if (!e.relatedTarget) return;
                if (e.relatedTarget.closest('.post_ref')) return;

                var relatedPostPreviewE = e.relatedTarget.closest('.post_preview');
                var postPreviewsEs = $a('.post_preview');

                var indexOfThisPostPreview = postPreviewsEs.indexOf(this.element);
                var indexOfRelatedPostPreview = postPreviewsEs.indexOf(relatedPostPreviewE);

                if (indexOfRelatedPostPreview > indexOfThisPostPreview) return;

                for (var i = postPreviewsEs.length - 1; i >= indexOfThisPostPreview; i--) {
                    postPreviewsEs[i].postPreviewV.remove();
                }
            }
        }]);

        return PostPreviewV;
    }();

    function initRemovePostPreviewsOnClick() {
        document.body.addEventListener('click', function (e) {
            if (e.target.closest('.post_preview')) return;

            $a('.post_preview').each(function (postPreviewE) {
                return postPreviewE.postPreviewV.remove();
            });
        });
    }

    var _createClass$i = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$q(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var DragV = function () {
        function DragV(dragE, isVertical) {
            var _this = this;

            _classCallCheck$q(this, DragV);

            this.element = dragE;
            dragE.dragV = this;

            // for vertical drags we resize width
            // for horizontal drags we resize height
            this.isVertical = isVertical; // false means horizontal

            var previousOrNext = dragE.getAttribute('dependent_sibling');
            this.dependentE = dragE[previousOrNext + 'ElementSibling'];
            this.direction = previousOrNext === 'next' ? -1 : 1;

            this._initialClientX = null;
            this._initialClientY = null;
            this._initialWidth = null;
            this._initialHeight = null;
            this._isResized = false;

            dragE.addEventListener("mousedown", this._onMouseDownHandler = function (e) {
                return _this._onMouseDown(e);
            }, false);

            window.addEventListener("mouseup", this._onMouseUpHandler = function (e) {
                return _this._onMouseUp(e);
            }, false);

            document.body.addEventListener("mousemove", this._onMouseMoveHandler = function (e) {
                return _this._onMouseMove(e);
            }, false);
        }

        _createClass$i(DragV, [{
            key: '_onMouseDown',
            value: function _onMouseDown(e) {
                e.preventDefault();

                // start resizing
                this._initialClientX = e.clientX;
                this._initialClientY = e.clientY;
                this._initialWidth = getComputedWidth(this.dependentE);
                this._initialHeight = getComputedHeight(this.dependentE);
            }
        }, {
            key: '_onMouseUp',
            value: function _onMouseUp(e) {
                // if "mousedown" happened for this drag and it hasn't been resized yet.
                if (this._isMouseDown() && !this._isResized) {
                    this.dependentE.dragDependantV.toggleHide();
                }
                this._isResized = false;

                // stop resizing
                this._stopResizing();
            }
        }, {
            key: '_onMouseMove',
            value: function _onMouseMove(e) {
                if (!this._isMouseDown()) return;
                e.preventDefault();

                if (this.dependentE.isHidden()) {
                    this.dependentE.dragDependantV.toggleHide();
                    this._initialWidth = 0;
                    this._initialHeight = 0;
                }

                var widthDelta = e.clientX - this._initialClientX;
                var heightDelta = e.clientY - this._initialClientY;
                var width = Math.max(0, this._initialWidth + this.direction * widthDelta);
                var height = Math.max(0, this._initialHeight + this.direction * heightDelta);

                if (this.isVertical) {
                    this.dependentE.dragDependantV.setWidth(width);
                } else {
                    this.dependentE.dragDependantV.setHeight(height);
                }

                this._isResized = true;
            }
        }, {
            key: '_isMouseDown',
            value: function _isMouseDown() {
                return !!this._initialClientX && !!this._initialClientY;
            }
        }, {
            key: '_stopResizing',
            value: function _stopResizing() {
                this._initialClientX = null;
                this._initialClientY = null;
            }
        }]);

        return DragV;
    }();

    function enableDrags() {
        $a(".drag").each(function (dragE) {
            return new DragV(dragE, true);
        });
    }

    function _classCallCheck$r(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var HomeLinkV = function HomeLinkV() {
        _classCallCheck$r(this, HomeLinkV);

        var linkE = $("#home_link");
        linkE.href = config.basePath + "/";

        this.element = linkE;
        linkE.homeLinkV = this;

        linkE.addEventListener('click', function (e) {
            e.preventDefault();
            setHome();
        }, false);
    };

    function _classCallCheck$s(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var SearchLinkV = function SearchLinkV() {
        _classCallCheck$s(this, SearchLinkV);

        var linkE = $("#search_link");

        this.element = linkE;
        linkE.searchLinkV = this;

        linkE.href = getSearchBaseUrl();

        linkE.addEventListener('click', function (e) {
            e.preventDefault();
            setSearch();
        }, false);
    };

    var _createClass$j = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$t(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ConnectionStatusV = function () {
        function ConnectionStatusV() {
            var _this = this;

            _classCallCheck$t(this, ConnectionStatusV);

            var connectionStatusE = $('#connection_status');

            this.element = connectionStatusE;
            connectionStatusE.connectionStatusV = this;

            this._iconE = connectionStatusE.$('.disk');
            connectionStatusE.title = 'disconnected';

            addConnectedEventListener(function () {
                return _this._connected();
            });
            addErrorEventListener(function () {
                return _this._disconnected();
            });
        }

        _createClass$j(ConnectionStatusV, [{
            key: "_connected",
            value: function _connected() {
                this.element.title = 'connected';
                this._iconE.classList.add('connected');
            }
        }, {
            key: "_disconnected",
            value: function _disconnected() {
                if (!this._iconE.classList.contains('connected')) return;

                this.element.title = 'disconnected';
                this._iconE.classList.remove('connected');

                showNotification(config.notification.type.error, "Connection is lost.");
            }
        }]);

        return ConnectionStatusV;
    }();

    var _createClass$k = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$u(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var PopupV = function () {
        function PopupV() {
            _classCallCheck$u(this, PopupV);

            var popupE = document.createElement('div');
            this.element = popupE;
            popupE.popupV = this;

            popupE.classList.add('popup');
            document.body.appendChild(popupE);

            this.titleBarE = document.createElement('div');
            this.titleBarE.classList.add('title_bar');
            popupE.appendChild(this.titleBarE);

            new DraggableWindowV(popupE);

            this.popupContentE = document.createElement('div');
            this.popupContentE.classList.add('popup_content');
            popupE.appendChild(this.popupContentE);
        }

        _createClass$k(PopupV, [{
            key: 'close',
            value: function close() {
                this.element.remove();
            }
        }]);

        return PopupV;
    }();

    function _classCallCheck$v(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var PopupWithCloseBtnV = function PopupWithCloseBtnV(popupV, close, title) {
        var _this = this;

        var isCloseShortcut = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : function () {
            return false;
        };

        _classCallCheck$v(this, PopupWithCloseBtnV);

        decorateView(popupV, this, 'popupV');

        var closeBtnE = document.createElement('button');
        closeBtnE.type = 'button';
        closeBtnE.classList.add('close_btn');
        closeBtnE.classList.add('icon-close');
        if (title) closeBtnE.title = title;
        popupV.titleBarE.appendChild(closeBtnE);

        closeBtnE.addEventListener('click', function (e) {
            return close ? close() : _this.close();
        }, false);

        this.close = function () {
            document.removeEventListener('keydown', _this._onKeyDownHandler, false);
            popupV.close();
        };

        document.addEventListener('keydown', this._onKeyDownHandler = function (e) {
            if (isCloseShortcut(e)) close ? close() : _this.close();
        }, false);
    };

    function _classCallCheck$w(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ModalPopupD = function ModalPopupD(popupV, close) {
        var _this = this;

        _classCallCheck$w(this, ModalPopupD);

        decorateView(popupV, this, 'popupV');

        var modalShadeE = document.createElement('div');
        modalShadeE.id = 'modal_shade';
        modalShadeE.appendBefore(popupV.element);

        this.close = function () {
            modalShadeE.remove();
            popupV.close();
        };

        modalShadeE.addEventListener('click', function (e) {
            return close ? close() : _this.close();
        }, false);
    };

    var _createClass$l = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$x(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var PopupSavingStateD = function () {
        function PopupSavingStateD(popupV) {
            var _this = this;

            var defaultWidth = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 600;
            var defaultHeight = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 500;

            _classCallCheck$x(this, PopupSavingStateD);

            var popupId = popupV.element.id;

            if (!popupId) throw new Error('The popup must have an id!');

            decorateView(popupV, this, 'popupV');
            this.element = popupV.element;

            this.lsKeyWidth = "popup_" + popupId + "_width";
            this.lsKeyHeight = "popup_" + popupId + "_height";
            this.lsKeyLeft = "popup_" + popupId + "_left";
            this.lsKeyTop = "popup_" + popupId + "_top";

            var width = getItem$1(this.lsKeyWidth, Math.min(defaultWidth, window.innerWidth));
            var height = getItem$1(this.lsKeyHeight, Math.min(defaultHeight, window.innerHeight));

            this.element.style.width = width + "px";
            this.element.style.height = height + "px";

            var left = getItem$1(this.lsKeyLeft, Math.max(0, window.innerWidth - width) / 2);
            var top = getItem$1(this.lsKeyTop, Math.max(0, window.innerHeight - height) / 2);

            this.element.style.left = left + "px";
            this.element.style.top = top + "px";

            document.addEventListener('mouseup', this._onMouseUpHandler = function () {
                _this.saveStateOnMouseUp();
            }, false);

            document.addEventListener('lsChange', this._onLsChangeHandler = function (e) {
                return _this._onLsChange(e.detail);
            }, false);

            this.close = function () {
                document.removeEventListener('mouseup', _this._onMouseUpHandler, false);
                document.removeEventListener('lsChange', _this._onLsChangeHandler, false);
                popupV.close();
            };
        }

        _createClass$l(PopupSavingStateD, [{
            key: "saveStateOnMouseUp",
            value: function saveStateOnMouseUp() {
                var style = getComputedStyle(this.element);

                setItem$1(this.lsKeyWidth, parseCssDimension(style.width));
                setItem$1(this.lsKeyHeight, parseCssDimension(style.height));
                setItem$1(this.lsKeyLeft, parseCssDimension(style.left));
                setItem$1(this.lsKeyTop, parseCssDimension(style.top));
            }
        }, {
            key: "_onLsChange",
            value: function _onLsChange(e) {
                if (!e.isFromForeignWindow) return;

                if (e.key === this.lsKeyWidth) this.element.style.width = e.newValue + "px";
                if (e.key === this.lsKeyHeight) this.element.style.height = e.newValue + "px";
                if (e.key === this.lsKeyLeft) this.element.style.left = e.newValue + "px";
                if (e.key === this.lsKeyTop) this.element.style.top = e.newValue + "px";
            }
        }]);

        return PopupSavingStateD;
    }();

    var _createClass$m = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$y(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    // file: textbox_v.js
    var TextboxV = function () {
        function TextboxV() {
            var afterResize = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : function () {};

            _classCallCheck$y(this, TextboxV);

            var tbxE = document.createElement('textarea');
            tbxE.setAttribute("type", "text");

            this.isAutoResizable = false;
            this.element = tbxE;
            this.afterResize = afterResize;

            this._shrinkable = true;
            this._textHeights = new Set();
        }

        _createClass$m(TextboxV, [{
            key: "insertText",
            value: function insertText(text) {
                var focus = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

                var tbxE = this.element;
                var oldText = tbxE.value;
                var selectionEnd = tbxE.selectionEnd;

                var firstPart = oldText.slice(0, tbxE.selectionEnd);
                var secondPart = oldText.slice(tbxE.selectionEnd);
                tbxE.value = firstPart + text + secondPart;

                tbxE.selectionEnd = selectionEnd + text.length;
                if (focus) tbxE.focus();
                if (this.isAutoResizable) this.resize();
            }
        }, {
            key: "getTextBeforeSelectionEnd",
            value: function getTextBeforeSelectionEnd() {
                var tbxE = this.element;
                return tbxE.value.slice(0, tbxE.selectionEnd);
            }
        }, {
            key: "resize",
            value: function resize() {
                var _this = this;

                if (this._isResizing) return;

                this._isResizing = true;
                var tbxE = this.element;

                var shrink = this._shrinkable && tbxE.scrollHeight === tbxE.clientHeight && this._previousValue.length > tbxE.value.length;
                this._previousValue = tbxE.value;

                requestAnimationFrame(function () {
                    var scrollHeight = tbxE.scrollHeight;

                    if (!shrink) {
                        _this._textHeights.add(scrollHeight);
                    }
                    if (shrink) {
                        // to prevent bug @80554
                        // first fix parent element
                        tbxE.parentNode.style.minHeight = tbxE.parentNode.scrollHeight + "px";

                        tbxE.style.height = '';

                        // scrollHeight is rounded to integer
                        // so sometimes it might be wrong after shrinking by 1px
                        if (_this._textHeights.has(tbxE.scrollHeight)) {
                            scrollHeight = tbxE.scrollHeight;
                        } else {
                            scrollHeight = tbxE.scrollHeight + 1;
                        }
                    }

                    tbxE.style.height = scrollHeight + tbxE.offsetHeight - tbxE.clientHeight + "px";
                    _this._isResizing = false;
                    _this.afterResize();

                    if (shrink) {
                        // release parent
                        tbxE.parentNode.style.minHeight = null;
                    }
                });
            }
        }, {
            key: "autoResize",
            value: function autoResize() {
                var _this2 = this;

                var shrinkable = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

                this.isAutoResizable = true;
                this._previousValue = this.element.value;
                this._shrinkable = shrinkable;

                this.element.addEventListener("input", function () {
                    return _this2.resize();
                }, false);
            }
        }, {
            key: "forceResize",
            value: function forceResize() {
                this._previousValue = this.element.value + " ";
                this.resize();
            }
        }, {
            key: "clear",
            value: function clear() {
                this.element.value = "";
                if (this.isAutoResizable) this.resize();
            }
        }]);

        return TextboxV;
    }();

    var _createClass$n = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$z(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var SettingsV = function () {
        function SettingsV() {
            _classCallCheck$z(this, SettingsV);

            var settingsE = document.createElement('form');
            settingsE.classList.add('settings');
            settingsE.onsubmit = 'return false';

            this.element = settingsE;
            settingsE.settingsV = this;

            this._createGroupTitle('General');

            if (getAccessToken()) {
                this._createBooleanSetting('isEphemeralPostingOn', 'Turn on ephemeral posting');
            }
            this._createBooleanSetting('isKatexEnabled', 'Enable KaTeX');
            this._createBooleanSetting('highlightCode', 'Highlight code');

            this._createBooleanSetting('showImages', newE('span', {}, "Show 3rd party images (imgur, youtube, twitter, peertube: ", newE('a', { target: '_blank', href: 'https://libre.video' }, "libre.video"), ", ", newE('a', { target: '_blank', href: 'https://peertube.social' }, "peertube.social"), ", ", newE('a', { target: '_blank', href: 'https://video.ploud.fr' }, "ploud.fr"), ")"));
            this._createBooleanSetting('embedBitchute', 'Embed bitchute videos');

            var trackNewPostsCbxE = this._createBooleanSetting('trackNewPosts', 'Track new posts');

            var trackNewPostsWarningE = this._createSettingWarning('If you disable it, current information about which posts are new will be deleted');
            trackNewPostsWarningE.hide();

            trackNewPostsCbxE.addEventListener('change', function () {
                if (!trackNewPostsCbxE.checked && getSettings().trackNewPosts) {
                    trackNewPostsWarningE.show();
                } else {
                    trackNewPostsWarningE.hide();
                }
            }, false);

            this._createClientIdSetting();

            this._createGroupTitle('Theme');
            this._createBooleanSetting('isCompactTheme', 'Compact');

            this._createGroupTitle('Filters');
            this._createBooleanSetting('hideEphemeralPosts', 'Hide ephemeral posts');
            this._createBooleanSetting('showOnlyChatPostsInChat', 'Show only chat posts in the chat');
            this._createBooleanSetting('displayHiddenPostsInChat', 'Display hidden posts in the chat');

            this._createNumberSetting('maxEntropy', "Max Shannon's entropy (0 - unlimited)");
            this._createTextareaSetting('regexpFilters', 'Filter by regexps (each line is a regexp)');
            this._createTextareaSetting('colorsFilters', 'Filter by colors (each line is a bg:fg pair)');
        }

        _createClass$n(SettingsV, [{
            key: "getSettings",
            value: function getSettings() {
                var settings = {};

                var _iteratorNormalCompletion = true;
                var _didIteratorError = false;
                var _iteratorError = undefined;

                try {
                    for (var _iterator = this.element.$a('input, select, textarea')[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                        var element = _step.value;

                        var value = element instanceof HTMLInputElement && element.type === 'checkbox' ? element.checked : element.value;

                        if (!element.id) throw new Error('The element must have an id');

                        settings[element.id] = value;
                    }
                } catch (err) {
                    _didIteratorError = true;
                    _iteratorError = err;
                } finally {
                    try {
                        if (!_iteratorNormalCompletion && _iterator.return) {
                            _iterator.return();
                        }
                    } finally {
                        if (_didIteratorError) {
                            throw _iteratorError;
                        }
                    }
                }

                return settings;
            }
        }, {
            key: "_createGroupTitle",
            value: function _createGroupTitle(groupName) {
                var groupTitleE = document.createElement('div');
                groupTitleE.classList.add('group_title');
                groupTitleE.appendText(groupName);
                this.element.appendChild(groupTitleE);
            }
        }, {
            key: "_createBooleanSetting",
            value: function _createBooleanSetting(id, description) {
                var containerE = this._createSettingContainer();

                var cbxE = document.createElement('input');
                cbxE.id = id;
                cbxE.type = 'checkbox';
                cbxE.checked = getSettings()[id];
                containerE.appendChild(cbxE);

                containerE.appendChild(this._createLabel(id, description));
                return cbxE;
            }
        }, {
            key: "_createNumberSetting",
            value: function _createNumberSetting(id, description) {
                var size = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 2;
                var min = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 0;
                var step = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 'any';

                var containerE = this._createSettingContainer();
                containerE.appendChild(this._createLabel(id, description));

                var inputE = document.createElement('input');
                inputE.id = id;
                inputE.type = 'number';
                inputE.size = size;
                inputE.min = min;
                inputE.step = step;
                inputE.value = getSettings()[id];
                containerE.appendChild(inputE);
            }
        }, {
            key: "_createTextareaSetting",
            value: function _createTextareaSetting(id, description) {
                var containerE = this._createSettingContainer();
                containerE.appendChild(this._createLabel(id, description));

                var textboxV = new TextboxV();
                textboxV.element.id = id;
                textboxV.insertText(getSettings()[id]);
                containerE.appendChild(textboxV.element);
            }
        }, {
            key: "_createClientIdSetting",
            value: function _createClientIdSetting() {
                var id = "clientId";
                var containerE = this._createSettingContainer();
                containerE.appendChild(this._createLabel(id, "Client id:"));

                var inputE = document.createElement('input');
                inputE.id = id;
                inputE.required = true;
                inputE.maxLength = config.maxClientIdLength;
                inputE.disabled = true;
                getOrCreateClientId().then(function (clientId) {
                    inputE.value = clientId;
                    inputE.disabled = false;
                });
                containerE.appendChild(inputE);
            }
        }, {
            key: "_createSettingContainer",
            value: function _createSettingContainer() {
                var containerE = document.createElement('div');
                containerE.classList.add('setting_container');
                this.element.appendChild(containerE);
                return containerE;
            }
        }, {
            key: "_createLabel",
            value: function _createLabel(id, description) {
                var labelE = document.createElement('label');
                labelE.htmlFor = id;
                labelE.appendAny(description);
                return labelE;
            }
        }, {
            key: "_createSettingWarning",
            value: function _createSettingWarning(message) {
                var warningE = document.createElement('div');
                warningE.classList.add('setting_warning');

                var iconE = document.createElement('i');
                iconE.classList.add('icon-warning');

                warningE.appendChild(iconE);
                warningE.appendText(' ');
                warningE.appendText(message);
                this.element.appendChild(warningE);
                return warningE;
            }
        }]);

        return SettingsV;
    }();

    // file: shortcut_service.js
    function isEscape(event) {
        return event.key === "Escape" || event.keyCode === 27;
    }

    function isEnter(event) {
        return event.key === "Enter" || event.keyCode === 13;
    }

    function isCtrlEnter(event) {
        return isEnter(event) && event.ctrlKey;
    }

    function isAltEnter(event) {
        return isEnter(event) && event.altKey;
    }

    var _createClass$o = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$A(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var SettingsPopupV = function () {
        function SettingsPopupV() {
            var _this = this;

            _classCallCheck$A(this, SettingsPopupV);

            var decoratedPopupV = new PopupV();
            decoratedPopupV.element.id = 'settings_popup';

            var closeBtnTitle = 'Esc';
            var close = function close() {
                return _this.close();
            };

            var popupV = new PopupSavingStateD(new PopupWithCloseBtnV(new ModalPopupD(decoratedPopupV, close), close, closeBtnTitle, isEscape));
            this._popupV = popupV;

            var popupE = popupV.element;
            this.element = popupE;
            popupE.settingsPopupV = this;

            popupE.id = 'settings_popup';
            popupE.classList.add('settings-popup');

            var titleE = document.createElement('div');
            titleE.classList.add('title');
            titleE.appendText('Settings');
            titleE.appendBefore(popupV.titleBarE.firstChild);

            this.settingsV = new SettingsV();
            popupV.popupContentE.appendChild(this.settingsV.element);

            var buttonsContainerE = document.createElement('div');
            buttonsContainerE.classList.add('buttons_container');
            popupV.popupContentE.appendChild(buttonsContainerE);

            var saveBtnE = document.createElement('button');
            saveBtnE.appendText('Save');
            saveBtnE.classList.add('save_btn');
            saveBtnE.classList.add('form_btn');
            buttonsContainerE.appendChild(saveBtnE);

            saveBtnE.addEventListener('click', function () {
                return _this._save();
            }, false);

            var cancelBtnE = document.createElement('button');
            cancelBtnE.appendText('Cancel');
            cancelBtnE.classList.add('cancel_btn');
            cancelBtnE.classList.add('form_btn');
            cancelBtnE.title = closeBtnTitle;
            buttonsContainerE.appendChild(cancelBtnE);

            cancelBtnE.addEventListener('click', function () {
                return _this.close();
            }, false);
        }

        _createClass$o(SettingsPopupV, [{
            key: "close",
            value: function close() {
                this._popupV.close();
            }
        }, {
            key: "_save",
            value: function _save() {
                var formE = this.settingsV.element;
                var validate = (formE.reportValidity || formE.checkValidity).bind(formE);

                if (!validate()) return;

                var settings = this.settingsV.getSettings();

                setClientId(settings.clientId);
                delete settings.clientId;

                saveSettings(settings);
                this.close();
            }
        }]);

        return SettingsPopupV;
    }();

    function _classCallCheck$B(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var SettingsBtnV = function SettingsBtnV() {
        _classCallCheck$B(this, SettingsBtnV);

        var btnE = $("#settings_btn");
        this.element = btnE;
        btnE.settingsBtnV = this;

        btnE.addEventListener('click', function (e) {
            return showSettingsPopup();
        }, false);
    };

    function showSettingsPopup() {
        var settingsPopupE = $("#settings_popup");

        if (settingsPopupE) {
            settingsPopupE.settingsPopupV.close();
        } else {
            new SettingsPopupV();
        }
    }

    var _createClass$p = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$C(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    // file: help_v.js
    var HelpV = function () {
        function HelpV() {
            _classCallCheck$C(this, HelpV);

            var helpE = document.createElement('div');
            helpE.classList.add('help');

            this.element = helpE;
            helpE.helpV = this;

            var helpTemplateE = document.importNode($("#help_template").content, true);
            this.element.appendChild(helpTemplateE);

            this._createGroupTitle('');
        }

        _createClass$p(HelpV, [{
            key: '_createGroupTitle',
            value: function _createGroupTitle(groupName) {
                var groupTitleE = document.createElement('div');
                groupTitleE.classList.add('group_title');
                groupTitleE.appendText(groupName);
                this.element.appendChild(groupTitleE);
                return groupTitleE;
            }
        }]);

        return HelpV;
    }();

    var _createClass$q = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$D(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var HelpPopupV = function () {
        function HelpPopupV() {
            var _this = this;

            _classCallCheck$D(this, HelpPopupV);

            var decoratedPopupV = new PopupV();
            decoratedPopupV.element.id = 'help_popup';

            var closeBtnTitle = 'Esc';
            var close = function close() {
                return _this.close();
            };

            var popupV = new PopupSavingStateD(new PopupWithCloseBtnV(new ModalPopupD(decoratedPopupV, close), close, closeBtnTitle, isEscape));
            this._popupV = popupV;

            var popupE = popupV.element;
            this.element = popupE;
            popupE.settingsPopupV = this;

            popupE.classList.add('help-popup');

            var titleE = document.createElement('div');
            titleE.classList.add('title');
            titleE.appendText('Help');
            titleE.appendBefore(popupV.titleBarE.firstChild);

            this.helpV = new HelpV();
            popupV.popupContentE.appendChild(this.helpV.element);
        }

        _createClass$q(HelpPopupV, [{
            key: "close",
            value: function close() {
                this._popupV.close();
            }
        }]);

        return HelpPopupV;
    }();

    function _classCallCheck$E(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var HelpBtnV = function HelpBtnV() {
        _classCallCheck$E(this, HelpBtnV);

        var btnE = $("#help_btn");
        this.element = btnE;
        btnE.helpBtnV = this;

        btnE.addEventListener('click', function (e) {
            return showHelpPopup();
        }, false);
    };

    function showHelpPopup() {
        var helpPopupE = $("#help_popup");

        if (helpPopupE) {
            helpPopupE.helpPopupV.close();
        } else {
            new HelpPopupV();
        }
    }

    function _classCallCheck$F(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ExportBtnV = function ExportBtnV() {
        _classCallCheck$F(this, ExportBtnV);

        var btnE = $("#export_btn");
        this.element = btnE;
        this.exportBtnV = this;

        btnE.addEventListener('click', function (e) {
            return exportUiState();
        }, false);
    };

    function exportUiState() {
        var linkE = document.createElement("a");
        linkE.hide();

        var uiState = {
            ui: "ui",
            version: config.version,
            localStorage: {}
        };

        Object.keys(localStorage).map(function (key) {
            if (key.startsWith('tab_') || key === lsKey || key !== key.toLowerCase() && key !== clientIdLsKey) {
                return;
            }
            uiState.localStorage[key] = getItem$1(key);
        });

        var blob = new Blob([JSON.stringify(uiState, null, 2)], { type: 'application/json' });

        var url = URL.createObjectURL(blob);
        linkE.href = url;
        linkE.download = "pfpmd_ui_settings.json";

        document.body.appendChild(linkE);
        linkE.click();
        linkE.remove();
        URL.revokeObjectURL(url);
    }

    function _classCallCheck$G(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ImportBtnV = function ImportBtnV() {
        _classCallCheck$G(this, ImportBtnV);

        var btnE = $("#import_btn");
        this.element = btnE;
        this.importBtnV = this;

        btnE.addEventListener('click', function (e) {
            return uploadUiState();
        }, false);
    };

    function importUiState(uiState) {
        // validate uiState
        if (!uiState.ui) {
            showError("Missing key: \"ui\".");
            return;
        }
        if (uiState.ui !== "ui") {
            showError("Wrong ui value: expected \"ui\"; got \"" + uiState.ui + "\".");
            return;
        }
        if (!uiState.version) {
            showError("Missing key: \"version\".");
            return;
        }
        if (uiState.version.constructor !== "".constructor) {
            showError("\"version\" must be a string");
            return;
        }
        if (!/^(0|[1-9]\d*)\.(0|[1-9]\d*)\.(0|[1-9]\d*)$/.test(uiState.version)) {
            showError("Wrong version value: \"" + uiState.version + "\" doesn't match \n            \"^(0|[1-9]\\d*)\\.(0|[1-9]\\d*)\\.(0|[1-9]\\d*)$\" pattern.\"");
            return;
        }
        if (!uiState.localStorage) {
            showError("Missing key: \"localStorage\".");
            return;
        }
        if (uiState.localStorage.constructor !== {}.constructor) {
            showError("\"localStorage\" must be an object.");
            return;
        }

        Object.keys(uiState.localStorage).map(function (key) {
            return setItem$1(key, uiState.localStorage[key], undefined, true);
        });
    }

    function uploadUiState() {
        var inputE = document.createElement("input");
        inputE.hide();
        inputE.type = "file";
        inputE.accept = "application/json";
        document.body.appendChild(inputE);

        inputE.addEventListener('change', function (e) {
            var reader = new FileReader();

            reader.onload = function (event) {
                var uiState = void 0;
                try {
                    uiState = JSON.parse(event.target.result);
                } catch (err) {
                    showError("The file must be a json file.");
                    return;
                }
                importUiState(uiState);
            };

            reader.readAsText(e.target.files[0]);
            inputE.remove();
        }, false);

        inputE.click();
    }

    function showError(message) {
        return showNotification(config.notification.type.error, message, 5000);
    }

    // file: server_info_service.js

    var serverInfo = null;

    function getServerInfo$2() {
        return serverInfo;
    }

    function initServerInfoService() {
        serverInfo = getServerInfo$1().then();
    }

    var _createClass$r = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$H(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var LoginFormV = function () {
        function LoginFormV(loginE) {
            var _this = this;

            _classCallCheck$H(this, LoginFormV);

            var loginFormE = loginE.$("#login_form");
            this.element = loginFormE;
            loginFormE.loginFromV = this;

            this._loginNicknameE = loginFormE.$("#login_nickname");
            this._loginPasswordE = loginFormE.$("#login_password");

            var loginSubmitBtnE = loginFormE.$("#login_submit_btn");

            loginSubmitBtnE.addEventListener('click', function (e) {
                if (!_this.element.reportValidity()) return;
                e.preventDefault();

                _this._onSubmit(e);
            }, false);
        }

        _createClass$r(LoginFormV, [{
            key: "_onSubmit",
            value: function _onSubmit(e) {
                authorize$1(new Auth(this._loginNicknameE.value, this._loginPasswordE.value)).then(function (authInfo) {
                    $("#login_popup").loginPopupV.close();
                    setAccessToken(authInfo.accessToken);
                    authorizeChange();

                    $("#login_btn").hide();
                    $("#profile_btn").profileBtnV.show();
                }).catch(function (e) {
                    return showNotification(config.notification.type.error, e.message);
                });
            }
        }]);

        return LoginFormV;
    }();

    var _createClass$s = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$I(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var SignupFormV = function () {
        function SignupFormV(loginE) {
            var _this = this;

            _classCallCheck$I(this, SignupFormV);

            var signupFormE = loginE.$("#signup_form");
            this.element = signupFormE;
            signupFormE.signupFromV = this;

            this._signupNicknameE = signupFormE.$("#signup_nickname");
            this._signupPasswordE = signupFormE.$("#signup_password");
            this._signupPowE = signupFormE.$("#signup_pow");

            var signupSubmitBtnE = loginE.$("#signup_submit_btn");

            signupSubmitBtnE.addEventListener('click', function (e) {
                if (!_this.element.reportValidity()) return;
                e.preventDefault();

                _this._onSubmit(e);
            }, false);
        }

        _createClass$s(SignupFormV, [{
            key: "_onSubmit",
            value: function _onSubmit(e) {
                createUser$1(new NewUser(this._signupNicknameE.value, this._signupPasswordE.value, this._signupPowE.value.trim())).then(function (message) {
                    showNotification(config.notification.type.success, message + ". Now you can log in.");
                }).catch(function (e) {
                    showNotification(config.notification.type.error, e.message, 4000);
                });
            }
        }]);

        return SignupFormV;
    }();

    var _createClass$t = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$J(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var LoginV = function () {
        function LoginV() {
            var _this = this;

            _classCallCheck$J(this, LoginV);

            var loginE = document.createElement('div');
            loginE.classList.add('login');

            this.element = loginE;
            loginE.loginV = this;

            var loginTemplateE = document.importNode($("#login_template").content, true);
            this.element.appendChild(loginTemplateE);

            this._createGroupTitle("");

            getServerInfo$2().then(function (serverInfo) {
                if (serverInfo.powBits) {
                    _this.element.$a('.pow_bits').forEach(function (element) {
                        return element.innerText = serverInfo.powBits;
                    });
                }
            });

            new LoginFormV(loginE);
            new SignupFormV(loginE);
        }

        _createClass$t(LoginV, [{
            key: "_createGroupTitle",
            value: function _createGroupTitle(groupName) {
                var groupTitleE = document.createElement('div');
                groupTitleE.classList.add('group_title');
                groupTitleE.appendText(groupName);
                this.element.appendChild(groupTitleE);
                return groupTitleE;
            }
        }]);

        return LoginV;
    }();

    var _createClass$u = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$K(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var LoginPopupV = function () {
        function LoginPopupV() {
            var _this = this;

            _classCallCheck$K(this, LoginPopupV);

            var decoratedPopupV = new PopupV();
            decoratedPopupV.element.id = 'login_popup';

            var closeBtnTitle = 'Esc';
            var close = function close() {
                return _this.close();
            };

            var popupV = new PopupSavingStateD(new PopupWithCloseBtnV(new ModalPopupD(decoratedPopupV, close), close, closeBtnTitle, isEscape));
            this._popupV = popupV;

            var popupE = popupV.element;
            this.element = popupE;
            popupE.loginPopupV = this;

            popupE.classList.add('login-popup');

            var titleE = document.createElement('div');
            titleE.classList.add('title');
            titleE.appendText('Login / create account');
            titleE.appendBefore(popupV.titleBarE.firstChild);

            this.loginV = new LoginV();
            popupV.popupContentE.appendChild(this.loginV.element);
        }

        _createClass$u(LoginPopupV, [{
            key: "close",
            value: function close() {
                this._popupV.close();
            }
        }]);

        return LoginPopupV;
    }();

    function _classCallCheck$L(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    // file: badge_v.js
    var BadgeV = function BadgeV(label) {
        _classCallCheck$L(this, BadgeV);

        var badgeE = document.createElement("span");
        this.element = badgeE;
        badgeE.badgeV = this;

        badgeE.classList.add("badge");
        badgeE.appendAny(label);
    };

    var _createClass$v = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$M(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var LoginBtnV = function () {
        function LoginBtnV() {
            _classCallCheck$M(this, LoginBtnV);

            var btnE = $("#login_btn");
            this.element = btnE;
            btnE.loginBtnV = this;

            if (getAccessToken()) btnE.hide();

            this._badgeV = new BadgeV("!");
            this.element.appendChild(this._badgeV.element);

            btnE.addEventListener('click', function (e) {
                return showLoginPopup();
            }, false);
        }

        _createClass$v(LoginBtnV, [{
            key: "_showBadge",
            value: function _showBadge() {
                this._badgeV.element.show();
            }
        }, {
            key: "_hideBadge",
            value: function _hideBadge() {
                this._badgeV.element.hide();
            }
        }]);

        return LoginBtnV;
    }();

    function showLoginPopup() {
        var loginPopupE = $("#login_popup");

        if (loginPopupE) {
            loginPopupE.loginPopupV.close();
        } else {
            new LoginPopupV();
        }
    }

    var _createClass$w = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$N(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ProfileV = function () {
        function ProfileV() {
            _classCallCheck$N(this, ProfileV);

            var profileE = document.createElement('div');
            profileE.classList.add('profile');

            this.element = profileE;
            profileE.profileV = this;

            var profileTemplateE = document.importNode($("#profile_template").content, true);
            this.element.appendChild(profileTemplateE);

            profileE.$("#nickname").innerText = getClaims().name;

            this._createGroupTitle("");

            var logoutBtnE = profileE.$("#logout_btn");

            logoutBtnE.addEventListener('click', function (e) {
                deleteAccessToken();
                authorizeChange();
                $('#profile_popup').profilePopupV.close();
                $("#login_btn").show();
                $("#profile_btn").hide();
            }, false);
        }

        // todo: this is repeated multiple times in other files. Refactor.


        _createClass$w(ProfileV, [{
            key: "_createGroupTitle",
            value: function _createGroupTitle(groupName) {
                var groupTitleE = document.createElement('div');
                groupTitleE.classList.add('group_title');
                groupTitleE.appendText(groupName);
                this.element.appendChild(groupTitleE);
                return groupTitleE;
            }
        }]);

        return ProfileV;
    }();

    var _createClass$x = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$O(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ProfilePopupV = function () {
        function ProfilePopupV() {
            var _this = this;

            _classCallCheck$O(this, ProfilePopupV);

            var decoratedPopupV = new PopupV();
            decoratedPopupV.element.id = 'profile_popup';

            var closeBtnTitle = 'Esc';
            var close = function close() {
                return _this.close();
            };

            var popupV = new PopupSavingStateD(new PopupWithCloseBtnV(new ModalPopupD(decoratedPopupV, close), close, closeBtnTitle, isEscape));
            this._popupV = popupV;

            var popupE = popupV.element;
            this.element = popupE;
            popupE.profilePopupV = this;

            popupE.classList.add('profile-popup');

            var titleE = document.createElement('div');
            titleE.classList.add('title');
            titleE.appendText('Profile');
            titleE.appendBefore(popupV.titleBarE.firstChild);

            this.profileV = new ProfileV();
            popupV.popupContentE.appendChild(this.profileV.element);
        }

        _createClass$x(ProfilePopupV, [{
            key: "close",
            value: function close() {
                this._popupV.close();
            }
        }]);

        return ProfilePopupV;
    }();

    var _createClass$y = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$P(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ProfileBtnV = function () {
        function ProfileBtnV() {
            _classCallCheck$P(this, ProfileBtnV);

            var btnE = $("#profile_btn");
            this.element = btnE;
            btnE.profileBtnV = this;

            if (getAccessToken()) this.show();

            btnE.addEventListener('click', function (e) {
                return showProfilePopup();
            }, false);
        }

        _createClass$y(ProfileBtnV, [{
            key: "show",
            value: function show() {
                var claims = getClaims();

                this.element.innerText = claims.name;
                this.element.show();
            }
        }]);

        return ProfileBtnV;
    }();

    function showProfilePopup() {
        var profilePopupE = $("#profile_popup");

        if (profilePopupE) {
            profilePopupE.profilePopupV.close();
        } else {
            new ProfilePopupV();
        }
    }

    function _classCallCheck$Q(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var MenuV = function MenuV() {
        _classCallCheck$Q(this, MenuV);

        var menuE = $("#menu");
        this.element = menuE;
        menuE.menuE = this;

        this.homeLinkV = new HomeLinkV();
        this.searchLinkV = new SearchLinkV();

        this.connectionStatusV = new ConnectionStatusV();
        this.exportBtnV = new ExportBtnV();
        this.importBtnV = new ImportBtnV();
        this.settingsBtnV = new SettingsBtnV();
        this.helpBtnV = new HelpBtnV();
        this.loginBtnV = new LoginBtnV();
        this.profileBtnV = new ProfileBtnV();
    };

    // file: hljs_service.js

    var serviceLoader$1 = new ServiceLoader(config.hljs.cssPath, config.hljs.scriptPath);

    function enableHljs(isEnabled) {
        return serviceLoader$1.enableService(isEnabled);
    }

    function highlightCode(element) {
        if (!serviceLoader$1.isServiceEnabled) {
            // undoHighlighting(element);
            return;
        }

        // show elements without highliting first, don't let highlighting slow displaying them
        setTimeout(function () {
            // :not(.hljs) is to prevent applying it more than once
            element.$a("code:not(.hljs)").forEach(function (codeE) {
                return hljs.highlightBlock(codeE);
            });
        }, 200);
    }

    // NOTE: it turns out that it is much slower that recreating html
    // function undoHighlighting(element) {
    //     element.$a("code.hljs").forEach(codeE => {
    //         codeE.classList.remove('hljs');
    //         const code = codeE.innerText;
    //         codeE.empty();
    //         codeE.appendText(code);
    //     });
    // }

    function _classCallCheck$R(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ThreadBannerQuotedSubjD = function ThreadBannerQuotedSubjD(threadBannerV) {
        _classCallCheck$R(this, ThreadBannerQuotedSubjD);

        decorateView(threadBannerV, this, 'threadBannerV');

        threadBannerV.subjectE.prependText("``");
        threadBannerV.subjectE.appendText("''");
    };

    function _classCallCheck$S(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ThreadBannerV = function ThreadBannerV(threadId, subject) {
        _classCallCheck$S(this, ThreadBannerV);

        this.threadId = threadId;
        this.subject = subject;

        var bannerE = document.createElement('h3');
        bannerE.classList.add('thread_banner');

        this.element = bannerE;
        bannerE.threadBannerV = this;

        this.threadIdE = document.createElement('span');
        this.threadIdE.classList.add('thread_id');
        this.threadIdE.appendText('' + toShortcutUuid(threadId));

        bannerE.appendChild(this.threadIdE);
        bannerE.appendText(' \u2013 ');

        this.subjectE = document.createElement('span');
        this.subjectE.classList.add('subject');
        this.subjectE.appendText(subject);

        bannerE.appendChild(this.subjectE);
    };

    function _classCallCheck$T(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var WarpPageLinkV = function WarpPageLinkV(linkE, boardName, pageNumber) {
        _classCallCheck$T(this, WarpPageLinkV);

        this.element = linkE;
        linkE.warpPageLinkV = this;

        linkE.href = getWarpUrl(boardName, pageNumber);

        linkE.addEventListener("click", function (e) {
            e.preventDefault();
            setWarp(boardName, pageNumber);
        }, false);
    };

    var _createClass$z = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$U(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var BoardLinkV = function () {
        function BoardLinkV(boardName, boardDescription) {
            _classCallCheck$U(this, BoardLinkV);

            var boardLinkE = document.createElement('a');
            boardLinkE.classList.add('board_link');

            this.element = boardLinkE;
            boardLinkE.boardLinkV = this;

            boardLinkE.appendText('' + boardName);
            boardLinkE.title = boardDescription || '';
            new WarpPageLinkV(boardLinkE, boardName, 0);
        }

        _createClass$z(BoardLinkV, [{
            key: 'changeBoardDescription',
            value: function changeBoardDescription(boardDescription) {
                this.element.title = boardDescription || '';
            }
        }]);

        return BoardLinkV;
    }();

    var _createClass$A = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$V(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ThreadBannerWithBoardLinkD = function () {
        function ThreadBannerWithBoardLinkD(threadBannerV, boardName, boardDescription) {
            _classCallCheck$V(this, ThreadBannerWithBoardLinkD);

            decorateView(threadBannerV, this, 'threadBannerV');

            var boardNameE = document.createElement('span');
            boardNameE.classList.add('board_name');
            boardNameE.appendText('/');

            var boardLinkV = new BoardLinkV(boardName, boardDescription);
            this._boardLinkV = boardLinkV;
            boardNameE.appendChild(boardLinkV.element);
            boardNameE.appendText('/');

            threadBannerV.element.prependChild(boardNameE);
        }

        _createClass$A(ThreadBannerWithBoardLinkD, [{
            key: "changeBoardDescription",
            value: function changeBoardDescription(boardDescription) {
                this._boardLinkV.changeBoardDescription(boardDescription);
            }
        }]);

        return ThreadBannerWithBoardLinkD;
    }();

    // file: ephemeral_posting_service.js

    function isReplyToEphemeralPost(text) {
        return getPostReferences(text).some(function (postId) {
            var cachedPost = get(postId);
            return cachedPost && cachedPost.ephemeral;
        });
    }

    function isEphemeralPostingOn() {
        return !getAccessToken() || getSettings().isEphemeralPostingOn;
    }

    function isEphemeralPosting(postText) {
        return isEphemeralPostingOn() || isReplyToEphemeralPost(postText);
    }

    var _createClass$B = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$W(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ReplyFormV = function () {
        function ReplyFormV(threadId, boardName) {
            var _this = this;

            _classCallCheck$W(this, ReplyFormV);

            var formE = document.createElement('form');
            formE.classList.add('reply_form');

            this.element = formE;
            formE.replyFormV = this;

            this._threadId = threadId;
            this._boardName = boardName;

            var formBodyE = document.createElement('div');
            formBodyE.classList.add('form_body');
            formE.appendChild(formBodyE);

            this.tbxV = new TextboxV();
            this.tbxV.autoResize(false);
            var tbxE = this.tbxV.element;
            tbxE.id = threadId + "tbx";
            tbxE.classList.add('reply_tbx');
            tbxE.maxLength = config.maxPostLength;
            formBodyE.appendChild(tbxE);

            var belowTbxE = document.createElement('div');
            belowTbxE.classList.add('below_tbx');
            formBodyE.appendChild(belowTbxE);

            var submitBtnE = document.createElement('button');
            submitBtnE.classList.add('form_btn', 'reply_btn');
            submitBtnE.type = 'submit';
            submitBtnE.title = 'Ctrl+Enter';

            var replyIconE = document.createElement('i');
            replyIconE.classList.add('icon-reply');
            submitBtnE.appendChild(replyIconE);
            submitBtnE.appendText('Reply');

            belowTbxE.appendChild(submitBtnE);

            var stayHereLabelE = document.createElement('label');
            stayHereLabelE.classList.add('stay_here_lbl');

            var stayHereCbxE = document.createElement('input');
            this._stayHereCbxE = stayHereCbxE;
            stayHereCbxE.classList.add('stay_here_cbx');
            stayHereCbxE.type = 'checkbox';
            stayHereLabelE.appendChild(stayHereCbxE);
            stayHereLabelE.appendText("stay here");

            belowTbxE.appendChild(stayHereLabelE);

            this.onAuthorizeChange();

            submitBtnE.addEventListener('click', function (e) {
                if (!_this.element.reportValidity()) return;
                e.preventDefault();

                _this._onSubmit(e);
            }, false);

            tbxE.addEventListener("keydown", function (e) {
                if (isCtrlEnter(e)) {
                    if (!_this.element.reportValidity()) return;
                    e.preventDefault();

                    _this._onSubmit(e);
                }
            }, false);

            this.onEphemeralPostingModeChange();
            addSettingsChangeEventListener(function (e) {
                if (e.oldSettings.isEphemeralPostingOn !== e.newSettings.isEphemeralPostingOn) {
                    _this.onEphemeralPostingModeChange();
                }
            });
        }

        _createClass$B(ReplyFormV, [{
            key: "onAuthorizeChange",
            value: function onAuthorizeChange() {
                var isAuthorized = getAccessToken();
                // this.element.elements.forEach(e => e.disabled = !isAuthorized);
                this.tbxV.element.placeholder = isAuthorized ? "" : config.ephemeralModeWarning;
            }
        }, {
            key: "onEphemeralPostingModeChange",
            value: function onEphemeralPostingModeChange() {
                this.tbxV.element.placeholder = isEphemeralPostingOn() ? config.ephemeralModeWarning : "";
            }
        }, {
            key: "_onSubmit",
            value: function _onSubmit(e) {
                var _this2 = this;

                var text = this.tbxV.element.value;
                if (!text || !text.trim()) return;

                var stayHere = this._stayHereCbxE.checked;

                var ephemeral = isEphemeralPosting(text);

                if (stayHere || ephemeral) this.element.reset();

                publish$1(new NewPost(null, text, this._threadId, ephemeral)).then(function (post) {
                    if (stayHere || ephemeral) return;

                    setThreadPage(_this2._boardName, _this2._threadId, null, post.postId);
                }).catch(function (e) {
                    if (e.errorCode === 'E0401') {
                        showNotification(config.notification.type.error, e.message + ". Please log in.");
                        return;
                    }

                    showNotification(config.notification.type.error, "Message failed to send.");
                });
            }
        }]);

        return ReplyFormV;
    }();

    function _classCallCheck$X(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ThreadPageLinkV = function ThreadPageLinkV(linkE, boardName, threadId, pageNumber) {
        _classCallCheck$X(this, ThreadPageLinkV);

        this.element = linkE;
        linkE.threadPageLinkV = this;

        linkE.href = getThreadUrl(boardName, threadId, pageNumber);

        linkE.addEventListener("click", function (e) {
            e.preventDefault();
            setThreadPage(boardName, threadId, pageNumber);
        }, false);
    };

    function _classCallCheck$Y(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ThreadSummaryV = function ThreadSummaryV(thread, shownPostsCount) {
        _classCallCheck$Y(this, ThreadSummaryV);

        var summaryE = document.createElement('div');
        summaryE.classList.add('thread_summary');

        this.element = summaryE;
        summaryE.summaryV = this;

        var omittedCount = thread.postCount - shownPostsCount;
        if (omittedCount <= 0) return;

        summaryE.appendText(omittedCount + ' replies omitted. ');

        var threadLinkE = document.createElement('a');
        threadLinkE.classList.add('thread_link');
        // threadLinkE.title = "the first page";
        threadLinkE.appendText('Click here');

        new ThreadPageLinkV(threadLinkE, thread.board.name, thread.threadId, 0);

        summaryE.appendChild(threadLinkE);
        summaryE.appendText(' to view the first page.');
    };

    function _classCallCheck$Z(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ThreadLinkV = function ThreadLinkV(boardName, threadId) {
        _classCallCheck$Z(this, ThreadLinkV);

        var threadLinkE = document.createElement('a');
        threadLinkE.classList.add('thread_link');

        this.element = threadLinkE;
        threadLinkE.threadLinkV = this;

        threadLinkE.appendText("" + toShortcutUuid(threadId));
        new ThreadPageLinkV(threadLinkE, boardName, threadId, null);
    };

    function _classCallCheck$_(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ThreadBannerWithThreadLinkD = function ThreadBannerWithThreadLinkD(threadBannerV, boardName, threadId) {
        _classCallCheck$_(this, ThreadBannerWithThreadLinkD);

        decorateView(threadBannerV, this, 'threadBannerV');

        // const boardNameE = document.createElement('span');
        // boardNameE.classList.add('board_name');
        // boardNameE.appendText('/');

        var threadLinkE = new ThreadLinkV(boardName, threadId).element;
        threadLinkE.empty();

        getNodesRange(threadBannerV.threadIdE, threadBannerV.subjectE).forEach(function (el) {
            return threadLinkE.appendChild(el);
        });
        // threadLinkE.appendChild(threadBannerV.threadIdE);
        // threadLinkE.appendChild(threadBannerV.subjectE);

        // threadBannerV.element.prependChild(threadLinkE);
        threadBannerV.element.appendChild(threadLinkE);
    };

    function _classCallCheck$$(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    // file: pagination_gap_v.js
    var PaginationGapV = function PaginationGapV(paginationV) {
        _classCallCheck$$(this, PaginationGapV);

        var paginationGapE = document.createElement('div');
        paginationGapE.classList.add('pagination_gap', 'item');

        this.element = paginationGapE;
        paginationGapE.paginationGapV = this;

        paginationV.element.appendChild(paginationGapE);
    };

    var _createClass$C = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$10(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var PaginationV = function () {
        function PaginationV(totalCount, currentPage, pageSize, siblingCount, pageLinkF) {
            _classCallCheck$10(this, PaginationV);

            this._pageSize = pageSize;
            this._currentPage = currentPage;
            this._siblingCount = siblingCount;
            this._pageLinkF = pageLinkF;

            // Note: pages start from 0.
            var paginationE = document.createElement('div');
            paginationE.classList.add('pagination');

            paginationE.paginationV = this;
            this.element = paginationE;

            this._addPaginationElements(totalCount, currentPage, siblingCount, pageLinkF);
        }

        _createClass$C(PaginationV, [{
            key: "onTotalCountChange",
            value: function onTotalCountChange(newTotalCount) {
                var newPageCount = Math.ceil(newTotalCount / this._pageSize);

                // if page count didn't change, then do nothing
                if (newPageCount === this._pageCount) return;

                // recreate elements
                this.element.empty();
                this._addPaginationElements(newTotalCount, this._currentPage, this._siblingCount, this._pageLinkF);
            }
        }, {
            key: "_addPaginationElements",
            value: function _addPaginationElements(totalCount, currentPage, siblingCount, pageLinkF) {
                var _this = this;

                this._pageCount = Math.ceil(totalCount / this._pageSize);
                if (this._pageCount <= 1) return;

                if (currentPage !== 0) pageLinkF.create(0, currentPage, this);

                var lastPageNumber = this._pageCount - 1;

                // 0 . 2 3 4 [5]
                var isLeftGapNeeded = currentPage - siblingCount - 1 > 0;
                if (isLeftGapNeeded) new PaginationGapV(this);

                range(siblingCount).map(function (_, i) {
                    return currentPage - i - 1;
                }).reverse().filter(function (pageNumber) {
                    return pageNumber > 0;
                }).forEach(function (pageNumber) {
                    return pageLinkF.create(pageNumber, currentPage, _this);
                });

                pageLinkF.create(currentPage, currentPage, this);

                range(siblingCount).map(function (_, i) {
                    return currentPage + i + 1;
                }).filter(function (pageNumber) {
                    return pageNumber < lastPageNumber;
                }).forEach(function (pageNumber) {
                    return pageLinkF.create(pageNumber, currentPage, _this);
                });

                // [5] 6 7 8 . 10
                var isRightGapNeeded = currentPage + siblingCount + 1 < lastPageNumber;
                if (isRightGapNeeded) new PaginationGapV(this);

                if (currentPage === lastPageNumber) return;
                var lastPageLinkV = pageLinkF.create(lastPageNumber, currentPage, this);
            }
        }]);

        return PaginationV;
    }();

    function _classCallCheck$11(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ThreadPageLinkD = function ThreadPageLinkD(pageLinkV, threadPage) {
        _classCallCheck$11(this, ThreadPageLinkD);

        decorateView(pageLinkV, this, 'pageLinkV');

        new ThreadPageLinkV(pageLinkV.pageLinkE, threadPage.data.thread.board.name, threadPage.data.thread.threadId, pageLinkV.pageNumber);
    };

    function _classCallCheck$12(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var PageLinkV = function PageLinkV(pageNumber, currentPage, paginationV) {
        _classCallCheck$12(this, PageLinkV);

        this.pageNumber = pageNumber;
        this.currentPage = currentPage;

        this.pageLinkE = document.createElement('a');
        this.pageLinkE.classList.add('link', 'item');
        this.pageLinkE.appendText(pageNumber);

        this.selectableV = new SelectableV(this.pageLinkE);

        if (pageNumber === currentPage) {
            this.selectableV.select();
        }

        this.pageLinkE.pageLinkV = this;
        this.element = this.pageLinkE;

        paginationV.element.appendChild(this.pageLinkE);
    };

    var _createClass$D = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$13(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ThreadPageLinkF = function () {
        function ThreadPageLinkF(threadPage) {
            _classCallCheck$13(this, ThreadPageLinkF);

            this._threadPage = threadPage;
        }

        _createClass$D(ThreadPageLinkF, [{
            key: "create",
            value: function create(pageNumber, currentPage, paginationV) {
                var pageLinkV = new PageLinkV(pageNumber, currentPage, paginationV);
                return new ThreadPageLinkD(pageLinkV, this._threadPage);
            }
        }]);

        return ThreadPageLinkF;
    }();

    var _createClass$E = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$14(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    // file: board_banner_v.js

    var BoardBannerV = function () {
        function BoardBannerV(boardName, boardDescription) {
            _classCallCheck$14(this, BoardBannerV);

            this.boardName = boardName;
            this.boardDescription = boardDescription;

            var bannerE = document.createElement('h2');
            bannerE.classList.add('board_banner');

            this.element = bannerE;
            bannerE.boardBannerV = this;

            this.boardNameE = document.createElement('span');
            this.boardNameE.classList.add('board_name');
            this.boardNameE.appendText('/' + boardName + '/');

            bannerE.appendChild(this.boardNameE);

            this.nameDescriptionSeparator = document.createElement('span');
            bannerE.appendChild(this.nameDescriptionSeparator);

            if (boardDescription) {
                this.nameDescriptionSeparator.appendText(' \u2013 ');
            }

            this.boardDescriptionE = document.createElement('span');
            this.boardDescriptionE.classList.add('board_description');
            this.boardDescriptionE.appendText('' + boardDescription);

            bannerE.appendChild(this.boardDescriptionE);
        }

        _createClass$E(BoardBannerV, [{
            key: 'changeBoardDescription',
            value: function changeBoardDescription(boardDescription) {
                this.boardDescription = boardDescription;

                if (boardDescription) {
                    this.nameDescriptionSeparator.replaceChildrenWith(' \u2013 ');
                } else {
                    this.nameDescriptionSeparator.replaceChildrenWith('');
                }

                this.boardDescriptionE.replaceChildrenWith('' + boardDescription);
            }
        }]);

        return BoardBannerV;
    }();

    // file: title_service.js

    var titlePrefix = "";
    var title = "";

    function setTitlePrefix(prefix) {
        titlePrefix = prefix ? prefix + " " : "";
        setTitle(title);
    }

    function setTitle(txt) {
        title = txt;
        document.title = "" + titlePrefix + txt;
    }

    function getSearchTitle(searchResultPage) {
        var srp = searchResultPage;

        return "Search" + getPageInfo(config.pageSize.general, srp.total, getPageNumber(srp.skipped));
    }

    function getThreadPageTitle(threadPage) {
        var tp = threadPage;
        var t = threadPage.data.thread;

        return "/" + t.board.name + "/" + toShortcutUuid(t.threadId) + " \u2013 ``" + t.subject + "''" + getPageInfo(config.pageSize.general, tp.total, getThreadPageNumber(tp.skipped));
    }

    function getWarpTitle(warpPage) {
        var board = warpPage.data.board;
        var pageNumber = getPageNumber(warpPage.skipped, config.pageSize.warp);

        var pageInfo = pageNumber ? getPageInfo(config.pageSize.warp, warpPage.total, pageNumber) : "";

        return (board ? "/" + board.name + "/" : "pfpmd") + (board && board.description ? " \u2013 " + board.description : "") + pageInfo;
    }

    function getPageInfo(pageSize, totalCount, pageNumber) {
        return pageSize >= totalCount ? "" : " \u2013 page " + pageNumber;
    }

    function _classCallCheck$15(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    // file: threads_separator_v.js

    var ThreadsSeparatorV = function ThreadsSeparatorV() {
        _classCallCheck$15(this, ThreadsSeparatorV);

        this.element = document.createElement('hr');
        this.element.classList.add('threads_separator');

        this.element.threadsSeparatorV = this;
    };

    function _classCallCheck$16(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var BoardBannerAsLinkD = function BoardBannerAsLinkD(boardBannerV) {
        _classCallCheck$16(this, BoardBannerAsLinkD);

        decorateView(boardBannerV, this);

        var boardLinkV = new BoardLinkV(boardBannerV.boardName, null);

        boardLinkV.element.empty();

        boardBannerV.element.moveChildNodesTo(boardLinkV.element);
        boardBannerV.element.appendChild(boardLinkV.element);
    };

    var _createClass$F = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$17(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var PostWithReplyD = function () {
        function PostWithReplyD(postV, replyFormV) {
            var _this = this;

            _classCallCheck$17(this, PostWithReplyD);

            decorateView(postV, this);

            this.element = postV.element;
            this.post = postV.post;
            this._replyFormV = replyFormV;

            postV.idE.addEventListener('click', function (e) {
                return _this.reply();
            }, false);
        }

        _createClass$F(PostWithReplyD, [{
            key: "reply",
            value: function reply() {
                var focus = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

                this._replyFormV.showAfterPost(this.element);
                this._replyFormV.tbxV.insertText("@" + toSimpleUuid(this.post.postId) + " ", focus);
            }
        }]);

        return PostWithReplyD;
    }();

    function _classCallCheck$18(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var PostWithIdD = function PostWithIdD(postV) {
        _classCallCheck$18(this, PostWithIdD);

        decorateView(postV, this);

        postV.postE.id = "p" + postV.post.postId;
    };

    var _createClass$G = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$19(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ReplyFormPositionedD = function () {
        function ReplyFormPositionedD(replyFormV) {
            _classCallCheck$19(this, ReplyFormPositionedD);

            decorateView(replyFormV, this, 'replyFormV');

            this.element = replyFormV.element;
        }

        _createClass$G(ReplyFormPositionedD, [{
            key: "showAfterPost",
            value: function showAfterPost(postE) {
                this.element.show();
                this.element.appendAfter(postE);
                this.postAbove = postE.postV.post;
            }
        }]);

        return ReplyFormPositionedD;
    }();

    function _classCallCheck$1a(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var PostTrackNewD = function PostTrackNewD(postV, lastSeenAt) {
        _classCallCheck$1a(this, PostTrackNewD);

        decorateView(postV, this, 'postV');
        this.postE = postV.postE;

        if (!getSettings().trackNewPosts) return;

        if (lastSeenAt && postV.post.postDate > lastSeenAt) {
            this.postE.classList.add("new-post");
        }
    };

    var _createClass$H = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$1b(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
    var ThreadPageV = function () {
        function ThreadPageV(threadPage) {
            _classCallCheck$1b(this, ThreadPageV);

            var thread = threadPage.data.thread;
            this._thread = thread;
            this._op = threadPage.data.op;
            this._posts = threadPage.data.posts;
            this._prevPost = threadPage.data.prevPost || {};

            var threadPageE = document.createElement('div');
            threadPageE.classList.add('thread_page');

            this.element = threadPageE;
            threadPageE.threadPageV = this;

            var boardBannerV = new BoardBannerAsLinkD(new BoardBannerV(thread.board.name, thread.board.description));
            this._boardBannerV = boardBannerV;
            threadPageE.appendChild(boardBannerV.element);

            var threadsSeparatorV = new ThreadsSeparatorV();
            threadPageE.appendChild(threadsSeparatorV.element);

            var threadBannerV = new ThreadBannerQuotedSubjD(new ThreadBannerV(thread.threadId, thread.subject));
            threadPageE.appendChild(threadBannerV.element);

            this.replyFormV = new ReplyFormPositionedD(new ReplyFormV(thread.threadId, thread.board.name));

            threadPageE.appendChild(this.replyFormV.element);

            this._loadPosts();

            var pageLinkF = new ThreadPageLinkF(threadPage);
            this._paginationV = new PaginationV(threadPage.total - 1, // -1 to exclude OP
            getThreadPageNumber(threadPage.skipped), config.pageSize.general, config.pagination.siblingCount, pageLinkF);
            threadPageE.appendChild(this._paginationV.element);

            setTitle(getThreadPageTitle(threadPage));

            if (getSettings().trackNewPosts) this._updateLastSeenAt();
        }

        _createClass$H(ThreadPageV, [{
            key: "onSettingsChange",
            value: function onSettingsChange(oldSettings, settings) {
                var _this = this;

                return (oldSettings.isKatexEnabled !== settings.isKatexEnabled ? enableKatex(settings.isKatexEnabled) : Promise.resolve()).then(function () {
                    return oldSettings.highlightCode !== settings.highlightCode ? enableHljs(settings.highlightCode).then() : null;
                }).then(function () {
                    _this.reloadPosts();

                    if (settings.highlightCode) highlightCode(_this.element);
                });
            }
        }, {
            key: "onNewPost",
            value: function onNewPost(post) {
                // ignore posts from other threads
                if (post.thread.threadId !== this._thread.threadId) return;

                this._thread.postCount++;
                this._paginationV.onTotalCountChange(this._thread.postCount - 1); // -1 to skip OP

                // quit if the current page is full
                if (this._posts.length === config.pageSize.general) return;

                // add the post to the current page
                this._posts.push(post);
                var postV = this._createThreadPostV(post, false);

                postV.element.appendAfter((this.postVs.last() || this.originalPostV).element);
                this.postVs.push(postV);
            }
        }, {
            key: "onUpdatedHiddenByOp",
            value: function onUpdatedHiddenByOp(hiddenByOpEvent) {
                this._posts.filter(function (post) {
                    return post.postId === hiddenByOpEvent.postId;
                }).forEach(function (post) {
                    return post.hiddenByOp = hiddenByOpEvent.hiddenByOp;
                });

                this.reloadPosts();
            }
        }, {
            key: "onBoardDescriptionChange",
            value: function onBoardDescriptionChange(boardName, boardDescription) {
                if (this._thread.board.name !== boardName) return;

                this._changeBoardDescription(boardDescription);
            }
        }, {
            key: "reloadPosts",
            value: function reloadPosts() {
                this.element.children.filter(function (e) {
                    return e.postV;
                }).forEach(function (e) {
                    return e.remove();
                });
                this._loadPosts();
            }
        }, {
            key: "_changeBoardDescription",
            value: function _changeBoardDescription(boardDescription) {
                this._boardBannerV.changeBoardDescription(boardDescription);
            }
        }, {
            key: "_loadPosts",
            value: function _loadPosts() {
                var _this2 = this;

                this.originalPostV = this._createThreadPostV(this._op, true);
                this.originalPostV.element.appendBefore(this.replyFormV.element);

                this.postVs = this._posts.map(function (post) {
                    return _this2._createThreadPostV(post, false);
                });

                this.postVs.forEach(function (postV) {
                    return postV.element.appendBefore(_this2.replyFormV.element);
                });
            }
        }, {
            key: "_createThreadPostV",
            value: function _createThreadPostV(post, isOriginalPost) {
                return createThreadPostV(post, isOriginalPost, this.replyFormV, this._thread.lastSeenAt);
            }
        }, {
            key: "_updateLastSeenAt",
            value: function _updateLastSeenAt() {
                var lastSeenAt = this._getLastSeenAt();
                this._thread.lastSeenAt = lastSeenAt;

                // quit if there're new posts on the previous page
                if (lastSeenAt && lastSeenAt < this._prevPost.postDate) return;

                var lastPostV = this.postVs.last();

                if (lastPostV && (!lastSeenAt || lastSeenAt < lastPostV.post.postDate)) {
                    // make a request to store lastSeenAt as lastPostV.post.postDate
                    updateLastSeenAt$1(this._thread.threadId, lastPostV.post.postDate);
                }
            }
        }, {
            key: "_getLastSeenAt",
            value: function _getLastSeenAt() {
                return  getLastSeenAt(this._thread.threadId);
            }
        }]);

        return ThreadPageV;
    }();

    function createThreadPostV(post, isOriginalPost, replyFormV, lastSeenAt) {
        var postV = new PostV(post, isOriginalPost);

        return new PostWithReplyD(new PostWithIdD(new PostTrackNewD(postV, lastSeenAt)), replyFormV);
    }

    var _createClass$I = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$1c(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ShellThreadV = function () {
        function ShellThreadV(shellThread, isGlobalWarp) {
            _classCallCheck$1c(this, ShellThreadV);

            var shellThreadE = document.createElement('div');
            shellThreadE.classList.add('shell_thread');

            this.element = shellThreadE;
            shellThreadE.shellThreadV = this;

            this.isGlobalWarp = isGlobalWarp;
            this.shellThread = shellThread;

            this._thread = shellThread.thread;
            this._thread.board = this._thread.board || {};

            this._thread.lastSeenAt =  getLastSeenAt(this._thread.threadId);

            this.threadBannerV = new ThreadBannerV(this._thread.threadId, this._thread.subject);
            this.threadBannerV = new ThreadBannerQuotedSubjD(this.threadBannerV);
            this.threadBannerV = new ThreadBannerWithThreadLinkD(this.threadBannerV, this._thread.board.name, this._thread.threadId);

            if (isGlobalWarp) {
                this.threadBannerV = new ThreadBannerWithBoardLinkD(this.threadBannerV, this._thread.board.name, this._thread.board.description);
            }
            shellThreadE.appendChild(this.threadBannerV.element);

            var replyFormV = new ReplyFormPositionedD(new ReplyFormV(this._thread.threadId, this._thread.board.name));
            this.replyFormV = replyFormV;

            replyFormV.element.hide();

            this._loadPosts();

            shellThreadE.appendChild(replyFormV.element);
        }

        _createClass$I(ShellThreadV, [{
            key: "onNewPost",
            value: function onNewPost(post) {}
        }, {
            key: "onUpdatedHiddenByOp",
            value: function onUpdatedHiddenByOp(hiddenByOpEvent) {
                this.shellThread.posts.filter(function (post) {
                    return post.postId === hiddenByOpEvent.postId;
                }).forEach(function (post) {
                    return post.hiddenByOp = hiddenByOpEvent.hiddenByOp;
                });

                this.reloadPosts();
            }
        }, {
            key: "onBoardDescriptionChange",
            value: function onBoardDescriptionChange(boardName, boardDescription) {
                if (boardName !== this._thread.board.name) return;

                this._changeBoardDescription(boardDescription);
            }
        }, {
            key: "_changeBoardDescription",
            value: function _changeBoardDescription(boardDescription) {
                this._thread.board.description = boardDescription;

                if (this.isGlobalWarp) {
                    this.threadBannerV.changeBoardDescription(boardDescription);
                }
            }
        }, {
            key: "reloadPosts",
            value: function reloadPosts() {
                this.element.children.filter(function (e) {
                    return e.postV || e.summaryV;
                }).forEach(function (e) {
                    return e.remove();
                });
                this._loadPosts();
            }
        }, {
            key: "_loadPosts",
            value: function _loadPosts() {
                var _this = this;

                var op = this.shellThread.op;

                this.originalPostV = this._createThreadPostV(op, true);
                this.element.appendChild(this.originalPostV.element);

                var threadSummaryV = new ThreadSummaryV(this._thread, this.shellThread.posts.length + 1);
                this.element.appendChild(threadSummaryV.element);

                this.postVs = this.shellThread.posts.map(function (post) {
                    return _this._createThreadPostV(post, false);
                });
                this.postVs.forEach(function (postV) {
                    return _this.element.appendChild(postV.element);
                });
            }
        }, {
            key: "_createThreadPostV",
            value: function _createThreadPostV(post, isOriginalPost) {
                return createThreadPostV(post, isOriginalPost, this.replyFormV, this._thread.lastSeenAt);
            }
        }]);

        return ShellThreadV;
    }();

    function _classCallCheck$1d(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var WarpPageLinkD = function WarpPageLinkD(pageLinkV, warpPage) {
        _classCallCheck$1d(this, WarpPageLinkD);

        decorateView(pageLinkV, this, 'pageLinkV');

        var boardName = warpPage.data.board ? warpPage.data.board.name : null;

        new WarpPageLinkV(pageLinkV.pageLinkE, boardName, pageLinkV.pageNumber);
    };

    var _createClass$J = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$1e(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var WarpPageLinkF = function () {
        function WarpPageLinkF(warpPage) {
            _classCallCheck$1e(this, WarpPageLinkF);

            this._warpPage = warpPage;
        }

        _createClass$J(WarpPageLinkF, [{
            key: "create",
            value: function create(pageNumber, currentPage, paginationV) {
                var pageLinkV = new PageLinkV(pageNumber, currentPage, paginationV);
                return new WarpPageLinkD(pageLinkV, this._warpPage);
            }
        }]);

        return WarpPageLinkF;
    }();

    var _createClass$K = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$1f(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var NewThreadFormV = function () {
        function NewThreadFormV(boardName) {
            var _this = this;

            _classCallCheck$1f(this, NewThreadFormV);

            var formE = document.createElement('form');
            formE.classList.add('new_thread_form');

            this.element = formE;
            this.replyFormV = this;
            formE.newThreadFormV = this;

            this._boardName = boardName;

            var formBodyE = document.createElement('div');
            formBodyE.classList.add('form_body');
            formE.appendChild(formBodyE);

            var inputsContainerE = document.createElement('div');
            inputsContainerE.classList.add('inputs_container');
            formBodyE.appendChild(inputsContainerE);

            if (!boardName) {
                var boardE = document.createElement('input');
                this.boardE = boardE;
                boardE.id = 'board_name_input';
                boardE.type = 'text';
                boardE.required = true;
                boardE.placeholder = 'Board';
                boardE.title = '1 to 3 alphabetic characters';
                boardE.maxLength = config.maxBoardNameLength;
                boardE.pattern = config.boardNamePattern;
                boardE.classList.add('board');
                inputsContainerE.appendChild(boardE);
            }

            var subjectE = document.createElement('input');
            this.subjectE = subjectE;
            subjectE.id = 'subject_input';
            subjectE.classList.add('subject');
            subjectE.required = true;
            subjectE.placeholder = 'Subject';
            subjectE.maxLength = config.maxSubjectLength;
            inputsContainerE.appendChild(subjectE);

            this.tbxV = new TextboxV();
            this.tbxV.autoResize(false);
            var tbxE = this.tbxV.element;
            tbxE.id = "new_post_tbx";
            tbxE.classList.add('post_tbx');
            tbxE.maxLength = config.maxPostLength;
            formBodyE.appendChild(tbxE);

            var belowTbxE = document.createElement('div');
            belowTbxE.classList.add('below_tbx');
            formBodyE.appendChild(belowTbxE);

            var submitBtnE = document.createElement('button');
            submitBtnE.classList.add('form_btn');
            submitBtnE.type = 'submit';
            submitBtnE.title = 'Ctrl+Enter';
            submitBtnE.appendText('Submit');

            belowTbxE.appendChild(submitBtnE);

            var stayHereLabelE = document.createElement('label');
            stayHereLabelE.classList.add('stay_here_lbl');

            var stayHereCbxE = document.createElement('input');
            this._stayHereCbxE = stayHereCbxE;
            stayHereCbxE.classList.add('stay_here_cbx');
            stayHereCbxE.type = 'checkbox';
            stayHereLabelE.appendChild(stayHereCbxE);
            stayHereLabelE.appendText("stay here");

            belowTbxE.appendChild(stayHereLabelE);

            this.onAuthorizeChange();

            submitBtnE.addEventListener('click', function (e) {
                if (!_this.element.reportValidity()) return;
                e.preventDefault();

                _this._onSubmit(e);
            }, false);

            tbxE.addEventListener("keydown", function (e) {
                if (isCtrlEnter(e)) {
                    if (!_this.element.reportValidity()) return;
                    e.preventDefault();

                    _this._onSubmit(e);
                }
            }, false);

            this.onEphemeralPostingModeChange();
            addSettingsChangeEventListener(function (e) {
                if (e.oldSettings.isEphemeralPostingOn !== e.newSettings.isEphemeralPostingOn) {
                    _this.onEphemeralPostingModeChange();
                }
            });
        }

        _createClass$K(NewThreadFormV, [{
            key: "onAuthorizeChange",
            value: function onAuthorizeChange() {
                var isAuthorized = getAccessToken();
                this.tbxV.element.placeholder = isAuthorized ? "" : config.ephemeralModeWarning;
                // this.element.elements.forEach(e => e.disabled = !isAuthorized);
            }
        }, {
            key: "onEphemeralPostingModeChange",
            value: function onEphemeralPostingModeChange() {
                this.tbxV.element.placeholder = isEphemeralPostingOn() ? config.ephemeralModeWarning : "";
            }
        }, {
            key: "_onSubmit",
            value: function _onSubmit(e) {
                var text = this.tbxV.element.value;
                if (!text || !text.trim()) return;

                var subject = this.subjectE.value;
                var boardName = this._boardName || this.boardE.value;
                var stayHere = this._stayHereCbxE.checked;

                var ephemeral = isEphemeralPosting(text);

                if (stayHere || ephemeral) this.element.reset();

                publish$1(new NewPost(null, text, null, ephemeral, subject, boardName)).then(function (post) {
                    if (stayHere || ephemeral) return;

                    setThreadPage(post.thread.board.name, post.thread.threadId, null, post.postId);
                }).catch(function (e) {
                    if (e.errorCode === 'E0401') {
                        showNotification(config.notification.type.error, e.message + ". Please log in.");
                        return;
                    }

                    showNotification(config.notification.type.error, "Message failed to send.");
                });
            }
        }]);

        return NewThreadFormV;
    }();

    var _createClass$L = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$1g(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var WarpPageV = function () {
        function WarpPageV(warpPage) {
            _classCallCheck$1g(this, WarpPageV);

            this.warpPage = warpPage;

            var warpPageE = document.createElement('div');
            warpPageE.classList.add('warp');

            this.element = warpPageE;
            warpPageE.threadPageV = this;

            var isGlobalWarp = !warpPage.data.board;
            var isChatBoard = !isGlobalWarp && warpPage.data.board.name === config.chat.boardName;
            this._isChatBoard = isChatBoard;

            if (!isGlobalWarp) {
                var boardBannerV = new BoardBannerV(warpPage.data.board.name, warpPage.data.board.description);
                this._boardBannerV = boardBannerV;
                warpPageE.appendChild(boardBannerV.element);

                if (!isChatBoard) {
                    var threadsSeparatorV = new ThreadsSeparatorV();
                    this._threadsSeparatorV = threadsSeparatorV;
                    warpPageE.appendChild(threadsSeparatorV.element);
                }
            }

            if (!isChatBoard) {
                var startNewThreadE = document.createElement('h3');
                this._startNewThreadE = startNewThreadE;
                startNewThreadE.classList.add('start_new_thread');
                startNewThreadE.appendText('Start a New Thread');
                warpPageE.appendChild(startNewThreadE);

                // const newThreadBtnContainerE = document.createElement('div');
                // newThreadBtnContainerE.classList.add('start_new_thread_btn_container');
                // warpPageE.appendChild(newThreadBtnContainerE);
                //
                // const newThreadBtnE = document.createElement('button');
                // newThreadBtnE.appendText('Start a New Thread');
                // warpPageE.appendChild(newThreadBtnE);

                var newThreadFormV = new NewThreadFormV(isGlobalWarp ? null : warpPage.data.board.name);
                this.newThreadFormV = newThreadFormV;
                var newThreadFormE = newThreadFormV.element;
                // newThreadFormE.hide();
                warpPageE.appendChild(newThreadFormV.element);

                // newThreadBtnE.addEventListener('click', e => {
                //     newThreadFormE.toggleHide();
                //     newThreadBtnE.toggleHide();
                //
                //     // newThreadBtnE.appendText(newThreadFormE.isHidden()
                //     //     ? 'Start a New Thread' : 'Cancel'
                //     // );
                // }, false);
            }

            this.shellThreadsVs = warpPage.data.shellThreads.map(function (shellThread) {
                var threadsSeparatorV = new ThreadsSeparatorV();
                warpPageE.appendChild(threadsSeparatorV.element);

                var shellThreadV = new ShellThreadV(shellThread, isGlobalWarp);
                warpPageE.appendChild(shellThreadV.element);
                return shellThreadV;
            });

            var pageLinkF = new WarpPageLinkF(warpPage);
            var paginationV = new PaginationV(warpPage.total, getPageNumber(warpPage.skipped, config.pageSize.warp), config.pageSize.warp, config.pagination.siblingCount, pageLinkF);
            warpPageE.appendChild(paginationV.element);

            setTitle(getWarpTitle(warpPage));
        }

        _createClass$L(WarpPageV, [{
            key: "onSettingsChange",
            value: function onSettingsChange(oldSettings, settings) {
                var _this = this;

                return (oldSettings.isKatexEnabled !== settings.isKatexEnabled ? enableKatex(settings.isKatexEnabled) : Promise.resolve()).then(function () {
                    return oldSettings.highlightCode !== settings.highlightCode ? enableHljs(settings.highlightCode).then() : null;
                }).then(function () {
                    _this.reloadPosts();

                    if (settings.highlightCode) highlightCode(_this.element);
                });
            }
        }, {
            key: "reloadPosts",
            value: function reloadPosts() {
                this.shellThreadsVs.forEach(function (shellThread) {
                    return shellThread.reloadPosts();
                });
            }
        }, {
            key: "onNewPost",
            value: function onNewPost(post) {
                this.shellThreadsVs.forEach(function (shellThread) {
                    return shellThread.onNewPost(post);
                });
            }
        }, {
            key: "onUpdatedHiddenByOp",
            value: function onUpdatedHiddenByOp(hiddenByOpEvent) {
                this.shellThreadsVs.forEach(function (shellThread) {
                    return shellThread.onUpdatedHiddenByOp(hiddenByOpEvent);
                });
            }
        }, {
            key: "onBoardDescriptionChange",
            value: function onBoardDescriptionChange(boardName, boardDescription) {
                var board = this.warpPage.data.board;

                if (board && board.name !== boardName) return;

                this._changeBoardDescription(boardDescription);

                this.shellThreadsVs.forEach(function (shellThreadV) {
                    return shellThreadV.onBoardDescriptionChange(boardName, boardDescription);
                });
            }
        }, {
            key: "_changeBoardDescription",
            value: function _changeBoardDescription(boardDescription) {
                var board = this.warpPage.data.board;
                if (!board) return;

                board.description = boardDescription;

                if (!this._boardBannerV) return;

                this._boardBannerV.changeBoardDescription(boardDescription);
            }
        }]);

        return WarpPageV;
    }();

    // file: session_storage_service.js

    function getItem$2(key, defaultValue) {
        return getItem(sessionStorage, key, defaultValue);
    }

    function setItem$2(key, value) {
        setItem(sessionStorage, key, value);
    }

    var _createClass$M = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$1h(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    // An instance of this class is used by some views to store and restore their state,
    // e.g. scroll position, texts of textboxes, etc. This is needed because it's SPA,
    // i.e. pushState popState functionality is used.
    var LocationState = function () {
        function LocationState() {
            _classCallCheck$1h(this, LocationState);

            // This property stores the previous location until a content is loaded by the view.
            // After the content is loaded, it must be updated (by the view) to the current location.
            this._locationHref = null;
        }

        _createClass$M(LocationState, [{
            key: "updateLocation",
            value: function updateLocation() {
                this._locationHref = getHref();
            }
        }, {
            key: "isLocationSet",
            value: function isLocationSet() {
                return this._locationHref !== null;
            }
        }, {
            key: "setState",
            value: function setState(state) {
                setItem$2(this._locationHref, state);
            }
        }, {
            key: "getState",
            value: function getState() {
                return getItem$2(this._locationHref);
            }
        }]);

        return LocationState;
    }();

    var _createClass$N = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$1i(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ThreadPageWithStateD = function () {
        function ThreadPageWithStateD(threadPageV) {
            _classCallCheck$1i(this, ThreadPageWithStateD);

            decorateView(threadPageV, this, 'threadPageV');

            this._threadPageV = threadPageV;
            this.element = threadPageV.element;
        }

        _createClass$N(ThreadPageWithStateD, [{
            key: "getState",
            value: function getState() {
                var postAbove = this.replyFormV.postAbove;
                var postId = postAbove ? postAbove.postId : null;

                return new ThreadState(postId, this.replyFormV.tbxV.element.value);
            }
        }, {
            key: "restoreState",
            value: function restoreState(threadState) {

                [this.originalPostV].concat(this.postVs).forEach(function (postV) {
                    if (threadState.replyFormAfterPostId === postV.post.postId) {
                        postV.reply(false);
                    }
                });

                if (this.replyFormV.tbxV.element.value !== threadState.text) {
                    this.replyFormV.tbxV.element.value = threadState.text;
                }
            }
        }, {
            key: "replyFormV",
            get: function get() {
                return this._threadPageV.replyFormV;
            }
        }, {
            key: "originalPostV",
            get: function get() {
                return this._threadPageV.originalPostV;
            }
        }, {
            key: "postVs",
            get: function get() {
                return this._threadPageV.postVs;
            }
        }]);

        return ThreadPageWithStateD;
    }();

    var ThreadState = function ThreadState(replyFormAfterPostId, text) {
        _classCallCheck$1i(this, ThreadState);

        this.replyFormAfterPostId = replyFormAfterPostId;
        this.text = text;
    };

    var _createClass$O = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$1j(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var WarpPageWithStateD = function () {
        function WarpPageWithStateD(warpPageV) {
            _classCallCheck$1j(this, WarpPageWithStateD);

            decorateView(warpPageV, this, 'warpPageV');

            this._warpPageV = warpPageV;
            this.element = warpPageV.element;
        }

        _createClass$O(WarpPageWithStateD, [{
            key: "getState",
            value: function getState() {
                var _this = this;

                return new WarpPageState(this._getNewThreadFormState(), this.shellThreadsVs.map(function (shellThreadV) {
                    return _this._getShellThreadState(shellThreadV);
                }));
            }
        }, {
            key: "restoreState",
            value: function restoreState(warpPageState) {
                var _this2 = this;

                if (warpPageState.newThreadFormState) {
                    this._restoreNewThreadFormState(warpPageState.newThreadFormState);
                }

                warpPageState.shellThreadsStates.forEach(function (shellThreadState) {
                    _this2._restoreShellThreadState(shellThreadState.threadId, shellThreadState);
                });
            }
        }, {
            key: "_getNewThreadFormState",
            value: function _getNewThreadFormState() {
                if (!this.newThreadFormV) return null;

                return new NewThreadFormState(this.newThreadFormV.boardE ? this.newThreadFormV.boardE.value : null, this.newThreadFormV.subjectE.value, this.newThreadFormV.tbxV.element.value);
            }
        }, {
            key: "_getShellThreadState",
            value: function _getShellThreadState(shellThreadV) {
                var postAbove = shellThreadV.replyFormV.postAbove;
                var replyFormAfterPostId = postAbove ? postAbove.postId : null;

                return new ShellThreadState(shellThreadV.shellThread.thread.threadId, replyFormAfterPostId, shellThreadV.replyFormV.tbxV.element.value);
            }
        }, {
            key: "_restoreNewThreadFormState",
            value: function _restoreNewThreadFormState(newThreadFormState) {
                if (newThreadFormState.boardName) {
                    this.newThreadFormV.boardE.value = newThreadFormState.boardName;
                }

                if (newThreadFormState.subject) {
                    this.newThreadFormV.subjectE.value = newThreadFormState.subject;
                }
                if (newThreadFormState.text) {
                    this.newThreadFormV.tbxV.element.value = newThreadFormState.text;
                }
            }
        }, {
            key: "_restoreShellThreadState",
            value: function _restoreShellThreadState(threadId, shellThreadState) {
                var shellThreadV = this.shellThreadsVs.find(function (shellThreadV) {
                    return shellThreadV.shellThread.thread.threadId === threadId;
                });

                if (!shellThreadV) return; // probably that thread is not on this page anymore

                var postsVs = [shellThreadV.originalPostV].concat(shellThreadV.postVs);

                if (shellThreadState.replyFormAfterPostId) {
                    (postsVs.find(function (postV) {
                        return postV.post.postId === shellThreadState.replyFormAfterPostId;
                    }) || postsVs.last()).reply(false);
                }

                shellThreadV.replyFormV.tbxV.element.value = shellThreadState.text;
            }
        }, {
            key: "newThreadFormV",
            get: function get() {
                return this._warpPageV.newThreadFormV;
            }
        }, {
            key: "shellThreadsVs",
            get: function get() {
                return this._warpPageV.shellThreadsVs;
            }
        }]);

        return WarpPageWithStateD;
    }();

    var WarpPageState = function WarpPageState(newThreadFormState, shellThreadsStates) {
        _classCallCheck$1j(this, WarpPageState);

        this.newThreadFormState = newThreadFormState;
        this.shellThreadsStates = shellThreadsStates;
    };

    var NewThreadFormState = function NewThreadFormState(boardName, subject, text) {
        _classCallCheck$1j(this, NewThreadFormState);

        this.boardName = boardName;
        this.subject = subject;
        this.text = text;
    };

    var ShellThreadState = function ShellThreadState(threadId, replyFormAfterPostId, text) {
        _classCallCheck$1j(this, ShellThreadState);

        this.threadId = threadId;
        this.replyFormAfterPostId = replyFormAfterPostId;
        this.text = text;
    };

    function _classCallCheck$1k(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var SearchFormV = function SearchFormV(searchArgs) {
        var _this = this;

        _classCallCheck$1k(this, SearchFormV);

        var formE = document.createElement('form');
        formE.classList.add('search_form');

        this.element = formE;
        this.searchFormV = this;

        searchArgs = searchArgs || new SearchArgs('', 'all', true, '', 0);
        this.searchArgs = searchArgs;

        var formBodyE = document.createElement('div');
        this.formBodyE = formBodyE;
        formBodyE.classList.add('form_body');
        formE.appendChild(formBodyE);

        var queryContainerE = document.createElement('div');
        queryContainerE.classList.add('inputs_container');
        formBodyE.appendChild(queryContainerE);

        var queryE = document.createElement('input');
        this.queryE = queryE;
        queryE.id = 'query';
        queryE.type = 'text';
        if (searchArgs.query) queryE.value = searchArgs.query;
        queryE.required = true;
        queryE.placeholder = 'SQL LIKE pattern (e.g. %pfpmd%, %100\\%%)';
        queryE.title = queryE.placeholder;
        queryContainerE.appendChild(queryE);

        var rangeAndBoardsContainerE = document.createElement('div');
        rangeAndBoardsContainerE.classList.add('inputs_container');
        formBodyE.appendChild(rangeAndBoardsContainerE);

        var rangeE = document.createElement('select');
        this.rangeE = rangeE;
        rangeE.id = 'range';
        rangeE.required = true;
        rangeAndBoardsContainerE.appendChild(rangeE);

        var allE = document.createElement('option');
        allE.value = 'all';
        allE.text = 'In posts and subjects';
        rangeE.add(allE);

        var postsOnlyE = document.createElement('option');
        postsOnlyE.value = 'text';
        postsOnlyE.text = 'In posts only';
        rangeE.add(postsOnlyE);

        var subjectsOnlyE = document.createElement('option');
        subjectsOnlyE.value = 'subject';
        subjectsOnlyE.text = 'In subjects only';
        rangeE.add(subjectsOnlyE);

        rangeE.value = searchArgs.range || 'all';

        var boardsE = document.createElement('input');
        this.boardsE = boardsE;
        boardsE.id = 'boards';
        if (searchArgs.boards) boardsE.value = searchArgs.boards;
        boardsE.classList.add('boards');
        boardsE.placeholder = 'In boards (e.g. b,pr,cht)';
        boardsE.title = boardsE.placeholder;
        boardsE.pattern = config.boardNamePattern + "(?: *, *" + config.boardNamePattern + ")*";
        rangeAndBoardsContainerE.appendChild(boardsE);

        var orderContainerE = document.createElement('div');
        orderContainerE.classList.add('inputs_container');
        formBodyE.appendChild(orderContainerE);

        var orderE = document.createElement('select');
        this.orderE = orderE;
        orderE.id = 'order';
        orderE.required = true;
        orderContainerE.appendChild(orderE);

        var orderDesc = document.createElement('option');
        orderDesc.value = true;
        orderDesc.text = 'Order by date: desc';
        orderE.add(orderDesc);

        var orderAsc = document.createElement('option');
        orderAsc.value = false;
        orderAsc.text = 'Order by date: asc';
        orderE.add(orderAsc);

        orderE.value = searchArgs.orderDesc || true;

        var searchBtnE = document.createElement('button');
        searchBtnE.classList.add('search_btn');
        searchBtnE.classList.add('form_btn');
        searchBtnE.type = 'submit';
        searchBtnE.title = 'Ctrl+Enter';

        var searchIconE = document.createElement('i');
        searchIconE.classList.add('icon-search');
        searchBtnE.appendChild(searchIconE);
        searchBtnE.appendText('Search');
        formBodyE.appendChild(searchBtnE);

        searchBtnE.addEventListener('click', function (e) {
            if (!_this.element.checkValidity()) return;
            e.preventDefault();

            _this.searchArgs = new SearchArgs(_this.queryE.value, _this.rangeE.value, _this.orderE.value, _this.boardsE.value ? _this.boardsE.value.replace(/\s/g, '') : '', 0);
            setSearch(_this.searchArgs);
        }, false);

        {
            formE.elements.forEach(function (e) {
                return e.disabled = true;
            });

            showNotification(config.notification.type.info, textToElement("This server doesn't support searching.\n" + "Please try another one or use the client."), 5000);
        }
    };

    function _classCallCheck$1l(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    // file: search_result_separator_v.js

    var SearchResultSeparatorV = function SearchResultSeparatorV() {
        _classCallCheck$1l(this, SearchResultSeparatorV);

        this.element = document.createElement('hr');
        this.element.classList.add('search_result_separator');

        this.element.searchResultSeparatorV = this;
    };

    function _classCallCheck$1m(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var SearchResultPageLinkV = function SearchResultPageLinkV(linkE, pageNumber) {
        _classCallCheck$1m(this, SearchResultPageLinkV);

        this.element = linkE;
        linkE.searchResultPageLinkV = this;

        var searchArgs = getSearchArgs();
        searchArgs.pageNumber = pageNumber;

        linkE.href = getSearchUrl(searchArgs);

        linkE.addEventListener("click", function (e) {
            e.preventDefault();
            setSearch(searchArgs);
        }, false);
    };

    var _createClass$P = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$1n(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var SearchResultPageLinkF = function () {
        function SearchResultPageLinkF() {
            _classCallCheck$1n(this, SearchResultPageLinkF);
        }

        _createClass$P(SearchResultPageLinkF, [{
            key: "create",
            value: function create(pageNumber, currentPage, paginationV) {
                var pageLinkV = new PageLinkV(pageNumber, currentPage, paginationV);

                return new SearchResultPageLinkV(pageLinkV.pageLinkE, pageLinkV.pageNumber);
            }
        }]);

        return SearchResultPageLinkF;
    }();

    function _classCallCheck$1o(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ThreadLinkWithTitleD = function ThreadLinkWithTitleD(threadLinkV, title) {
        _classCallCheck$1o(this, ThreadLinkWithTitleD);

        decorateView(threadLinkV, this);

        threadLinkV.element.title = title;
    };

    var _createClass$Q = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$1p(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var BoardThreadLinksV = function () {
        function BoardThreadLinksV(post) {
            _classCallCheck$1p(this, BoardThreadLinksV);

            var postInfoE = document.createElement('span');
            postInfoE.classList.add('board_thread_links');

            this.element = postInfoE;
            postInfoE.postInfoV = this;

            var boardLinkV = new BoardLinkV(post.thread.board.name, post.thread.board.description);
            this._boardLinkV = boardLinkV;
            postInfoE.appendText("/");
            postInfoE.appendChild(boardLinkV.element);

            var threadLinkV = new ThreadLinkWithTitleD(new ThreadLinkV(post.thread.board.name, post.thread.threadId), post.thread.subject);
            postInfoE.appendText("/");
            postInfoE.appendChild(threadLinkV.element);

            var postLinkV = new PostLinkV(post.postId, "@", "Go to the post");
            postInfoE.appendText("/");
            postInfoE.appendChild(postLinkV.element);
        }

        _createClass$Q(BoardThreadLinksV, [{
            key: "changeBoardDescription",
            value: function changeBoardDescription(boardDescription) {
                this._boardLinkV.changeBoardDescription(boardDescription);
            }
        }]);

        return BoardThreadLinksV;
    }();

    var _createClass$R = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$1q(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var SearchResultPostD = function () {
        function SearchResultPostD(postV) {
            _classCallCheck$1q(this, SearchResultPostD);

            decorateView(postV, this, 'postV');

            this._postV = postV;
            postV.postE.classList.remove("original");

            var boardThreadLinksV = new BoardThreadLinksV(postV.post);
            this._boardThreadLinksV = boardThreadLinksV;
            postV.headerE.appendText(' ');
            postV.headerE.appendChild(boardThreadLinksV.element);
        }

        _createClass$R(SearchResultPostD, [{
            key: "changeBoardDescription",
            value: function changeBoardDescription(boardDescription) {
                this._boardThreadLinksV.changeBoardDescription(boardDescription);
                this._postV.changeBoardDescription(boardDescription);
            }
        }]);

        return SearchResultPostD;
    }();

    var _createClass$S = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$1r(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var SearchResultV = function () {
        function SearchResultV(searchResultPage) {
            _classCallCheck$1r(this, SearchResultV);

            var searchResultE = document.createElement('div');
            this.element = searchResultE;
            searchResultE.searchV = this;
            searchResultE.classList.add("search_result");

            this._posts = searchResultPage.data;

            this._postsContainerE = document.createElement('div');
            this._postsContainerE.classList.add('posts_container');
            searchResultE.appendChild(this._postsContainerE);

            this._loadPosts();

            var pageLinkF = new SearchResultPageLinkF();
            var paginationV = new PaginationV(searchResultPage.total, getPageNumber(searchResultPage.skipped), config.pageSize.general, config.pagination.siblingCount, pageLinkF);
            searchResultE.appendChild(paginationV.element);

            setTitle(getSearchTitle(searchResultPage));
        }

        _createClass$S(SearchResultV, [{
            key: "onBoardDescriptionChange",
            value: function onBoardDescriptionChange(boardName, boardDescription) {
                this.postVs.filter(function (postV) {
                    return postV.post.thread.board.name === boardName;
                }).forEach(function (postV) {
                    postV.changeBoardDescription(boardDescription);
                });
            }

            /**
             * Reloads current posts, i.e. recreates post views.
             * This is needed after, for example, some user setting has been changed.
             */

        }, {
            key: "reloadPosts",
            value: function reloadPosts() {
                var scrollTop = this.element.scrollTop;
                this._postsContainerE.empty();

                this._loadPosts();
                this.element.scrollTop = scrollTop;
            }
        }, {
            key: "_loadPosts",
            value: function _loadPosts() {
                var _this = this;

                this.postVs = this._posts.map(function (post) {
                    return new SearchResultPostD(new PostV(post));
                });

                this.postVs.forEach(function (postV) {
                    _this._postsContainerE.appendChild(new SearchResultSeparatorV().element);
                    _this._postsContainerE.appendChild(postV.element);
                });
            }
        }]);

        return SearchResultV;
    }();

    var _createClass$T = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$1s(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var SearchV = function () {
        function SearchV(searchArgs, searchResultPage) {
            _classCallCheck$1s(this, SearchV);

            var searchE = document.createElement('div');
            this.element = searchE;
            searchE.searchV = this;
            searchE.classList.add("search");

            var searchTitleE = document.createElement('h3');
            searchTitleE.classList.add('search_title');
            searchTitleE.appendText('Search');
            searchE.appendChild(searchTitleE);

            var searchFormV = new SearchFormV(searchArgs);
            searchE.appendChild(searchFormV.element);

            var searchResultV = new SearchResultV(searchResultPage);
            this._searchResultV = searchResultV;
            searchE.appendChild(searchResultV.element);
        }

        _createClass$T(SearchV, [{
            key: "onSettingsChange",
            value: function onSettingsChange(oldSettings, settings) {
                var _this = this;

                return (oldSettings.isKatexEnabled !== settings.isKatexEnabled ? enableKatex(settings.isKatexEnabled) : Promise.resolve()).then(function () {
                    return oldSettings.highlightCode !== settings.highlightCode ? enableHljs(settings.highlightCode).then() : null;
                }).then(function () {
                    _this._searchResultV.reloadPosts();

                    if (settings.highlightCode) highlightCode(_this.element);
                });
            }
        }, {
            key: "onBoardDescriptionChange",
            value: function onBoardDescriptionChange(boardName, boardDescription) {
                this._searchResultV.onBoardDescriptionChange(boardName, boardDescription);
            }
        }]);

        return SearchV;
    }();

    var _createClass$U = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$1t(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var SearchWithStateD = function () {
        function SearchWithStateD(searchV) {
            _classCallCheck$1t(this, SearchWithStateD);

            decorateView(searchV, this, 'searchV');
        }

        _createClass$U(SearchWithStateD, [{
            key: "getState",
            value: function getState() {
                // it's empty because in the current implementation the state can be fully restored from url
                return null;
            }
        }, {
            key: "restoreState",
            value: function restoreState(searchState) {
                // it's empty because in the current implementation the state can be fully restored from url
                return null;
            }
        }]);

        return SearchWithStateD;
    }();

    var _createClass$V = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$1u(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ContainerV = function () {
        function ContainerV() {
            var _this = this;

            _classCallCheck$1u(this, ContainerV);

            var containerE = $("#container");
            this.element = containerE;
            containerE.containerV = this;

            this._locationState = new LocationState();
            this._currentV = null; // the loaded view at the moment

            if (getSelectedThreadId()) {
                this._loadThread(getSelectedThreadId(), getPage(), getPostIdAnchor(), false);
            } else if (isSearchPage()) {
                this._loadSearch(getSearchArgs(), false);
            } else {
                this._loadWarp(getSelectedBoardName(), getPage(), false);
            }

            document.addEventListener('selectBoard', function (e) {
                return _this._onSelectBoard(e);
            }, false);
            document.addEventListener('selectThread', function (e) {
                return _this._onSelectThread(e);
            }, false);
            document.addEventListener('selectGlobalPage', function (e) {
                return _this._onSelectGlobalPage(e);
            }, false);
            document.addEventListener('selectSearch', function (e) {
                return _this._onSelectSearch(e);
            }, false);
            document.addEventListener('setPostIdHash', function (e) {
                return _this._onSetPostIdHash(e);
            }, false);
            document.addEventListener('authorizeChange', function (e) {
                return _this._onAuthorizeChange(e);
            }, false);

            // listen to settings changes
            // NOTE: we listen here, because ContainerV is, unlike its children, created only once
            addSettingsChangeEventListener(this._onSettingsChangeHandler = function (e) {
                _this._onSettingsChange(e.oldSettings, e.newSettings);
            });

            new ContainerPostsListener(function () {
                return _this._currentV;
            });
            new ContainerBoardDescriptionListener(function () {
                return _this._currentV;
            });
            new ContainerHiddenByOpListener(function () {
                return _this._currentV;
            });
        }

        _createClass$V(ContainerV, [{
            key: "_onSelectBoard",
            value: function _onSelectBoard(e) {
                var boardName = e.detail.boardName;
                var pageNumber = e.detail.pageNumber;
                var isPopState = e.detail.isPopState;

                this._loadWarp(boardName, pageNumber, isPopState);
            }
        }, {
            key: "_onSelectThread",
            value: function _onSelectThread(e) {
                var threadId = e.detail.threadId;
                var pageNumber = e.detail.pageNumber;
                var postId = e.detail.postId;
                var isPopState = e.detail.isPopState;

                this._loadThread(threadId, pageNumber, postId, isPopState);
            }
        }, {
            key: "_onSelectGlobalPage",
            value: function _onSelectGlobalPage(e) {
                var pageNumber = e.detail.pageNumber;
                var isPopState = e.detail.isPopState;

                this._loadWarp(null, pageNumber, isPopState);
            }
        }, {
            key: "_onSelectSearch",
            value: function _onSelectSearch(e) {
                var searchArgs = e.detail.searchArgs;
                var isPopState = e.detail.isPopState;

                this._loadSearch(searchArgs, isPopState);
            }
        }, {
            key: "_loadWarp",
            value: function _loadWarp(boardName, pageNumber, isPopState) {
                var _this2 = this;

                getWarpPage$1(boardName, pageNumber).then(function (warpPage) {
                    var warpPageV = new WarpPageWithStateD(new WarpPageV(warpPage));
                    _this2._loadContent(warpPageV, null, isPopState);
                });
            }
        }, {
            key: "_loadThread",
            value: function _loadThread(threadId, pageNumber, postId, isPopState) {
                var _this3 = this;

                getThreadPage$1(threadId, pageNumber).then(function (threadPage) {
                    var threadPageV = new ThreadPageWithStateD(new ThreadPageV(threadPage));
                    _this3._loadContent(threadPageV, postId, isPopState);
                }).catch(function (e) {
                    showNotification(config.notification.type.error, e.message);
                });
            }
        }, {
            key: "_loadSearch",
            value: function _loadSearch(searchArgs, isPopState) {
                var _this4 = this;

                getSearch$1(searchArgs).then(function (searchResultPage) {
                    var searchV = new SearchWithStateD(new SearchV(searchArgs, searchResultPage));
                    _this4._loadContent(searchV, null, isPopState);
                });
            }
        }, {
            key: "_loadContent",
            value: function _loadContent(view, postId, isPopState) {
                if (this._locationState.isLocationSet()) {
                    // save the state
                    this._locationState.setState(this._getState());
                }
                // update the previous location href to the current location href
                this._locationState.updateLocation();

                this.element.replaceChildrenWith(view.element);
                this._currentV = view;

                highlightCode(this.element);

                if (postId) activateTarget();

                if (isPopState) {
                    this._restoreState(this._locationState.getState());
                    return;
                }

                if (!postId) this.element.scrollToTop();
            }
        }, {
            key: "_onSetPostIdHash",
            value: function _onSetPostIdHash(e) {
                // save the scroll position only (because the content is not changed.)
                this._locationState.setState({
                    scrollTop: this.element.scrollTop
                });
                this._locationState.updateLocation();
            }
        }, {
            key: "_onAuthorizeChange",
            value: function _onAuthorizeChange(e) {
                $a(".new_thread_form").forEach(function (e) {
                    return e.newThreadFormV.onAuthorizeChange();
                });
                $a(".reply_form").forEach(function (e) {
                    return e.replyFormV.onAuthorizeChange();
                });
            }
        }, {
            key: "_onSettingsChange",
            value: function _onSettingsChange(oldSettings, settings) {
                var _this5 = this;

                var state = this._getState();

                return this._currentV.onSettingsChange(oldSettings, settings).then(function () {
                    _this5._restoreState(state);
                });
            }
        }, {
            key: "_getState",
            value: function _getState() {
                return new ContainerState(this.element.scrollTop, this._currentV.getState());
            }
        }, {
            key: "_restoreState",
            value: function _restoreState(state) {
                this._currentV.restoreState(state.viewState);
                this.element.scrollTop = state.scrollTop;
            }
        }]);

        return ContainerV;
    }();

    var ContainerState = function ContainerState(scrollTop, viewState) {
        _classCallCheck$1u(this, ContainerState);

        this.scrollTop = scrollTop;
        this.viewState = viewState;
    };

    var ContainerPostsListener = function () {
        function ContainerPostsListener(getCurrentView) {
            var _this6 = this;

            _classCallCheck$1u(this, ContainerPostsListener);

            this._getCurrentView = getCurrentView;

            addPostsListener(function (e, post) {
                return _this6._handlePost(post);
            });
        }

        _createClass$V(ContainerPostsListener, [{
            key: "_handlePost",
            value: function _handlePost(post) {
                if (post.ephemeral && getSettings().hideEphemeralPosts) return;

                var currentV = this._getCurrentView();
                if (currentV.onNewPost) currentV.onNewPost(post);
            }
        }]);

        return ContainerPostsListener;
    }();

    var ContainerBoardDescriptionListener = function () {
        function ContainerBoardDescriptionListener(getCurrentView) {
            var _this7 = this;

            _classCallCheck$1u(this, ContainerBoardDescriptionListener);

            this._getCurrentView = getCurrentView;

            addBoardDescriptionListener(function (e, board) {
                return _this7._onBoardDescriptionChange(board.name, board.description);
            });
        }

        _createClass$V(ContainerBoardDescriptionListener, [{
            key: "_onBoardDescriptionChange",
            value: function _onBoardDescriptionChange(boardName, boardDescription) {
                var currentV = this._getCurrentView();

                if (currentV.onBoardDescriptionChange) {
                    currentV.onBoardDescriptionChange(boardName, boardDescription);
                }
            }
        }]);

        return ContainerBoardDescriptionListener;
    }();

    var ContainerHiddenByOpListener = function () {
        function ContainerHiddenByOpListener(getCurrentView) {
            var _this8 = this;

            _classCallCheck$1u(this, ContainerHiddenByOpListener);

            this._getCurrentView = getCurrentView;

            addHiddenByOpListener(function (e, hiddenByOpEvent) {
                return _this8._onUpdatedHiddenByOp(hiddenByOpEvent);
            });
        }

        _createClass$V(ContainerHiddenByOpListener, [{
            key: "_onUpdatedHiddenByOp",
            value: function _onUpdatedHiddenByOp(hiddenByOpEvent) {
                updateHiddenByOp$1(hiddenByOpEvent);

                var currentV = this._getCurrentView();
                if (currentV.onUpdatedHiddenByOp) currentV.onUpdatedHiddenByOp(hiddenByOpEvent);
            }
        }]);

        return ContainerHiddenByOpListener;
    }();

    var _createClass$W = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$1v(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ChatTextboxD = function () {
        function ChatTextboxD(textboxV) {
            var _this = this;

            _classCallCheck$1v(this, ChatTextboxD);

            decorateView(textboxV, this);
            this.element = textboxV.element;

            this._boardRe = new RegExp("^\\/(" + config.boardNamePattern + ") ");
            this._boardThreadRe = new RegExp("^\\/(" + config.boardNamePattern + ")\\/" + uuidPattern + " ");
            this._boardSubjectRe = new RegExp("^\\/(" + config.boardNamePattern + ")\\[(.+)\\] ");
            this._hideColorRe = new RegExp("^\\/hide\\s*@" + uuidPattern);
            this._unhideColorRe = new RegExp("^\\/unhide\\s*@" + uuidPattern);

            this.element.addEventListener("keydown", function (e) {
                if (isAltEnter(e)) {
                    _this.insertText("\n");
                } else if (isEnter(e)) {
                    e.preventDefault();
                    _this._onSubmit();
                }
            }, false);
        }

        _createClass$W(ChatTextboxD, [{
            key: "_onSubmit",
            value: function _onSubmit() {
                var text = this.element.value.trim();
                if (!text) return;

                this.clear();

                if (text === "/ep") {
                    this._toggleEphemeralPosting();
                    return;
                }

                if (text === "/cc") {
                    this._changeColors();
                    return;
                }

                if (text === "/help") {
                    showHelpPopup();
                    return;
                }

                if (text === "/settings") {
                    showSettingsPopup();
                    return;
                }

                if (text === "//") {
                    this._contextThreadId = null;
                    this._getChatBodyV().leaveThread();
                    return;
                }

                var hideColorMatch = this._hideColorRe.exec(text);
                if (hideColorMatch) {
                    var postId = hideColorMatch[1];
                    this._hideColor(postId);
                    return;
                }

                var unhideColorMatch = this._unhideColorRe.exec(text);
                if (unhideColorMatch) {
                    var _postId = unhideColorMatch[1];
                    this._unhideColor(_postId);
                    return;
                }

                var boardMatch = this._boardRe.exec(text);
                if (boardMatch) {
                    var boardName = boardMatch[1];
                    var boardDescription = text.substr(boardName.length + 2);

                    this._updateBoardDescription(boardName, boardDescription);
                    return;
                }

                var boardThreadMatch = this._boardThreadRe.exec(text);
                if (boardThreadMatch) {
                    var _boardName = boardThreadMatch[1];
                    var threadId = boardThreadMatch[2];

                    // tmp disabled todo
                    // this._contextThreadId = toHyphenatedUuid(threadId);
                    // this._getChatBodyV().openThreadInChat(this._contextThreadId);

                    this._postToThread(text.substr(("/" + _boardName + "/" + threadId + " ").length), threadId);
                    return;
                }

                var boardSubjectMatch = this._boardSubjectRe.exec(text);
                if (boardSubjectMatch) {
                    var _boardName2 = boardSubjectMatch[1];
                    var subject = boardSubjectMatch[2];

                    this._createThread(text.substr(("/" + _boardName2 + "[" + subject + "] ").length), _boardName2, subject);
                    return;
                }

                if (this._contextThreadId) {
                    this._postToThread(text, this._contextThreadId);
                    return;
                }

                this._postToChat(text);
            }
        }, {
            key: "_updateBoardDescription",
            value: function _updateBoardDescription(boardName, boardDescription) {
                if (boardName === config.chat.boardName) {
                    showNotification(config.notification.type.warning, "/cht/ description cannot be changed.");
                    return;
                }

                updateBoardDescription$1(new Board(boardName, boardDescription)).then(function (_) {
                    showNotification(config.notification.type.success, "The description was changed.");
                }).catch(function (e) {
                    showNotification(config.notification.type.error, "Failed to change the description.");
                });
            }
        }, {
            key: "_changeColors",
            value: function _changeColors() {
                updateClientId().then(function () {
                    showNotification(config.notification.type.success, "Your colors were changed.");
                }).catch(function (e) {
                    showNotification(config.notification.type.error, "Failed to change your colors.");
                });
            }
        }, {
            key: "_hideColor",
            value: function _hideColor(postId) {
                getPost$1(postId).then(function (post) {
                    var settings = getSettings();
                    settings.colorsFilters = settings.colorsFilters || "";

                    var filters = parseColorsFilters(settings.colorsFilters);

                    if (!filters.some(function (_ref) {
                        var bg = _ref.bg,
                            fg = _ref.fg;
                        return post.bg === bg && post.fg === fg;
                    })) {
                        filters.push({ bg: post.bg, fg: post.fg });
                    }

                    settings.colorsFilters = filters.map(function (_ref2) {
                        var bg = _ref2.bg,
                            fg = _ref2.fg;
                        return bg + ":" + fg;
                    }).join('\n');
                    saveSettings(settings);
                }).catch(function (errorResponse) {
                    showNotification(config.notification.type.error, errorResponse.message);
                });
            }
        }, {
            key: "_unhideColor",
            value: function _unhideColor(postId) {
                getPost$1(postId).then(function (post) {
                    var settings = getSettings();
                    var filter = post.bg + ":" + post.fg + "\n";

                    settings.colorsFilters = settings.colorsFilters || "";

                    var filters = parseColorsFilters(settings.colorsFilters);
                    settings.colorsFilters = filters.filter(function (_ref3) {
                        var bg = _ref3.bg,
                            fg = _ref3.fg;
                        return !(bg === post.bg && fg === post.fg);
                    }).map(function (_ref4) {
                        var bg = _ref4.bg,
                            fg = _ref4.fg;
                        return bg + ":" + fg + "\n";
                    }).join('');

                    saveSettings(settings);
                }).catch(function (errorResponse) {
                    showNotification(config.notification.type.error, errorResponse.message);
                });
            }
        }, {
            key: "_toggleEphemeralPosting",
            value: function _toggleEphemeralPosting() {
                var settings = getSettings();
                settings.isEphemeralPostingOn = !settings.isEphemeralPostingOn;
                saveSettings(settings);
            }
        }, {
            key: "_postToChat",
            value: function _postToChat(text) {
                var _this2 = this;

                var post = new Post(config.pendingPostId, new Date(), 0, 0, text, false, config.chat.threadId);

                var chatContentV = this._getChatContentV();
                // todo: finish the idea of pending posts: chatContentV contains PendingPostsContainerV
                // which contains PendingPostVs

                var ephemeral = isEphemeralPosting(text);

                publish$1(new NewPost(null, text, config.chat.threadId, ephemeral)).then(function (post) {
                    // todo: remove pending post
                }).catch(function (e) {
                    return _this2._onPublishError(e);
                });
            }
        }, {
            key: "_postToThread",
            value: function _postToThread(text, threadId) {
                var _this3 = this;

                var ephemeral = isEphemeralPosting(text);

                publish$1(new NewPost(null, text, threadId, ephemeral)).catch(function (e) {
                    return _this3._onPublishError(e);
                });
            }
        }, {
            key: "_createThread",
            value: function _createThread(text, boardName, subject) {
                var _this4 = this;

                var ephemeral = isEphemeralPosting(text);

                publish$1(new NewPost(null, text, null, ephemeral, subject, boardName)).catch(function (e) {
                    return _this4._onPublishError(e);
                });
            }
        }, {
            key: "_getChatBodyV",
            value: function _getChatBodyV() {
                return $('#chat_body').chatBodyV;
            }
        }, {
            key: "_getChatContentV",
            value: function _getChatContentV() {
                return $('#chat_content').chatContentV;
            }
        }, {
            key: "_onPublishError",
            value: function _onPublishError(error) {
                if (error.errorCode === 'E0401') {
                    showNotification(config.notification.type.error, error.message + ". Please log in.");
                    return;
                }

                showNotification(config.notification.type.error, "Message failed to send.");
            }
        }]);

        return ChatTextboxD;
    }();

    function _classCallCheck$1w(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ChatFormV = function ChatFormV(keepScrollBottomV) {
        _classCallCheck$1w(this, ChatFormV);

        var chatFormE = document.createElement('form');
        chatFormE.id = 'chat_form';

        this.element = chatFormE;
        chatFormE.chatFormV = this;

        this.tbxV = new TextboxV(function () {
            return keepScrollBottomV.keepScrollBottom();
        });
        this.tbxV = new ChatTextboxD(this.tbxV);

        this.tbxV.autoResize();
        var tbxE = this.tbxV.element;
        tbxE.id = "chat_tbx";
        tbxE.classList.add('chat_tbx');
        tbxE.maxLength = config.maxPostLength;
        tbxE.title = "Press Alt+Enter to start a new line";
        tbxE.autofocus = true;

        chatFormE.appendChild(tbxE);

        function onAuthorizeChange() {
            var isAuthorized = getAccessToken();
            // tbxE.disabled = !isAuthorized;
            tbxE.placeholder = isAuthorized ? "" : config.ephemeralModeWarning;
        }
        onAuthorizeChange();
        document.addEventListener('authorizeChange', function (e) {
            return onAuthorizeChange();
        }, false);

        function onEphemeralPostingModeChange() {
            tbxE.placeholder = isEphemeralPostingOn() ? config.ephemeralModeWarning : "";
        }
        onEphemeralPostingModeChange();
        addSettingsChangeEventListener(function (e) {
            if (e.oldSettings.isEphemeralPostingOn !== e.newSettings.isEphemeralPostingOn) {
                onEphemeralPostingModeChange();
            }
        });
    };

    var _createClass$X = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$1x(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    // file: keep_scroll_bottom_v.js
    var KeepScrollBottomV = function () {
        function KeepScrollBottomV(element, threshold) {
            var _this = this;

            _classCallCheck$1x(this, KeepScrollBottomV);

            this.element = element;
            this._threshold = threshold;

            this._clientWidth = element.clientWidth;
            this._clientHeight = element.clientHeight;

            this.element.scrollToBottom();
            this._wasScrolledToBottom = true;

            this._prevScrollTop = this.element.scrollTop;
            element.addEventListener("scroll", function (e) {
                return _this._keepTrackOfScrollStatus();
            }, false);

            this._running = false;
            window.addEventListener("resize", function (e) {
                return _this._keepScrollBottomOnResize();
            }, false);
            window.addEventListener("mouseup", function (e) {
                return _this._onMouseUp(e);
            }, false);
            window.addEventListener("mousedown", function (e) {
                return _this._onMouseDown(e);
            }, false);

            document.addEventListener('lsChange', this._onLsChangeHandler = function (e) {
                return _this._onLsChange(e);
            }, false);
        }

        _createClass$X(KeepScrollBottomV, [{
            key: "keepScrollBottom",
            value: function keepScrollBottom() {
                if (this._wasScrolledToBottom) this.element.scrollToBottom();
            }
        }, {
            key: "_onMouseUp",
            value: function _onMouseUp(e) {
                var _this2 = this;

                this._isMouseDown = false;
                this._clientWidthOnMouseDown = null;
                this._clientHeightOnMouseDown = null;
                requestAnimationFrame(function () {
                    return _this2.keepScrollBottom();
                });
            }
        }, {
            key: "_onMouseDown",
            value: function _onMouseDown(e) {
                this._isMouseDown = true;
                this._clientWidthOnMouseDown = this.element.clientWidth;
                this._clientHeightOnMouseDown = this.element.clientHeight;
            }
        }, {
            key: "_onLsChange",
            value: function _onLsChange(e) {
                var _this3 = this;

                // e.g. width might shrink in another tab
                if (e.detail.isFromForeignWindow) requestAnimationFrame(function () {
                    return _this3.keepScrollBottom();
                });
            }

            // _forEachAnimationFrame() {
            //     if (this._stopWatching) return;
            //
            //     this.keepScrollBottom();
            //     requestAnimationFrame(this._forEachAnimationFrameBound);
            // }

        }, {
            key: "_keepTrackOfScrollStatus",
            value: function _keepTrackOfScrollStatus() {
                if (this._isMouseDown && (this._clientWidthOnMouseDown !== this.element.clientWidth || this._clientHeightOnMouseDown !== this.element.clientHeight)) {
                    // this is not the user scrolling, he's resizing
                    return;
                }

                // workaround: for hidden elements, setting scrollTop doesn't do anything
                if (this._prevScrollTop === this.element.scrollTop) {
                    // if we're here, scroll actually hasn't changed, just the element has become visible
                    if (this._wasScrolledToBottom) {
                        // while the element was hidden, we called scrollToBottom,
                        // but it didn't do anything, but now we can actually change it
                        this.element.scrollToBottom();
                    }
                    return;
                }
                this._prevScrollTop = this.element.scrollTop;

                // keep track whether the element is scrolled to the bottom or not
                this._wasScrolledToBottom = this._isScrolledToBottom();
            }
        }, {
            key: "_keepScrollBottomOnResize",
            value: function _keepScrollBottomOnResize() {
                var _this4 = this;

                if (this._running) return;

                this._running = true;
                requestAnimationFrame(function () {
                    _this4.keepScrollBottom();
                    _this4._running = false;
                });
            }
        }, {
            key: "_isScrolledToBottom",
            value: function _isScrolledToBottom() {
                var el = this.element;
                return el.scrollHeight - el.offsetHeight - this._threshold <= el.scrollTop;
            }

            // _widthOrHeightShrunk() {
            //     const result = (this.element.clientWidth < this._clientWidth
            //         || this.element.clientHeight < this._clientHeight);
            //
            //     this._clientWidth = this.element.clientWidth;
            //     this._clientHeight = this.element.clientHeight;
            //
            //     return result;
            // }

        }]);

        return KeepScrollBottomV;
    }();

    var _createClass$Y = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$1y(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ChatPostBodyD = function () {
        function ChatPostBodyD(postBodyV) {
            _classCallCheck$1y(this, ChatPostBodyD);

            decorateView(postBodyV, this);

            var post = postBodyV.post;

            if (post.thread.board.name !== config.chat.boardName) {
                this._boardThreadLinksV = new BoardThreadLinksV(post);

                postBodyV.element.prependChild(this._boardThreadLinksV.element);
            }
        }

        _createClass$Y(ChatPostBodyD, [{
            key: "changeBoardDescription",
            value: function changeBoardDescription(boardDescription) {
                if (!this._boardThreadLinksV) return;

                this._boardThreadLinksV.changeBoardDescription(boardDescription);
            }
        }]);

        return ChatPostBodyD;
    }();

    var _createClass$Z = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$1z(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ChatPostV = function () {
        function ChatPostV(post) {
            var _this = this;

            _classCallCheck$1z(this, ChatPostV);

            this.post = post;

            var postE = document.createElement('div');
            postE.classList.add('post');

            this.element = postE;
            postE.chatPostV = this;

            var infoE = document.createElement('div');
            this.infoE = infoE;
            infoE.classList.add('info');
            postE.appendChild(infoE);

            var infoTimeE = document.createElement('div');
            infoTimeE.classList.add('info_time');
            infoE.appendChild(infoTimeE);

            var infoIdE = document.createElement('div');
            infoIdE.classList.add('info_id');
            infoE.appendChild(infoIdE);

            var infoNameE = document.createElement('div');
            infoNameE.classList.add('info_name');
            infoE.appendChild(infoNameE);

            var idE = document.createElement('span');
            idE.classList.add('id');
            idE.style.color = post.fg.toHSL();
            if (post.ephemeral) {
                idE.style.backgroundImage = "repeating-linear-gradient(\n            -45deg, transparent, transparent 1px, " + post.bg.toHSL() + " 2px, " + post.bg.toHSL() + " 10px)";
            } else {
                idE.style.backgroundColor = post.bg.toHSL();
            }
            idE.title = PostV.formatDate(post.postDate);
            idE.appendText(toShortcutUuid(post.postId.toString()));

            // const timeE = document.createElement('span');
            // timeE.classList.add('date');
            // timeE.appendText(ChatPostV.formatTime(post.postDate));

            var nameE = document.createElement('span');
            nameE.classList.add('name');
            if (post.user.banned) nameE.classList.add('banned');
            nameE.appendText(post.user.nickname);

            // infoTimeE.appendChild(timeE);
            infoNameE.appendChild(nameE);
            infoIdE.appendChild(idE);

            idE.addEventListener('click', function (e) {
                return _this._reply();
            }, false);

            if (this._isFromOtherBoard()) {
                postE.classList.add('from_other_board');
            }

            var isPostHidden = this.post.hiddenByOp || getSettings().hideEphemeralPosts && this.post.ephemeral || this._isFromOtherBoard() && getSettings().showOnlyChatPostsInChat || getSettings().ignoreCyrillicShit && containCyrillic(post.text) || getSettings().maxEntropy > 0 && getShannonEntropy(post.text) > getSettings().maxEntropy || parseRegexpFilters(getSettings().regexpFilters).some(function (re) {
                return re.test(post.text);
            }) || parseColorsFilters(getSettings().colorsFilters).some(function (_ref) {
                var bg = _ref.bg,
                    fg = _ref.fg;
                return post.bg === bg && post.fg === fg;
            });

            if (isPostHidden) {
                this._hide();
                // avoid creating postBodyV in order to not waste time parsing
            } else {
                this._createPostBody();
            }
        }

        _createClass$Z(ChatPostV, [{
            key: "changeBoardDescription",
            value: function changeBoardDescription(boardDescription) {
                this.post.thread.board.description = boardDescription;

                if (this.postBodyV) this.postBodyV.changeBoardDescription(boardDescription);
            }
        }, {
            key: "_hide",
            value: function _hide() {
                var _this2 = this;

                if (!getSettings().displayHiddenPostsInChat) {
                    this.element.classList.add('hidden');
                }
                this.element.classList.add('hidden_post');

                var buttonsContainerE = document.createElement('div');
                buttonsContainerE.classList.add('buttons_container');
                this.element.appendChild(buttonsContainerE);

                var unhideBtnE = document.createElement('i');
                unhideBtnE.classList.add('unhide_post_btn');
                unhideBtnE.classList.add('icon-eye');
                unhideBtnE.title = "Unhide: hidden by " + (this.post.hiddenByOp ? "OP" : "you");
                buttonsContainerE.appendChild(unhideBtnE);

                var hideBtnE = document.createElement('i');
                hideBtnE.classList.add('hide_post_btn');
                hideBtnE.classList.add('icon-eye-blocked');
                hideBtnE.title = 'Hide (only for you)';
                hideBtnE.hide();
                buttonsContainerE.appendChild(hideBtnE);

                unhideBtnE.addEventListener('click', function (e) {
                    _this2._createPostBody();
                    _this2.element.classList.remove('hidden');
                    _this2.element.classList.remove('hidden_post');

                    hideBtnE.show();
                    unhideBtnE.hide();
                }, false);

                hideBtnE.addEventListener('click', function (e) {
                    if (!getSettings().displayHiddenPostsInChat) {
                        _this2.element.classList.add('hidden');
                    }
                    _this2.element.classList.add('hidden_post');

                    unhideBtnE.show();
                    hideBtnE.hide();
                }, false);
            }
        }, {
            key: "_createPostBody",
            value: function _createPostBody() {
                if (this.postBodyV) return;

                this.postBodyV = new ChatPostBodyD(new PostBodyV(this.post));
                this.postBodyV.element.appendAfter(this.infoE);
                // this.element.appendChild(this.postBodyV.element);
            }
        }, {
            key: "_isFromOtherBoard",
            value: function _isFromOtherBoard() {
                return this.post.thread.board.name !== config.chat.boardName;
            }
        }, {
            key: "_reply",
            value: function _reply() {
                var chatFormV = $('#chat_form').chatFormV;
                var isAtBeginning = chatFormV.tbxV.getTextBeforeSelectionEnd().trim().length === 0;

                var postRef = "@" + toSimpleUuid(this.post.postId) + " ";
                var boardName = this.post.thread.board.name;
                var threadId = toSimpleUuid(this.post.thread.threadId);

                var text = boardName !== config.chat.boardName && isAtBeginning ? "/" + boardName + "/" + threadId + " " + postRef : postRef;

                chatFormV.tbxV.insertText(text);
            }
        }], [{
            key: "formatTime",
            value: function formatTime(postDate) {
                return postDate.toISOString().slice(0, 19).split('T')[1];
            }
        }]);

        return ChatPostV;
    }();

    var _createClass$_ = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$1A(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    // file: days_separator_v.js
    var DaysSeparatorV = function () {
        function DaysSeparatorV(date) {
            _classCallCheck$1A(this, DaysSeparatorV);

            var daysSeparatorE = document.createElement("div");
            daysSeparatorE.classList.add("days_separator");

            this.element = daysSeparatorE;
            daysSeparatorE.daysSeparatorV = this;

            var dateE = document.createElement("span");
            dateE.classList.add("date");
            dateE.appendText(this._formatDate(date));

            daysSeparatorE.appendChild(dateE);
        }

        _createClass$_(DaysSeparatorV, [{
            key: "_formatDate",
            value: function _formatDate(date) {
                var monthNames = ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"];

                return date.getDate() + " " + monthNames[date.getMonth()] + " " + date.getFullYear();
            }
        }]);

        return DaysSeparatorV;
    }();

    var _createClass$$ = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$1B(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ChatContentV = function () {
        function ChatContentV() {
            _classCallCheck$1B(this, ChatContentV);

            var contentE = document.createElement('div');
            contentE.id = 'chat_content';

            this.element = contentE;
            contentE.chatContentV = this;

            contentE.classList.add("content");

            this.chatPosts = new ChatPosts();

            this.keepScrollBottomV = new KeepScrollBottomV(contentE, config.chat.keepScrollBottomThreshold);
        }

        _createClass$$(ChatContentV, [{
            key: "initEndlessScroll",
            value: function initEndlessScroll(afterPrependPosts) {
                var _this = this;

                this.element.addEventListener('scroll', function (e) {
                    if (_this.element.scrollTop > 0 || !_this.getFirstPostV()) return;

                    getPostsBefore$1(_this.getFirstNonEphemeralPostV().post.postId, _this._contextThreadId).then(function (posts) {
                        _this.prependPosts(posts);
                        afterPrependPosts();
                    });
                }, false);

                return this;
            }
        }, {
            key: "afterRecentPostsLoaded",
            value: function afterRecentPostsLoaded() {}
        }, {
            key: "getFirstPostV",
            value: function getFirstPostV() {
                return this._getPostsVs()[0];
            }
        }, {
            key: "getFirstNonEphemeralPostV",
            value: function getFirstNonEphemeralPostV() {
                return this._getPostsVs().filter(function (chatPostV) {
                    return !chatPostV.post.ephemeral;
                })[0];
            }
        }, {
            key: "getLastPostV",
            value: function getLastPostV() {
                return this._getPostsVs().last();
            }
        }, {
            key: "getLastNonEphemeralPostV",
            value: function getLastNonEphemeralPostV() {
                return this._getPostsVs().filter(function (chatPostV) {
                    return !chatPostV.post.ephemeral;
                }).last();
            }
        }, {
            key: "prependPosts",
            value: function prependPosts(posts) {
                var _this2 = this;

                var prevScrollHeight = this.element.scrollHeight;

                this.chatPosts.addPosts(posts).filter(function (post) {
                    return !getSettings().hideEphemeralPosts || !post.ephemeral;
                }).reverse().map(function (post) {
                    return new ChatPostV(post);
                }).reduce(function (nextV, postV) {
                    return _this2.prependPostV(nextV, postV);
                }, this.getFirstPostV());

                this.element.scrollTop += this.element.scrollHeight - prevScrollHeight;
            }
        }, {
            key: "appendPosts",
            value: function appendPosts(posts) {
                var _this3 = this;

                this.chatPosts.addPosts(posts).filter(function (post) {
                    return !getSettings().hideEphemeralPosts || !post.ephemeral;
                }).map(function (post) {
                    return new ChatPostV(post);
                }).reduce(function (prevV, postV) {
                    return _this3.appendPostV(prevV, postV);
                }, this.getLastPostV());

                this.keepScrollBottomV.keepScrollBottom();
            }
        }, {
            key: "prependPostV",
            value: function prependPostV(nextV, postV) {
                this.prependBetweenPosts(postV, nextV);
                this.element.prependChild(postV.element); // NOTE: it's ok if postV.element = null

                return postV;
            }
        }, {
            key: "appendPostV",
            value: function appendPostV(prevV, postV) {
                this.appendBetweenPosts(prevV, postV);
                this.element.appendChild(postV.element);

                return postV;
            }
        }, {
            key: "prependBetweenPosts",
            value: function prependBetweenPosts(postV, nextV) {
                if (isDifferentDay(postV, nextV)) {
                    var daysSeparatorV = new DaysSeparatorV(postV.post.postDate);
                    this.element.prependChild(daysSeparatorV.element);
                }
            }
        }, {
            key: "appendBetweenPosts",
            value: function appendBetweenPosts(prevV, postV) {
                if (isDifferentDay(prevV, postV)) {
                    var daysSeparatorV = new DaysSeparatorV(postV.post.postDate);
                    this.element.appendChild(daysSeparatorV.element);
                }
            }

            /**
             * Clears the chat content, i.e. removes all posts and day separators.
             */

        }, {
            key: "clear",
            value: function clear() {
                this.chatPosts.clear();
                this.element.empty();
            }

            /**
             * Gets the loaded posts in the order they are shown.
             */

        }, {
            key: "getPosts",
            value: function getPosts() {
                return this._getPostsVs().map(function (chatPostV) {
                    return chatPostV.post;
                });
            }

            /**
             * Reloads current posts, i.e. recreates chat post views.
             * This is needed after, for example, some user setting has been changed.
             */

        }, {
            key: "reloadPosts",
            value: function reloadPosts() {
                var posts = this.getPosts();
                var maxScrollTop = this.element.scrollHeight - this.element.offsetHeight;
                var scrollTop = this.element.scrollTop;
                var scrolledFromBottom = maxScrollTop - scrollTop;

                this.clear();
                this.appendPosts(posts);
                var newMaxScrollTop = this.element.scrollHeight - this.element.offsetHeight;
                this.element.scrollTop = newMaxScrollTop - scrolledFromBottom;
            }
        }, {
            key: "onBoardDescriptionChange",
            value: function onBoardDescriptionChange(boardName, boardDescription) {
                this._getPostsVs().filter(function (chatPostV) {
                    return chatPostV.post.thread.board.name === boardName;
                }).forEach(function (chatPostV) {
                    return chatPostV.changeBoardDescription(boardDescription);
                });
            }
        }, {
            key: "onUpdatedHiddenByOp",
            value: function onUpdatedHiddenByOp(hiddenByOpEvent) {
                this.getPosts().filter(function (post) {
                    return post.postId === hiddenByOpEvent.postId;
                }).forEach(function (post) {
                    return post.hiddenByOp = hiddenByOpEvent.hiddenByOp;
                });

                this.reloadPosts();
            }
        }, {
            key: "setContextThreadId",
            value: function setContextThreadId(threadId) {
                this._contextThreadId = threadId;
            }
        }, {
            key: "_getPostsVs",
            value: function _getPostsVs() {
                return this.element.children.filter(function (element) {
                    return element.chatPostV;
                }).map(function (postE) {
                    return postE.chatPostV;
                });
            }
        }]);

        return ChatContentV;
    }();

    function isDifferentDay(postV1, postV2) {
        if (!postV1 || !postV2) return false;

        var dateX = postV1.post.postDate.toISOString().substr(0, 10);
        var dateY = postV2.post.postDate.toISOString().substr(0, 10);

        return dateX < dateY;
    }

    var ChatPosts = function () {
        function ChatPosts() {
            _classCallCheck$1B(this, ChatPosts);

            // loaded posts. Key: postId, value: post.
            this._posts = {};
        }

        /**
         * Adds new (not loaded yet) posts to the loaded ones.
         * This function must guarantee that no post will be loaded more than once.
         * @param {Post[]} posts
         * @returns {Post[]} Added posts.
         */


        _createClass$$(ChatPosts, [{
            key: "addPosts",
            value: function addPosts(posts) {
                return this._addLoadedPosts(this._skipLoadedPosts(posts));
            }
        }, {
            key: "clear",
            value: function clear(posts) {
                this._posts = {};
            }
        }, {
            key: "_skipLoadedPosts",
            value: function _skipLoadedPosts(posts) {
                var _this4 = this;

                return posts.filter(function (post) {
                    return !(post.postId in _this4._posts);
                });
            }
        }, {
            key: "_addLoadedPosts",
            value: function _addLoadedPosts(posts) {
                var _this5 = this;

                return posts.map(function (post) {
                    return _this5._posts[post.postId] = post;
                });
            }
        }]);

        return ChatPosts;
    }();

    function _classCallCheck$1C(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    // file: new_posts_separator_v.js
    var NewPostsSeparatorV = function NewPostsSeparatorV() {
        _classCallCheck$1C(this, NewPostsSeparatorV);

        var separatorE = document.createElement("div");
        separatorE.classList.add("new_posts_separator");

        this.element = separatorE;
        separatorE.newPostsSeparatorV = this;

        var textE = document.createElement("span");
        textE.classList.add("title");
        textE.appendText("new posts");

        separatorE.appendChild(textE);
    };

    var _createClass$10 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$1D(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    /**
     * A chat content decorator that tracks which posts are new.
     */
    var ChatContentTrackNewD = function () {
        function ChatContentTrackNewD(chatContentV, threadPromise) {
            var _this = this;

            _classCallCheck$1D(this, ChatContentTrackNewD);

            decorateView(chatContentV, this, 'chatContentV');

            this.chatContentV = chatContentV;
            this.element = chatContentV.element;

            this._newPostsSeparatorV = new NewPostsSeparatorV();

            this._lastSeenAt = null;
            this._storedLastSeenAt = null; // the value in the storage

            threadPromise.then(function (thread) {
                _this._lastSeenAt = thread.lastSeenAt;
                _this._storedLastSeenAt = thread.lastSeenAt;

                document.addEventListener('chatVisibilityChange', function (e) {
                    return _this._onChatVisibilityChange(e);
                }, false);
            });

            this.appendPosts = this.chatContentV.appendPosts.bind(this);
            this.prependPosts = this.chatContentV.prependPosts.bind(this);
            this.prependPostV = this.chatContentV.prependPostV.bind(this);
            this.reloadPosts = this.chatContentV.reloadPosts.bind(this);

            window.addEventListener('keydown', function (e) {
                return setTitlePrefix("");
            }, false);
            window.addEventListener('mousedown', function (e) {
                return setTitlePrefix("");
            }, false);
        }

        _createClass$10(ChatContentTrackNewD, [{
            key: "appendPostV",
            value: function appendPostV(prevV, postV) {
                if (getSettings().trackNewPosts && isChatOrDocumentHidden() && postV && postV.post.postDate > this._lastSeenAt) {
                    setTitlePrefix("[new posts]");
                }
                return this.chatContentV.appendPostV.call(this, prevV, postV);
            }
        }, {
            key: "getLastPostV",
            value: function getLastPostV() {
                return this.chatContentV.getLastPostV();
            }
        }, {
            key: "getLastNonEphemeralPostV",
            value: function getLastNonEphemeralPostV() {
                return this.chatContentV.getLastNonEphemeralPostV();
            }
        }, {
            key: "afterRecentPostsLoaded",
            value: function afterRecentPostsLoaded() {
                this.chatContentV.afterRecentPostsLoaded();
                this._updateLastSeenAtRepeatedly();
            }
        }, {
            key: "prependBetweenPosts",
            value: function prependBetweenPosts(postV, nextV) {
                if (this._isNewPostsSeparatorBetween(postV, nextV)) {
                    this._newPostsSeparatorV.element.detach();
                    this.element.prependChild(this._newPostsSeparatorV.element);
                }

                this.chatContentV.prependBetweenPosts(postV, nextV);
            }
        }, {
            key: "appendBetweenPosts",
            value: function appendBetweenPosts(prevV, postV) {
                this.chatContentV.appendBetweenPosts(prevV, postV);

                if (this._isNewPostsSeparatorBetween(prevV, postV)) {
                    this._newPostsSeparatorV.element.detach();
                    this.element.appendChild(this._newPostsSeparatorV.element);
                }
            }
        }, {
            key: "disableTracking",
            value: function disableTracking() {
                this._lastSeenAt = null;
                this._storedLastSeenAt = null;
                this._newPostsSeparatorV.element.detach();
            }
        }, {
            key: "_updateLastSeenAt",
            value: function _updateLastSeenAt() {
                var _this2 = this;

                if (!getSettings().trackNewPosts) return;

                // do not do anything if chat is hidden
                if (isChatOrDocumentHidden()) return;

                // because when chat is visible
                // we want to prevent moving the separator lower
                // and also we want to prevent showing the separator if it's not shown now
                this._lastSeenAt = null;

                var lastPostV = this.getLastPostV();

                if (lastPostV && (!this._storedLastSeenAt || this._storedLastSeenAt < lastPostV.post.postDate)) {
                    // make a request to store lastSeenAt as lastPostV.post.postDate
                    updateLastSeenAt$1(config.chat.threadId, lastPostV.post.postDate).then(function () {
                        // success => change the local variable
                        _this2._storedLastSeenAt = lastPostV.post.postDate;
                    });
                }
            }
        }, {
            key: "_updateLastSeenAtRepeatedly",
            value: function _updateLastSeenAtRepeatedly() {
                var _this3 = this;

                this._updateLastSeenAt();

                setTimeout(function () {
                    return _this3._updateLastSeenAtRepeatedly();
                }, config.chat.updateLastSeenAtTimeout);
            }
        }, {
            key: "_isNewPostsSeparatorBetween",
            value: function _isNewPostsSeparatorBetween(postV1, postV2) {
                if (!getSettings().trackNewPosts) return false;

                if (!postV1 || !postV2 || !this._lastSeenAt) return false;

                return postV1.post.postDate <= this._lastSeenAt && this._lastSeenAt < postV2.post.postDate;
            }
        }, {
            key: "_onChatVisibilityChange",
            value: function _onChatVisibilityChange(e) {
                if (!getSettings().trackNewPosts) return;

                if (e.detail.isHidden) {
                    this._newPostsSeparatorV.element.detach();

                    var lastPostV = this.getLastPostV();
                    if (lastPostV) {
                        this._lastSeenAt = lastPostV.post.postDate;
                    }

                    setTitlePrefix("");
                }
            }
        }]);

        return ChatContentTrackNewD;
    }();

    var _createClass$11 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$1E(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ChatBodyV = function () {
        function ChatBodyV() {
            var _this = this;

            _classCallCheck$1E(this, ChatBodyV);

            var chatBodyE = $('#chat_body');

            this.element = chatBodyE;
            chatBodyE.chatBodyV = this;

            var recentPostsPromise = getRecentPosts$1();
            var threadPromise = recentPostsPromise.then(function (rp) {
                return rp.chatThread || {};
            });

            var chatContentV = new ChatContentTrackNewD(new ChatContentV().initEndlessScroll(function () {
                return highlightCode(_this.element);
            }), threadPromise);

            this._chatContentV = chatContentV;
            chatBodyE.appendChild(chatContentV.element);

            var chatFormV = new ChatFormV(chatContentV.keepScrollBottomV);
            this._chatFromV = chatFormV;
            chatBodyE.appendChild(chatFormV.element);

            this._gapPrevention = new GapPrevention();
            this._gapPrevention.start();

            addPostsListener(function (e, post) {
                if (_this._contextThreadId && post.thread.threadId !== _this._contextThreadId) {
                    return;
                }

                if (_this._gapPrevention.isInProgress) {
                    _this._gapPrevention.addPost(post);
                    return;
                }

                chatContentV.appendPosts([post]);
            });

            recentPostsPromise.then(function (rp) {
                return rp.posts;
            }).then(function (posts) {
                chatContentV.appendPosts(posts.concat(_this._gapPrevention.getPosts()));

                _this._gapPrevention.stop();
            }).then(function () {
                return _this._afterRecentPostsLoaded();
            });

            // listen to settings changes
            addSettingsChangeEventListener(this._onSettingsChangeHandler = function (e) {
                _this._onSettingsChange(e.oldSettings, e.newSettings);
            });

            addBoardDescriptionListener(function (_, board) {
                return _this._onBoardDescriptionChange(board);
            });
            addHiddenByOpListener(function (_, hiddenByOpEvent) {
                return _this._onUpdatedHiddenByOp(hiddenByOpEvent);
            });
        }

        _createClass$11(ChatBodyV, [{
            key: "openThreadInChat",
            value: function openThreadInChat(threadId) {
                if (threadId !== this._contextThreadId) {
                    this._setContextReloadChat(threadId);
                }
            }
        }, {
            key: "leaveThread",
            value: function leaveThread() {
                if (this._contextThreadId) {
                    this._setContextReloadChat(null);
                }
            }
        }, {
            key: "_setContextReloadChat",
            value: function _setContextReloadChat(threadId) {
                var _this2 = this;

                this._contextThreadId = threadId;
                this._chatContentV.setContextThreadId(this._contextThreadId);
                this._disableConnectedEventListener = true;
                this._chatContentV.clear();

                this._gapPrevention.start();

                getRecentPosts$1(this._contextThreadId).then(function (rp) {
                    return rp.posts;
                }).then(function (posts) {
                    _this2._chatContentV.appendPosts(posts.concat(_this2._gapPrevention.getPosts()));

                    _this2._gapPrevention.stop();
                    _this2._disableConnectedEventListener = false;
                }).then(function () {
                    _this2._chatContentV.afterRecentPostsLoaded();
                    highlightCode(_this2.element);
                });
            }
        }, {
            key: "_afterRecentPostsLoaded",
            value: function _afterRecentPostsLoaded() {
                var _this3 = this;

                // if EventSource fails, it reconnects and 'connected' event happens
                // we listen to this event and request posts after the last loaded one
                // and meanwhile we prevent gaps
                addConnectedEventListener(function () {
                    if (_this3._disableConnectedEventListener) return;

                    _this3._gapPrevention.start();

                    getPostsAfter$1(_this3._chatContentV.getLastNonEphemeralPostV().post.postId, _this3._contextThreadId).then(function (posts) {
                        _this3._chatContentV.appendPosts(posts.concat(_this3._gapPrevention.getPosts()));

                        _this3._gapPrevention.stop();
                        highlightCode(_this3.element);
                    });
                });

                this._chatContentV.afterRecentPostsLoaded();
                highlightCode(this.element);
            }
        }, {
            key: "_onSettingsChange",
            value: function _onSettingsChange(oldSettings, settings) {
                var _this4 = this;

                return (oldSettings.isKatexEnabled !== settings.isKatexEnabled ? enableKatex(settings.isKatexEnabled) : Promise.resolve()).then(function () {
                    return oldSettings.highlightCode !== settings.highlightCode ? enableHljs(settings.highlightCode).then() : null;
                }).then(function () {
                    _this4._chatContentV.reloadPosts();

                    if (settings.highlightCode) highlightCode(_this4.element);

                    if (oldSettings.trackNewPosts !== settings.trackNewPosts && !settings.trackNewPosts) {
                        _this4._chatContentV.disableTracking();
                    }

                    if (oldSettings.isCompactTheme !== settings.isCompactTheme) {
                        if (!isChatHidden()) {
                            _this4._chatFromV.tbxV.forceResize();
                        }
                    }
                });
            }
        }, {
            key: "_onBoardDescriptionChange",
            value: function _onBoardDescriptionChange(board) {
                this._chatContentV.onBoardDescriptionChange(board.name, board.description);
            }
        }, {
            key: "_onUpdatedHiddenByOp",
            value: function _onUpdatedHiddenByOp(hiddenByOpEvent) {
                this._chatContentV.onUpdatedHiddenByOp(hiddenByOpEvent);
            }
        }]);

        return ChatBodyV;
    }();

    var GapPrevention = function () {
        function GapPrevention() {
            _classCallCheck$1E(this, GapPrevention);

            // Posts awaiting to be added. These posts are from the event source only.
            this._posts = [];
            this._isInProgress = false;
        }

        _createClass$11(GapPrevention, [{
            key: "addPost",
            value: function addPost(post) {
                if (this._posts.some(function (p) {
                    return p.postId === post.postId;
                })) return;

                this._posts.push(post);
            }
        }, {
            key: "getPosts",
            value: function getPosts() {
                return this._posts.slice();
            }
        }, {
            key: "start",
            value: function start() {
                this._posts = [];
                this._isInProgress = true;
            }
        }, {
            key: "stop",
            value: function stop() {
                this._posts = [];
                this._isInProgress = false;
            }
        }, {
            key: "isInProgress",
            get: function get() {
                return this._isInProgress;
            }
        }]);

        return GapPrevention;
    }();

    var _createClass$12 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$1F(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ContentFrameV = function () {
        function ContentFrameV() {
            var _this = this;

            _classCallCheck$1F(this, ContentFrameV);

            var contentFrameE = $('#content_frame');

            this.element = contentFrameE;
            contentFrameE.contentFrameV = this;

            var containerV = new ContainerV();
            var chatBodyV = new ChatBodyV();

            addSettingsChangeEventListener(this._onSettingsChangeHandler = function (e) {
                _this._onSettingsChange(e.oldSettings, e.newSettings);
            });
        }

        _createClass$12(ContentFrameV, [{
            key: "_onSettingsChange",
            value: function _onSettingsChange(oldSettings, settings) {
                if (oldSettings.trackNewPosts !== settings.trackNewPosts) {
                    // we place it here and not in the chat and container to avoid calling it twice
                    if (settings.trackNewPosts) {
                        // for all threads sets lastSeenAt to the latest post date of the thread
                        resetAllLastSeenAt();
                    } else {
                        // for all threads sets lastSeenAt to null
                        deleteAllLastSeenAt$1();
                    }
                }
            }
        }]);

        return ContentFrameV;
    }();

    // themes_service.js

    function initThemesService() {
        enableCompactTheme(getSettings().isCompactTheme);

        addSettingsChangeEventListener(function (e) {
            enableCompactTheme(e.newSettings.isCompactTheme);
        });
    }

    function enableCompactTheme(isEnabled) {
        if (isEnabled) {
            document.body.classList.add('compact_theme');
        } else {
            document.body.classList.remove('compact_theme');
        }
    }

    // file: main.js

    document.addEventListener("DOMContentLoaded", function () {
        // todo: call just main when tor browser bug is fixed
        initLocalStorageService(main);
    });

    function main() {
        initAliveTabsService();
        initServerInfoService();
        subscribe();
        new MenuV();
        //lsService.addStorageEventListener(); // todo: uncomment when tor browser bug is fixed
        getOrCreateClientId();

        enableToggles();
        enableDrags();

        var postId = getPostId();
        (postId ? getPost$1(postId, true).then(function (post) {
            return replaceByThreadPage(post.thread.board.name, post.thread.threadId, getThreadPageNumber(post.index), postId);
        }).catch(function (e) {
            showNotification(config.notification.type.error, e.message);
            throw e;
        }) : Promise.resolve()).then(function () {
            new BoardsSidebarV();
            new ThreadsSidebarV();

            new ChatContainerV();

            Promise.all([enableKatex(getSettings().isKatexEnabled), enableHljs(getSettings().highlightCode)]).then(function () {
                new ContentFrameV();
            });
        });

        var oldUiLinkE = document.createElement('a');
        oldUiLinkE.href = '/ui2/';
        oldUiLinkE.appendText("The old ui is here: /ui2/");
        showNotification(config.notification.type.info, newE("div", { style: "max-width: 300px" }, "FYI: Ephemeral posting mode means that your posts don't get stored in the database,\n        they are only retransmitted to other online users.\n        It's the default posting mode for Anonymous.\n        Create an account if you want your posts to be saved.\n        More info can be found in the help."), 20000);

        addEventListener("popstate", function (e) {
            //console.log("location: " + document.location + ", state: ", e.state);

            handlePopState(e.state);
        }, false);

        initThemesService();

        initRemovePostPreviewsOnClick();
        {
            initFavoritesService();
        }
    }

}());
